!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine es_advect (ng)

! This routine computes the Eulerian advection tendencies for the
! concentrations of all species.

use hyp_coms
use mem_met
use mem_grid
use mem_euler
use mem_scratch

implicit none

integer :: ng

real :: advscr(maxk*7)
real, save :: ncall(maxg)=0
real, save, allocatable :: awtsx(:,:),awtsy(:,:),awtsz(:,:)

integer :: ia,iz,ja,jz,i0,j0,jd,n,nzp,nxp,nyp

nzp=nnzh(ng)
nxp=nnxh(ng)
nyp=nnyh(ng)
ia=2
iz=nxp-1
ja=2
jz=nyp-1
i0=0
j0=0
jd=1
if(nyp == 1) then
   jd=0
   ja=1
   jz=1
endif

! Get the advective weights if it is the first call
   
if(ncall(ng) == 0) then
   !print*,'doing advective weights for grid: ',ng
   if(ncall(1) == 0) then
      if(allocated(awtsx)) deallocate(awtsx)
      allocate (awtsx(6*6*maxxp,ngrh))
      if(allocated(awtsy)) deallocate(awtsy)
      allocate (awtsy(6*6*maxyp,ngrh))
      if(allocated(awtsz)) deallocate(awtsz)
      allocate (awtsz(6*6*maxzp,ngrh))
   endif
   call advwts (iadvord,1,1,0,awtsx(1,ng),awtsy(1,ng),awtsz(1,ng)  &
               ,dtpart,nxp,nyp,nzp,xth(1,ng),xmh(1,ng)  &
               ,yth(1,ng),ymh(1,ng),zth(1,ng),zmh(1,ng)  &
               ,maxxp,maxyp,maxzp)
   ncall(ng)=1
endif

call fvert (nzp,nxp,nyp,nzp,nxp,nyp,ia,iz,ja,jz,i0,j0  &
           ,scr%scr1,metc(ng)%u,metc(ng)%v,metc(ng)%w  &
           ,grid(ng)%f13t,grid(ng)%f23t,grid(ng)%rtgu  &
           ,grid(ng)%rtgv,grid(ng)%rtgt,hwh(1,ng),jd)

call fa_prepxt (nzp,nxp,nyp,nzp,nxp,nyp,ia,iz,ja,jz,i0,j0  &
               ,metc(ng)%u,scr%scr2,scr%scr3,scr%scr4  &
               ,grid(ng)%rtgu,grid(ng)%rtgt,grid(ng)%dxt  &
               ,grid(ng)%fmapu,grid(ng)%fmapt)

call fa_prepyt (nzp,nxp,nyp,nzp,nxp,nyp,ia,iz,ja,jz,i0,j0  &
               ,metc(ng)%v,scr%scr5,scr%scr6,scr%scr7  &
               ,grid(ng)%rtgv,grid(ng)%rtgt,grid(ng)%dyt  &
               ,grid(ng)%fmapv,grid(ng)%fmapt)

call fa_prepzt (nzp,nxp,nyp,nzp,nxp,nyp,ia,iz,ja,jz,i0,j0  &
               ,scr%scr1,scr%scr8,scr%scr9  &
               ,grid(ng)%rtgt,grid(ng)%rtgt,dzth(1,ng)  &
               ,grid(ng)%fmapt,grid(ng)%fmapt)

do n=1,nspecies

   call atob(nzp*nxp*nyp,eul(n,ng)%spec(1,1,1),scr%scr10)

   call fa_x (nzp,nxp,nyp,nzp,nxp,nyp,ia,iz,ja,jz,i0,j0  &
             ,metc(ng)%u,scr%scr2,scr%scr10,eul(n,ng)%spec(1,1,1)  &
             ,scr%scr11,scr%scr3,scr%scr4,advscr  &
             ,awtsx(1,ng),maxxp,iadvord,dtpart)
     
   call fa_y (nzp,nxp,nyp,nzp,nxp,nyp,ia,iz,ja,jz,i0,j0  &
             ,metc(ng)%v,scr%scr5,scr%scr10,eul(n,ng)%spec(1,1,1)  &
             ,scr%scr11,scr%scr6,scr%scr7,advscr  &
             ,awtsy(1,ng),maxyp,iadvord,dtpart)

   call fa_z (nzp,nxp,nyp,nzp,nxp,nyp,ia,iz,ja,jz,i0,j0  &
             ,scr%scr1,scr%scr1,scr%scr10,eul(n,ng)%spec(1,1,1)  &
             ,scr%scr11,scr%scr8,scr%scr9  &
             ,grid(ng)%rtgt,advscr  &
             ,awtsz(1,ng),maxzp,iadvord,dtpart)

   call advfill (nzp,nxp,nyp,ia,iz,ja,jz,scr%scr10,eul(n,ng)%spec(1,1,1))

enddo

return
end

!***************************************************************************

subroutine advfill (n1,n2,n3,ia,iz,ja,jz,a,b)

dimension a(n1,n2,n3),b(n1,n2,n3)

do j=ja,jz
   do i=ia,iz
      do k=2,n1-1
         b(k,i,j)=a(k,i,j)
      enddo
   enddo
enddo

RETURN
END

!***************************************************************************

subroutine fa_prepxt (n1,n2,n3,m1,m2,m3,ia,iz,ja,jz,i0,j0  &
                     ,vel,velw,wtf,wtr,rtgf,rtgr,dxr,fmapf,fmapr)
                     
dimension vel(m1,m2,m3),wtf(m1,m2,m3),wtr(m1,m2,m3)  &
         ,dxr(n2,n3),fmapf(n2,n3),fmapr(n2,n3)  &
         ,rtgf(n2,n3),rtgr(n2,n3),velw(m1,m2,m3)

do j=ja,jz
   do i=ia-1,iz
      do k=2,m1-1
         velw(k,i,j)=vel(k,i,j)*fmapf(i+i0,j+j0)
         wtf(k,i,j)=rtgf(i+i0,j+j0)/fmapf(i+i0,j+j0)**2
         wtr(k,i,j)=dxr(i+i0,j+j0)*fmapr(i+i0,j+j0)/rtgr(i+i0,j+j0)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine fa_prepyt (n1,n2,n3,m1,m2,m3,ia,iz,ja,jz,i0,j0  &
                     ,vel,velw,wtf,wtr,rtgf,rtgr,dxr,fmapf,fmapr)
                     
dimension vel(m1,m2,m3),wtf(m1,m2,m3),wtr(m1,m2,m3)  &
         ,dxr(n2,n3),fmapf(n2,n3),fmapr(n2,n3)  &
         ,rtgf(n2,n3),rtgr(n2,n3),velw(m1,m2,m3)

do j=ja-1,jz
   do i=ia,iz
      do k=2,m1-1
         velw(k,i,j)=vel(k,i,j)*fmapf(i+i0,j+j0)
         wtf(k,i,j)=rtgf(i+i0,j+j0)/fmapf(i+i0,j+j0)**2
         wtr(k,i,j)=dxr(i+i0,j+j0)*fmapr(i+i0,j+j0)/rtgr(i+i0,j+j0)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine fa_prepzt (n1,n2,n3,m1,m2,m3,ia,iz,ja,jz,i0,j0  &
                     ,vel,wtf,wtr,rtgf,rtgr,dzr,fmapf,fmapr)
     
dimension vel(m1,m2,m3),wtf(m1,m2,m3),wtr(m1,m2,m3)  &
         ,dzr(n1),fmapf(n2,n3),fmapr(n2,n3)  &
         ,rtgf(n2,n3),rtgr(n2,n3),velw(m1,m2,m3)

do j=ja,jz
   do i=ia,iz
      do k=1,m1-1
         wtf(k,i,j)=1.
         wtr(k,i,j)=dzr(k)/rtgr(i+i0,j+j0)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine fa_x (n1,n2,n3,m1,m2,m3,ia,iz,ja,jz,i0,j0  &
                ,velf,velw,scp,scpp,flux,wtf,wtr,advscr,awt,np  &
                ,iadvord,dtlt)
                
dimension velf(m1,m2,m3),scp(m1,m2,m3),advscr(*),wtf(m1,m2,m3)  &
         ,scpp(m1,m2,m3),flux(m1,m2,m3),wtr(m1,m2,m3)  &
         ,awt(*),velw(m1,m2,m3),rtg(n2,n3)

! call routine to compute fluxes

if(iadvord == 2) then
   call faord2x (n1,n2,n3,m1,m2,m3,i0,j0,velw,flux,scp  &
                ,wtf,advscr,ia-1,iz,ja,jz,awt,np)
elseif(iadvord == 6) then
   call faord6x (n1,n2,n3,m1,m2,m3,i0,j0,velw,flux,scp  &
                ,wtf,advscr,ia-1,iz,ja,jz,awt,np)
endif

! Modify fluxes to retain positive-definite-ness. FLUX array
! is multiplied by wtr(k,i,j) so flux is now actual change
! in quantity. Another assumption is that if velocities
! on sides of box are in opposite directions, each flux can
! only remove 1/2 of quantity.

! Update quantity first with divergence term

!do j=ja,jz
!   do i=ia-1,iz
!      do k=2,m1-1
!         scp(k,i,j)=scp(k,i,j)
!                   +( (velw(k,i,j)*wtf(k,i,j)
!                   -velw(k,i-1,j)*wtf(k,i-1,j))*scpp(k,i,j)*dtlt
!                   )*wtr(k,i,j)
!      enddo
!   enddo
!enddo

! Then modify fluxes

!do j=ja,jz
!   do i=ia-1,iz
!      do k=2,m1-1
!         dfact=1.
!         if(velw(k,i,j).gt.0.) then
!            if(i.ne.ia-1) then
!               if(velw(k,i-1,j).lt.0.) dfact=.5
!            endif
!            flux(k,i,j)= max(0.,min(flux(k,i,j)  &
!                            ,dfact*scp(k,i,j)/wtr(k,i,j)))
!         elseif(velw(k,i,j).lt.0.) then
!            if(velw(k,i+1,j).gt.0..and.i.ne.iz) dfact=.5
!            flux(k,i,j)=-max(0.,min(-flux(k,i,j)  &
!                            ,dfact*scp(k,i+1,j)/wtr(k,i,j)))
!         endif
!      enddo
!   enddo
!enddo

! Compute convergence of fluxes

do j=ja,jz
   do i=ia,iz
      do k=2,m1-1
         !if(j==9.and.k==2) print*,i,scp(k,i,j),flux(k,i,j),flux(k,i-1,j),wtr(k,i,j)
         scp(k,i,j)=scp(k,i,j)-(flux(k,i,j)-flux(k,i-1,j))*wtr(k,i,j)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine fa_y (n1,n2,n3,m1,m2,m3,ia,iz,ja,jz,i0,j0  &
                ,velf,velw,scp,scpp,flux,wtf,wtr,advscr,awt,np  &
                ,iadvord,dtlt)
                
dimension velf(m1,m2,m3),scp(m1,m2,m3),advscr(*),wtf(m1,m2,m3)  &
         ,scpp(m1,m2,m3),flux(m1,m2,m3),wtr(m1,m2,m3)  &
         ,awt(*),velw(m1,m2,m3),rtg(n2,n3)

! Call routine to compute fluxes

if(iadvord == 2) then
   call faord2y (n1,n2,n3,m1,m2,m3,i0,j0,velw,flux,scp  &
                ,wtf,advscr,ia,iz,ja-1,jz,awt,np)
elseif(iadvord == 6) then
   call faord2y (n1,n2,n3,m1,m2,m3,i0,j0,velw,flux,scp  &
                ,wtf,advscr,ia,iz,ja-1,jz,awt,np)
endif

! Modify fluxes to retain positive-definite-ness

!do j=ja,jz
!   do i=ia,iz
!      do k=2,m1-1
!         scp(k,i,j)=scp(k,i,j)
!                   +((velw(k,i,j)*wtf(k,i,j)
!                   -velw(k,i,j-1)*wtf(k,i,j-1))*scpp(k,i,j)*dtlt
!                   )*wtr(k,i,j)
!      enddo
!   enddo
!enddo

!do j=ja,jz
!   do i=ia,iz
!      do k=2,m1-1
!         dfact=1.
!         if(velw(k,i,j).gt.0.) then
!            if(j.ne.ja-1) then
!               if(velw(k,i,j-1).lt.0.) dfact=.5
!            endif
!            flux(k,i,j)= max(0., min(flux(k,i,j)  &
!                            ,dfact*SCP(K,I,J)/wtr(k,i,j)))
!         elseif(velw(k,i,j).lt.0.) then
!            if(velw(k,i,j+1).gt.0..and.j.ne.jz) dfact=.5
!            flux(k,i,j)= - max(0.,min(-flux(k,i,j)  &
!                              ,dfact*SCP(K,I,J+1)/wtr(k,i,j)))
!         endif
!      enddo
!   enddo
!enddo

! Compute convergence of fluxes

do j=ja,jz
   do i=ia,iz
      do k=2,m1-1
         scp(k,i,j)=scp(k,i,j)-(flux(k,i,j)-flux(k,i,j-1))*wtr(k,i,j)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine fa_z (n1,n2,n3,m1,m2,m3,ia,iz,ja,jz,i0,j0  &
                ,velf,velw,scp,scpp,flux,wtf,wtr,rtg,advscr,awt,np  &
                ,iadvord,dtlt)
                
dimension velf(m1,m2,m3),scp(m1,m2,m3),advscr(*),wtf(m1,m2,m3)  &
         ,scpp(m1,m2,m3),flux(m1,m2,m3),wtr(m1,m2,m3)  &
         ,awt(*),velw(m1,m2,m3),rtg(n2,n3)

! Call routine to compute fluxes

if(iadvord == 2) then
   call faord2z (n1,n2,n3,m1,m2,m3,i0,j0,velw,flux,scp  &
                ,wtf,rtg,advscr,ia,iz,ja,jz,awt,np)
elseif(iadvord == 6) then
   call faord6z (n1,n2,n3,m1,m2,m3,i0,j0,velw,flux,scp  &
                ,wtf,rtg,advscr,ia,iz,ja,jz,awt,np)
endif

! Modify fluxes to retain positive-definite-ness

!do j=ja,jz
!   do i=ia,iz
!      do k=2,m1-1
!         scp(k,i,j)=scp(k,i,j)  &
!                   +((velw(k,i,j)*wtf(k,i,j)  &
!                   -velw(k-1,i,j)*wtf(k-1,i,j))*scpp(k,i,j)*dtlt  &
!                    )*wtr(k,i,j)
!      enddo
!   enddo
!enddo

!do j=ja,jz
!   do i=ia,iz
!      ! Not sure about this one - lower bounds of 1st index of velw exceeded
!      !do k=1,m1-1
!      do k=2,m1-1
!         dfact=1.
!         if(velw(k,i,j).gt.0.) then
!            if(k.ne.1) then
!               if(velw(k-1,i,j).lt.0.) dfact=.5
!            endif
!            flux(k,i,j)= max(0., min(flux(k,i,j)  &
!                        ,dfact*SCP(K,I,J)/wtr(k,i,j)))
!         elseif(velw(k,i,j).lt.0.) then
!            if(velw(k+1,i,j).gt.0..and.k.ne.m1-1) dfact=.5
!            flux(k,i,j)= - max(0.,min(-flux(k,i,j)  &
!                              ,dfact*SCP(K+1,I,J)/wtr(k,i,j)))
!         endif
!      enddo
!   enddo
!enddo

! Compute convergence of fluxes

do j=ja,jz
   do i=ia,iz
      do k=2,m1-1
         scp(k,i,j)=scp(k,i,j)-(flux(k,i,j)-flux(k-1,i,j))*wtr(k,i,j)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine faord6x (n1,n2,n3,m1,m2,m3,i0,j0,vel,flx,s,trwt  &
                   ,scr,i1,i2,j1,j2,ax,np)

! routine computes the 6th order advective fluxes given:
!  vel - advecting velocity
!  s - variable to be advected
     
dimension flx(m1,m2,m3),s(m1,m2,m3),trwt(m1,m2,m3)  &
         ,vel(m1,m2,m3),scr(m1,*),rtg(n2,n3),ax(np,6,6)

do j=j1,j2
   do i=i1,i2

      do k=1,m1
         scr(k,1)=(ax(i+i0,1,1)*s(k,i-2,j)+ax(i+i0,1,2)*s(k,i-1,j)  &
                  +ax(i+i0,1,3)*s(k,i,j)  +ax(i+i0,1,4)*s(k,i+1,j)  &
                  +ax(i+i0,1,5)*s(k,i+2,j)+ax(i+i0,1,6)*s(k,i+3,j))
         scr(k,2)=(ax(i+i0,2,1)*s(k,i-2,j)+ax(i+i0,2,2)*s(k,i-1,j)  &
                  +ax(i+i0,2,3)*s(k,i,j)  +ax(i+i0,2,4)*s(k,i+1,j)  &
                  +ax(i+i0,2,5)*s(k,i+2,j)+ax(i+i0,2,6)*s(k,i+3,j))
         scr(k,3)=(ax(i+i0,3,1)*s(k,i-2,j)+ax(i+i0,3,2)*s(k,i-1,j)  &
                  +ax(i+i0,3,3)*s(k,i,j)  +ax(i+i0,3,4)*s(k,i+1,j)  &
                  +ax(i+i0,3,5)*s(k,i+2,j)+ax(i+i0,3,6)*s(k,i+3,j))
         scr(k,4)=(ax(i+i0,4,1)*s(k,i-2,j)+ax(i+i0,4,2)*s(k,i-1,j)  &
                  +ax(i+i0,4,3)*s(k,i,j)  +ax(i+i0,4,4)*s(k,i+1,j)  &
                  +ax(i+i0,4,5)*s(k,i+2,j)+ax(i+i0,4,6)*s(k,i+3,j))
         scr(k,5)=(ax(i+i0,5,1)*s(k,i-2,j)+ax(i+i0,5,2)*s(k,i-1,j)  &
                  +ax(i+i0,5,3)*s(k,i,j)  +ax(i+i0,5,4)*s(k,i+1,j)  &
                  +ax(i+i0,5,5)*s(k,i+2,j)+ax(i+i0,5,6)*s(k,i+3,j))
         scr(k,6)=(ax(i+i0,6,1)*s(k,i-2,j)+ax(i+i0,6,2)*s(k,i-1,j)  &
                  +ax(i+i0,6,3)*s(k,i,j)  +ax(i+i0,6,4)*s(k,i+1,j)  &
                  +ax(i+i0,6,5)*s(k,i+2,j)+ax(i+i0,6,6)*s(k,i+3,j))
      enddo

      do k=1,m1
         scr(k,7)=vel(k,i,j)
         flx(k,i,j)=(scr(k,7)*(scr(k,1)+scr(k,7)*(scr(k,2)  &
                    +scr(k,7)*(scr(k,3)+scr(k,7)*(scr(k,4)  &
                    +scr(k,7)*(scr(k,5)+scr(k,7)*scr(k,6)))))))*trwt(k,i,j)
      enddo

   enddo
enddo

return
end

!***************************************************************************

subroutine faord6y (n1,n2,n3,m1,m2,m3,i0,j0,vel,s,trwt  &
                   ,scr,i1,i2,j1,j2,ay,np)
                   
dimension flx(m1,m2,m3),s(m1,m2,m3),trwt(m1,m2,m3)  &
         ,vel(m1,m2,m3),scr(m1,*),rtg(n2,n3),ay(np,6,6)

do j=j1,j2
   do i=i1,i2

      do k=1,m1
         scr(k,1)=(ay(j+j0,1,1)*s(k,i,j-2)+ay(j+j0,1,2)*s(k,i,j-1)  &
                  +ay(j+j0,1,3)*s(k,i,j)  +ay(j+j0,1,4)*s(k,i,j+1)  &
                  +ay(j+j0,1,5)*s(k,i,j+2)+ay(j+j0,1,6)*s(k,i,j+3))

         scr(k,2)=(ay(j+j0,2,1)*s(k,i,j-2)+ay(j+j0,2,2)*s(k,i,j-1)  &
                  +ay(j+j0,2,3)*s(k,i,j)  +ay(j+j0,2,4)*s(k,i,j+1)  &
                  +ay(j+j0,2,5)*s(k,i,j+2)+ay(j+j0,2,6)*s(k,i,j+3))

         scr(k,3)=(ay(j+j0,3,1)*s(k,i,j-2)+ay(j+j0,3,2)*s(k,i,j-1)  &
                  +ay(j+j0,3,3)*s(k,i,j)  +ay(j+j0,3,4)*s(k,i,j+1)  &
                  +ay(j+j0,3,5)*s(k,i,j+2)+ay(j+j0,3,6)*s(k,i,j+3))

         scr(k,4)=(ay(j+j0,4,1)*s(k,i,j-2)+ay(j+j0,4,2)*s(k,i,j-1)  &
                  +ay(j+j0,4,3)*s(k,i,j)  +ay(j+j0,4,4)*s(k,i,j+1)  &
                  +ay(j+j0,4,5)*s(k,i,j+2)+ay(j+j0,4,6)*s(k,i,j+3))

         scr(k,5)=(ay(j+j0,5,1)*s(k,i,j-2)+ay(j+j0,5,2)*s(k,i,j-1)  &
                  +ay(j+j0,5,3)*s(k,i,j)  +ay(j+j0,5,4)*s(k,i,j+1)  &
                  +ay(j+j0,5,5)*s(k,i,j+2)+ay(j+j0,5,6)*s(k,i,j+3))

         scr(k,6)=(ay(j+j0,6,1)*s(k,i,j-2)+ay(j+j0,6,2)*s(k,i,j-1)  &
                  +ay(j+j0,6,3)*s(k,i,j)  +ay(j+j0,6,4)*s(k,i,j+1)  &
                  +ay(j+j0,6,5)*s(k,i,j+2)+ay(j+j0,6,6)*s(k,i,j+3))

      enddo

      do k=1,m1
         scr(k,7)=vel(k,i,j)
         flx(k,i,j)=(scr(k,7)*(scr(k,1)+scr(k,7)*(scr(k,2)  &
                    +scr(k,7)*(scr(k,3)+scr(k,7)*(scr(k,4)  &
                    +scr(k,7)*(scr(k,5)+scr(k,7)*scr(k,6)))))))*trwt(k,i,j)

      enddo

   enddo
enddo

return
end

!***************************************************************************

subroutine faord6z (n1,n2,n3,m1,m2,m3,i0,j0,vel,flx,s,trwt  &
                   ,rtg,scr,i1,i2,j1,j2,az,np)
     
dimension flx(m1,m2,m3),s(m1,m2,m3),trwt(m1,m2,m3)  &
         ,vel(m1,m2,m3),scr(m1,*),rtg(n2,n3),az(np,6,6)

do j=j1,j2
   do i=i1,i2

      do k=1,m1
         scr(k,1)=(az(k,1,1)*s(k-2,i,j)+az(k,1,2)*s(k-1,i,j)  &
                  +az(k,1,3)*s(k,i,j)  +az(k,1,4)*s(k+1,i,j)  &
                  +az(k,1,5)*s(k+2,i,j)+az(k,1,6)*s(k+3,i,j))

         scr(k,2)=(az(k,2,1)*s(k-2,i,j)+az(k,2,2)*s(k-1,i,j)  &
                  +az(k,2,3)*s(k,i,j)  +az(k,2,4)*s(k+1,i,j)  &
                  +az(k,2,5)*s(k+2,i,j)+az(k,2,6)*s(k+3,i,j))

         scr(k,3)=(az(k,3,1)*s(k-2,i,j)+az(k,3,2)*s(k-1,i,j)  &
                  +az(k,3,3)*s(k,i,j)  +az(k,3,4)*s(k+1,i,j)  &
                  +az(k,3,5)*s(k+2,i,j)+az(k,3,6)*s(k+3,i,j))

         scr(k,4)=(az(k,4,1)*s(k-2,i,j)+az(k,4,2)*s(k-1,i,j)  &
                  +az(k,4,3)*s(k,i,j)  +az(k,4,4)*s(k+1,i,j)  &
                  +az(k,4,5)*s(k+2,i,j)+az(k,4,6)*s(k+3,i,j))

         scr(k,5)=(az(k,5,1)*s(k-2,i,j)+az(k,5,2)*s(k-1,i,j)  &
                  +az(k,5,3)*s(k,i,j)  +az(k,5,4)*s(k+1,i,j)  &
                  +az(k,5,5)*s(k+2,i,j)+az(k,5,6)*s(k+3,i,j))

         scr(k,6)=(az(k,6,1)*s(k-2,i,j)+az(k,6,2)*s(k-1,i,j)  &
                  +az(k,6,3)*s(k,i,j)  +az(k,6,4)*s(k+1,i,j)  &
                  +az(k,6,5)*s(k+2,i,j)+az(k,6,6)*s(k+3,i,j))

      enddo

      do k=1,m1
         scr(k,7)=vel(k,i,j)/rtg(i+i0,j+j0)
         flx(k,i,j)=(+scr(k,7)*(scr(k,1)+scr(k,7)*(scr(k,2)  &
                     +scr(k,7)*(scr(k,3)+scr(k,7)*(scr(k,4)  &
                     +scr(k,7)*(scr(k,5)+scr(k,7)*scr(k,6)))))))*trwt(k,i,j)
      enddo

   enddo
enddo

return
end

!***************************************************************************

subroutine faord2x (n1,n2,n3,m1,m2,m3,i0,j0,vel,flx,s,trwt  &
                   ,scr,i1,i2,j1,j2,ax,np)
                   
dimension flx(m1,m2,m3),s(m1,m2,m3),trwt(m1,m2,m3)  &
         ,vel(m1,m2,m3),scr(m1,*),rtg(n2,n3),ax(np,6,6)

do j=j1,j2
   do i=i1,i2
      do k=1,m1
         !if(j==9.and.k==2) print*,ax(i+i0,1,3),s(k,i,j),ax(i+i0,1,4),s(k,i+1,j)
         scr(k,1)=(ax(i+i0,1,3)*s(k,i,j)+ax(i+i0,1,4)*s(k,i+1,j))
         !if(j==9.and.k==2) print*,ax(i+i0,2,3),s(k,i,j),ax(i+i0,2,4),s(k,i+1,j)
         scr(k,2)=(ax(i+i0,2,3)*s(k,i,j)+ax(i+i0,2,4)*s(k,i+1,j))
      enddo

      do k=1,m1
         scr(k,7)=vel(k,i,j)
         flx(k,i,j)=(scr(k,7)*(scr(k,1)+scr(k,7)*scr(k,2)))*trwt(k,i,j)
         !if(j==9.and.k==2) print*,scr(k,7),scr(k,1),scr(k,2),trwt(k,i,j),flx(k,i,j)
      enddo

   enddo
enddo

return
end

!***************************************************************************

subroutine faord2y (n1,n2,n3,m1,m2,m3,i0,j0,vel,flx,s,trwt  &
                   ,scr,i1,i2,j1,j2,ay,np)
     
dimension flx(m1,m2,m3),s(m1,m2,m3),trwt(m1,m2,m3)  &
         ,vel(m1,m2,m3),scr(m1,*),rtg(n2,n3),ay(np,6,6)

do j=j1,j2
   do i=i1,i2
      do k=1,m1
         scr(k,1)=(ay(j+j0,1,3)*s(k,i,j)+ay(j+j0,1,4)*s(k,i,j+1))
         scr(k,2)=(ay(j+j0,2,3)*s(k,i,j)+ay(j+j0,2,4)*s(k,i,j+1))
      enddo
      do k=1,m1
         scr(k,7)=vel(k,i,j)
         flx(k,i,j)=( scr(k,7)*(scr(k,1)+scr(k,7)*scr(k,2)))*trwt(k,i,j)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine faord2z (n1,n2,n3,m1,m2,m3,i0,j0,vel,flx,s,trwt  &
                   ,rtg,scr,i1,i2,j1,j2,az,np)
dimension flx(m1,m2,m3),s(m1,m2,m3),trwt(m1,m2,m3)  &
         ,vel(m1,m2,m3),scr(m1,*),rtg(n2,n3),az(np,6,6)

do j=j1,j2
   do i=i1,i2
      do k=1,m1-1
         scr(k,1)=(az(k,1,3)*s(k,i,j)+az(k,1,4)*s(k+1,i,j))
         scr(k,2)=(az(k,2,3)*s(k,i,j)+az(k,2,4)*s(k+1,i,j))
      enddo
      do k=1,m1
         scr(k,7)=vel(k,i,j)/rtg(i+i0,j+j0)
         flx(k,i,j)=(scr(k,7)*(scr(k,1)+scr(k,7)*scr(k,2)))  &
                   *trwt(k,i,j)*rtg(i+i0,j+j0)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine advwts (iadvf,idxcon,idycon,idzcon,advwtx,advwty  &
                  ,advwtz,dtlt,nxp,nyp,nzp,xt,xm,yt,ym,zt,zm  &
                  ,maxxp,maxyp,maxzp)
     
dimension wtscr(6,6),xt(*),xm(*),yt(*),ym(*),zt(*),zm(*)  &
         ,advwtx(maxxp,6,6),advwty(maxyp,6,6),advwtz(maxzp,6,6)

! x - direction
do ii=1,nxp
   call awtcmp (xt,ii,xm,ii,0,1,nxp,idxcon,wtscr,iadvf)
   do ll=1,6
      do lll=1,6
         advwtx(ii,ll,lll)=wtscr(ll,lll)*dtlt**ll
      enddo
   enddo
enddo

! y - direction
if(nyp.gt.1) then
   do jj=1,nyp
      call awtcmp (yt,jj,ym,jj,0,1,nyp,idycon,wtscr,iadvf)
      do ll=1,6
         do lll=1,6
            advwty(jj,ll,lll)=wtscr(ll,lll)*dtlt**ll
         enddo
      enddo
   enddo
endif

! z - direction
do k=1,nzp-1
   call awtcmp (zt,k,zm,k,0,1,nzp-1,idzcon,wtscr,iadvf)
   do ll=1,6
      do lll=1,6
         advwtz(k,ll,lll)=wtscr(ll,lll)*dtlt**ll
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine fvert (n1,n2,n3,m1,m2,m3,ia,iz,ja,jz,i0,j0  &
                 ,wv,u,v,w,f13t,f23t,rtgu,rtgv,rtgt,hw,jdim)
                 
dimension wv(m1,m2,m3),w(m1,m2,m3),u(m1,m2,m3),v(m1,m2,m3)  &
         ,f13t(n2,n3),f23t(n2,n3),hw(*),rtgt(n2,n3)  &
         ,rtgu(n2,n3),rtgv(n2,n3)

do j=2,m3
   do i=2,m2
      do k=1,m1-1
         wv(k,i,j)=w(k,i,j)  &
              +.25*hw(k)*(  &
              ( (u(k,i,j)+u(k+1,i,j))*rtgu(i+i0,j+j0)  &
              +(u(k,i-1,j)+u(k+1,i-1,j))*rtgu(i+i0-1,j+j0)  &
              )*f13t(i+i0,j+j0)  &
              +( (v(k,i,j)+v(k+1,i,j))*rtgv(i+i0,j+j0)  &
              +(v(k,i,j-jdim)+v(k+1,i,j-jdim))  &
              *rtgv(i+i0,j+j0-jdim)  &
              )*f23t(i+i0,j+j0) )

      enddo
   enddo
enddo

return
end
