!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine hyp_driver

use hyp_coms
use mem_met

implicit none

! hg     - hypact grid data (7*3d & 1*2d for the hybrid grid)
!        - no concentrations
! es     - 3d concentration data on rams grids (if ieulout or ilagout
!          ge 2 nspecies*3*3d, if ieulout and ilagout ge 2 nspecies*4*3d,
!          if not nspecies*2*3d for each rams grid + a set of scratch
!          arrays with the largest of the grid sizes)
! eb     - 2b boundary data for rams grids (nspecies*2d between each
!          rams grid nested boundary)
! atp    - particle data (npartall,maxatp)
! latp   - particle data (npartall,maxlatp)
! lcatp  - particle data (npartall,maxlcatp)
! catp   - particle data (npartall,maxcatp)

integer :: nfn,lenf,irfile,iroff  &
          ,npsteps,nsumpart,npartall,npblks  &
          ,noinc,nstp,nstpb,i,ng,nval,isstp,ngrid
real :: simend,reldur,hytim,strtim,tavgtime,t1,t2
character(len=32) :: head1,head2
character(len=80) :: flnm

!-----------------------------------------------------------------
! fill normally-distributed, random number array
!-----------------------------------------------------------------
nent=maxent
call normdist(anorm,nent)

!-----------------------------------------------
! read analysis file info; find run and source 
!  beginnings and ends, read in first file times
!-----------------------------------------------
call hyp_setup()

!-----------------------------------------------
! set up h-grid
!-----------------------------------------------

call hgridpts ()
call hgridset ()
print*,'finished h-grid setup',nih,njh,nkh

!----------------------------------------------------------------------
! compute how many timesteps to run
!----------------------------------------------------------------------

npsteps=int((hytimemax-hytime)/abs(dtpart))
print*,'timesteps-',npsteps,hytimemax,hytime,dtpart

!----------------------------------------------------------------------
! print description of each source and calc number of particles
!----------------------------------------------------------------------
call emission_spec (nsumpart)
!print*,'nsumpart-  ',type(1)(1:3),nsumpart

!----------------------------------------------------------------------
! allocate particle attribute arrays
!----------------------------------------------------------------------
   
!print*,'allocating memory for particle arrays'
npblks=1
npartall=nsumpart*npblks
if(allocated(atp)) deallocate(atp); allocate(atp(npartall))

maxclusters=int(npartall/ihybpart+1.)
if(allocated(ncfill)) deallocate(ncfill); allocate (ncfill(maxclusters))
ncfill=0

!----------------------------------------------------------------------
! allocate memory for Eulerian species and define sources and
! initial conditions (includes Lag parts on Eul (rams) grid)
!----------------------------------------------------------------------

call euler_init()

!----------------------------------------------------------------------
! run through timestep loop
!----------------------------------------------------------------------


npart=0
nrelpart=0
ncluster=0
do ng=1,ngrh
   nnfmh(ng)=1
   ncnth(ng)=0
   nndtrat(ng)=1
enddo

do nstp=1,npsteps

   call timing(1,t1)
   
!----------------------------------------------------------------------
! read anal files for new future time if it is the appropriate time
!----------------------------------------------------------------------
!   print*,'fchk:',hytime,ftimes(ipstfl),ftimes(ifutfl)
!   print*,'fchk:',hytime,fnums(ipstfl),fnums(ifutfl),ipstfl,ifutfl
   do ng=1,ngrh
      if(nint(hytime) >= nint(ftimes(ifutfl(ng)))) then
   
         ! Transfer future into past
         metp(ng)=metf(ng)
         ipstfl(ng)=ifutfl(ng)
      
         ! Find future file for this grid
         do nfn = ipstfl(ng)+1,nend
            if (fexist(nfn,ng) .and.  &
                  nint(hytime) < nint(ftimes(nfn))) then
               ifutfl(ng)=nfn
               exit
            endif
         enddo

         call hyp_rdanal (metf(ng),ifutfl(ng),ng)
         print*,'future read:',ifutfl(1:ngrh)
      endif
   enddo

!-----------------------------------------------------------------
! average met files at specified frequency
!-----------------------------------------------------------------

         !print*,'avg:',hytime,freqavg,mod(hytime,freqavg)
   !if (mod(hytime,freqavg) < dtpart) call hyp_gridavg ()
   call hyp_gridavg ()

!-----------------------------------------------------------------
! skip this step if not upto any release yet
!-----------------------------------------------------------------
        
   if(hytime < hytim) then
      hytime=hytime+abs(dtpart)
      print*,'------------------------------------------------------'
      write(*,'(a,i5,a,f8.0,a)') ' timestep-',nstp,'    hypact time-',hytime  &
                                ,'   Skipping - no emissions yet'
      goto 5
   endif


!---------------------------------------------------
! emissions
!---------------------------------------------------

   call emissions ()
print*,'hd:3'

!-----------------------------------------------------------------------
! output initial fields. It is done here in case there are are instantaneous
!     sources.
!-----------------------------------------------------------------------

   if (nstp == 1) call hout_init()

print*,'hd:4'
   
!---------------------------------------------------
! move the particles forward 1 timestep
!---------------------------------------------------

   if(ilagran == 1)  call hadvect ()
print*,'hd:5'

!---------------------------------------------------
! timestep driver for eulerian mode
! loop through all grids and advance them a 'DTLONG' timestep.
!---------------------------------------------------

   if(ieuler==1) then
      ngrid=1
      30 continue
         ng=ngrid
         nndtrat(ng)=1
         isstp=mod(ncnth(ngrid),nndtrat(ngrid))+1
         
         if(ng /= 1) call es_nstbdriv (ng,isstp)
         call es_advect (ng)
         call es_diffuse (ng)
         call es_scsets (ng)
         
         39 ncnth(ngrid)=ncnth(ngrid)+1
         40 nnfmh(ngrid)=nnfmh(ngrid)+1
         if(nnfmh(ngrid).gt.ngrh) goto 50
         if(nxtnest(nnfmh(ngrid)).ne.ngrid) goto 40
         ngrid=nnfmh(ngrid)
         45 isstp=mod(ncnth(ngrid),nndtrat(ngrid))+1
         
         ! corase grid to nested bnds
         call es_prgintrp (ngrid)
         
         goto 30
         50 nnfmh(ngrid)=1
         if (ngrid==1) goto 80
         if(ncnth(ngrid).lt.ncnth(nxtnest(ngrid))*nndtrat(ngrid)) goto 30
         
         call es_nstfeed (ngrid)
         call es_scsets (ng-1)
         
         ngrid=nxtnest(ngrid)
         goto 40
     80 continue
   endif

!-----------------------------------------
! update time counter
!-----------------------------------------

   hytime=hytime+abs(dtpart)
   
   print*,'------------------------------------------------------'
   write(*,'(a,i5,a,f8.0)')   ' Timestep-',nstp,'    hypact time-',hytime

!---------------------------------------------------
! check clusters
!---------------------------------------------------

   if((mod(hytime+.001,hybfreq) < dtpart) .and. ilagran == 1)  &
      call cluster ()

!-----------------------------------------
! do averages
!-----------------------------------------

   if(mod(hytime+.001,hypfreq) >= hypfreq-avgtime) then
      if(ieulout >= 2) call eul_avg ()
      if(ilagout >= 2) then
         call lag_conc ()
         call lag_avg ()
      endif
   endif

!-----------------------------------------
! calc ppm - this is for parts - what about concs? - can be done post
! process using rams fields - but then cannot do avgs... hmm
!-----------------------------------------

   if(ilagran == 1)  call lag_ppm ()

!-----------------------------------------
! write output
!-----------------------------------------
   !print*,'+++++++out:',hytime,hypfreq,nstp,npsteps,mod(hytime+.001,hypfreq)
   if( mod(hytime+.001,hypfreq) < abs(dtpart) .or. nstp == npsteps) then

      ! convert lag particles to concs and probs on rams grid
      
      if(ilagout == 1.or.ilagout == 3) then
         call lag_conc ()
         if(ilagprob == 1) call lag_prob ()
      endif
      
      
      ! now output stuff
      call hout ()

      ! reset the average arrays
      
      if(ieulout >= 2.or.ilagout >= 2) call reset_avg ()
      
   endif

!-----------------------------------------
! end of time step
!-----------------------------------------

   call timing(2,t2)
   print 200, t2-t1
   200 format('  CPU time(sec): ',F8.3)
   5 continue

enddo
print*,'------------------------------------------------------'

return
end
