!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine lag_to_eul (atp1,atp2,atp3,rat,ng,nsp,ifl)

!------------------------------------------------------------------------

! This routine computes a concentration field on the RAMS Eulerian grids

! from a sum over Lagrangian particles based on the location and assigned
! mass or similar attribute of each particle.  The kernel method is used

! in which each particle contributes concentration uniformly over a
! particle-centered volume whose size is identical to the local cell
! volume on the finest RAMS grid in which the particle resides.  This
! volume most often overlaps parts of 8 different model grid cells
! (constituting a block of 2x2x2 cells in 3-d space).
!------------------------------------------------------------------------

use hyp_coms
use mem_grid
use mem_euler

implicit none

integer :: ng,nsp,ifl
real    :: atp1,atp2,atp3,rat

integer :: ihp,jhp,khp,mgp,itp,jtp,ktp,i,j,k,ind,indm,indm2,ind2d
real :: rtgt,dz,hg_xy,atp11,atp12,atp13,zkern

real :: qh100,qh010,qh001,qh200,qh020,qh002  &
       ,rimp,rjmp,rkmp,ritp,rjtp,rktp,mass_inc
 
real :: xykernel=0.,zkernel=0.      

! Find integer coordinates on h grid
ihp = int(atp1)
jhp = int(atp2)
khp = int(atp3)

! Find coordinate weights on h grid
qh200 = atp1 - float(ihp)
qh020 = atp2 - float(jhp)
qh002 = atp3 - float(khp)

! Find the rams grid
mgp = hg%mg(ihp,jhp,khp)
if(ng > 0) then
   if(ng > mgp) return
   mgp = ng
endif

! Find floating point and integer locations on the rams grid
rimp = hgg(mgp)%rim(ihp,jhp,khp) + qh200 * drim(ihp,mgp)
rjmp = hgg(mgp)%rjm(ihp,jhp,khp) + qh020 * drjm(jhp,mgp)
rkmp = hgg(mgp)%rkm(ihp,jhp,khp) + qh002 * drkm(khp,mgp)

if(xykernel <= 0. .or. zkernel <= 0.) then

   !-----------------------------------------------------------------------
   ! Here is where we use the direct count of particles in the grid cell

   ! The following conc values would usually be in ug/m3 since the units of
   !   rat are usually ug/particle and conc gets summed over all particles.

   itp = int(rimp) + 1
   jtp = int(rjmp) + 1
   ktp = int(rkmp) + 1

   rtgt = grid(mgp)%rtgt(itp,jtp)
   dz = zmh(ktp,mgp) - zmh(ktp-1,mgp)

   ! Find fill mass increment in correct array location
   mass_inc = rat * grid(mgp)%dxt(itp,jtp) * grid(mgp)%dxt(itp,jtp) &
            / (dz * rtgt)
   
   if    (ifl == 1) then
                eul(nsp,mgp)%spec(ktp,itp,jtp) =  &
                eul(nsp,mgp)%spec(ktp,itp,jtp) + mass_inc
   elseif(ifl == 2) then
                eul(nsp,mgp)%specl(ktp,itp,jtp) =  &
                eul(nsp,mgp)%specl(ktp,itp,jtp) + mass_inc
   elseif(ifl == 3) then
                eul(nsp,mgp)%speclprob(ktp,itp,jtp) =  &
                eul(nsp,mgp)%speclprob(ktp,itp,jtp) + mass_inc
   endif

else

   !-----------------------------------------------------------------------
   !  For now, don't use the kernel method but instead just count particles
   !  in each grid cell.  (All lines beginning with double !! get commented
   !  out for this reason.)
                 
   ritp = rimp + .5
   rjtp = rjmp + .5
   rktp = hgg(mgp)%rkt(ihp,jhp,khp) + (atp3 - float(khp)) * drkm(khp,mgp)
 
   itp = int(ritp)
   jtp = int(rjtp)
   ktp = int(rktp)
   
   rtgt = grid(mgp)%rtgt(itp,jtp)
   
   zkern=zkernel/rtgt
   
   atp11=hg_xy (nih,xh,atp1)
   atp12=hg_xy (njh,yh,atp2)
   atp13=hg_xy (nkh,zh,atp3)
   
   print*,atp11,ritp,itp,xykernel
   print*,atp12,rjtp,jtp,xykernel
   print*,atp13,rktp,ktp,rtgt,zkern
   
   print*,atp11-xykernel,atp11+xykernel
   print*,atp12-xykernel,atp12+xykernel
   print*,atp13-zkern,atp13+zkern
   
   stop 'hlag2eul: not implemented'

endif

return
end


