!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine hyp_gridset (h1,h2,h3,nnxh,nnyh,nnzh,ngrh  &
                        ,nxtnest,nestz1  &
                        ,ninest,njnest,nknest,nstratx,nstraty,nstratz1  &
                        ,delxh,delyh,delzh  &
                        ,xmh,xth,ipm,ymh,yth,jpm  &
                        ,zmh,dzmh,zth,dzth,kpm,nrz)
     
implicit none

integer :: h1,h2,h3,ngrh,nestz1
integer, dimension(*) :: nnxh,nnyh,nnzh
integer, dimension(*) :: ninest,njnest,nknest  &
                        ,nstratx,nstraty,nstratz1,nxtnest
real, dimension(*) :: delxh,delyh,delzh
real, dimension(h2,*) :: xmh,xth
real, dimension(h3,*) :: ymh,yth
real, dimension(h1,*) :: zmh,dzmh,zth,dzth
integer, dimension(h2,*) :: ipm
integer, dimension(h3,*) :: jpm
integer, dimension(h1,*) :: kpm,nrz
 
integer :: ngr,i,j,k,iinc,icnt,jinc,jcnt,kinc,kcnt,nrat,nestza


! set NRZFLG to the FM which influences the CM vertical spacing
! fill NRZ with the variable nest ratios.
! THIS DOES NOT TAKE INTO ACCOUNT GLOBAL RUNS

do ngr=1,ngrh
   do k=1,h1
      nrz(k,ngr)=1
   enddo
enddo

nestza=abs(nestz1)

if(nestza.gt.1.and.nestza.le.ngrh) then
   do k=2,nnzh(1)
      nrz(k,nestza)=max(1,nstratz1(k))
   enddo
   nrz(1,nestza)=nstratz1(2)
endif

! fill IPM, JPM and KPM arrays with parent grid index values for all fine grids.

do ngr=2,ngrh
   ipm(1,ngr)=ninest(ngr)
   iinc=1
   icnt=0
   do i=2,nnxh(ngr)
      ipm(i,ngr)=ipm(i-1,ngr)+iinc
      icnt=icnt+1
      if(icnt.ge.nstratx(ngr)) then
         icnt=0
         iinc=1
      else
         iinc=0
      endif
   enddo

   jpm(1,ngr)=njnest(ngr)
   jinc=1
   jcnt=0
   do j=2,nnyh(ngr)
      jpm(j,ngr)=jpm(j-1,ngr)+jinc
      jcnt=jcnt+1
      if(jcnt.ge.nstraty(ngr)) then
         jcnt=0
         jinc=1
      else
         jinc=0
      endif
   enddo

   if(nknest(ngr).eq.1) then
      kpm(1,ngr)=2
      kinc=0
   else
      kpm(1,ngr)=nknest(ngr)
      kinc=1
   endif
   kcnt=0
   do k=2,nnzh(ngr)
      kpm(k,ngr)=kpm(k-1,ngr)+kinc
      nrat=nrz(kpm(k,ngr),ngr)
      kcnt=kcnt+1
      if(kcnt.ge.nrat.and.(k.lt.nnzh(ngr)-1.or.  &
         kpm(k,ngr).lt.nnxh(nxtnest(ngr))-1)) then
         kcnt=0
         kinc=1
      else
         kinc=0
      endif
   enddo
enddo

return
end

!***************************************************************************

subroutine hyp_polarst (n2,n3,n2m,n3m,ngr,glat,glon  &
                       ,fmapu,fmapv,fmapt,fmapm,fmapui,fmapvi,fmapti  &
                       ,fmapmi,plat,plon,xth,xmh,yth,ymh,ihtran)
     
dimension glat(n2,n3),glon(n2,n3),fmapu(n2,n3),fmapui(n2,n3)  &
         ,fmapv(n2,n3),fmapt(n2,n3),fmapm(n2,n3),fmapvi(n2,n3)  &
         ,fmapti(n2,n3),fmapmi(n2,n3),xth(n2m,*),xmh(n2m,*)  &
         ,yth(n3m,*),ymh(n3m,*)
     
data erad/6367000./

!  Calculates map factors and inverse map factors at u,v,t,m-points and
!  geographical lat/lon at t-points for a given polar stereographic grid

!print*, 'in polarst:n2,n3',n2,n3

c1 = (2. * erad) ** 2
do i = 1,n2
   do j = 1,n3
      xm2 = xmh(i,ngr) * xmh(i,ngr)
      xt2 = xth(i,ngr) * xth(i,ngr)
      ym2 = ymh(j,ngr) * ymh(j,ngr)
      yt2 = yth(j,ngr) * yth(j,ngr)

      fmapt(i,j) = 1. + (xt2 + yt2) / c1
      fmapu(i,j) = 1. + (xm2 + yt2) / c1
      fmapv(i,j) = 1. + (xt2 + ym2) / c1
      fmapm(i,j) = 1. + (xm2 + ym2) / c1

      fmapui(i,j) = 1.0 / fmapu(i,j)
      fmapvi(i,j) = 1.0 / fmapv(i,j)
      fmapti(i,j) = 1.0 / fmapt(i,j)
      fmapmi(i,j) = 1.0 / fmapm(i,j)

      call xy_ll (glat(i,j),glon(i,j),plat,plon,xth(i,ngr),yth(j,ngr))

      !write(6,344) i,j,fmapt(i,j),fmapm(i,j),glat(i,j),glon(i,j)  &
      !            ,xth(i,ngr),yth(j,ngr)
      !344 format('polst:i,j,fmt,fmm,glt,gln,x,y',2i4,6e15.6)

   enddo
enddo

if (ihtran .eq. 0) then
   call ae0(n2*n3,fmapu,1.)
   call ae0(n2*n3,fmapv,1.)
   call ae0(n2*n3,fmapt,1.)
   call ae0(n2*n3,fmapm,1.)
   call ae0(n2*n3,fmapui,1.)
   call ae0(n2*n3,fmapvi,1.)
   call ae0(n2*n3,fmapti,1.)
   call ae0(n2*n3,fmapmi,1.)
endif

return
end

!***************************************************************************

subroutine hyp_grdspc (n2,n3,maxi,maxj,ngrid  &
                      ,dxu,dxv,dxt,dxm,dyu,dyv,dyt,dym  &
                      ,fmapu,fmapv,fmapt,fmapm,xth,xmh,yth,ymh)
     
dimension dxu(n2,n3),dxv(n2,n3),dxt(n2,n3),dxm(n2,n3)  &
         ,dyu(n2,n3),dyv(n2,n3),dyt(n2,n3),dym(n2,n3)  &
         ,fmapu(n2,n3),fmapv(n2,n3),fmapt(n2,n3),fmapm(n2,n3)  &
         ,xth(maxi,*),xmh(maxi,*),yth(maxj,*),ymh(maxj,*)

do j=1,n3
   do i=1,n2-1
      dxu(i,j)=fmapu(i,j)/(xth(i+1,ngrid)-xth(i,ngrid))
      dxm(i,j)=fmapm(i,j)/(xth(i+1,ngrid)-xth(i,ngrid))
   enddo
   dxu(n2,j)=dxu(n2-1,j)*fmapu(n2,j)/fmapu(n2-1,j)
   dxm(n2,j)=dxm(n2-1,j)*fmapm(n2,j)/fmapm(n2-1,j)
   do i=2,n2
      dxv(i,j)=fmapv(i,j)/(xmh(i,ngrid)-xmh(i-1,ngrid))
      dxt(i,j)=fmapt(i,j)/(xmh(i,ngrid)-xmh(i-1,ngrid))
   enddo
   dxv(1,j)=dxv(2,j)*fmapv(1,j)/fmapv(2,j)
   dxt(1,j)=dxt(2,j)*fmapt(1,j)/fmapt(2,j)
enddo

do i=1,n2
   do j=1,n3-1
      dyv(i,j)=fmapv(i,j)/(yth(j+1,ngrid)-yth(j,ngrid))
      dym(i,j)=fmapm(i,j)/(yth(j+1,ngrid)-yth(j,ngrid))
   enddo
   dyv(i,n3)=dyv(i,n3-1)*fmapv(i,n3)/fmapv(i,n3-1)
   dym(i,n3)=dym(i,n3-1)*fmapm(i,n3)/fmapm(i,n3-1)
   do j=2,n3
      dyu(i,j)=fmapu(i,j)/(ymh(j,ngrid)-ymh(j-1,ngrid))
      dyt(i,j)=fmapt(i,j)/(ymh(j,ngrid)-ymh(j-1,ngrid))
   enddo
   dyu(i,1)=dyu(i,2)*fmapu(i,1)/fmapu(i,2)
   dyt(i,1)=dyt(i,2)*fmapt(i,1)/fmapt(i,2)
enddo

return
end

!***************************************************************************

subroutine hyp_jacob (n1,n2,n3,ngr,maxi,maxj,maxk,topt,topu,topv  &
                     ,topm,rtgt,rtgu,rtgv,rtgm,f13u,f13v,f13t,f13m  &
                     ,f23u,f23v,f23t,f23m,dxu,dxv,dxt,dxm,dyu,dyv  &
                     ,dyt,dym,xth,xmh,yth,ymh,zth,zmh,hth,hwh,ztoph)
                     
dimension topt(n2,n3),topu(n2,n3),topv(n2,n3),topm(n2,n3)  &
         ,rtgt(n2,n3),rtgu(n2,n3),rtgv(n2,n3),rtgm(n2,n3)  &
         ,f13u(n2,n3),f13v(n2,n3),f13t(n2,n3),f13m(n2,n3)  &
         ,f23u(n2,n3),f23v(n2,n3),f23t(n2,n3),f23m(n2,n3)  &
         ,dxu(n2,n3),dxv(n2,n3),dxt(n2,n3),dxm(n2,n3)  &
         ,dyu(n2,n3),dyv(n2,n3),dyt(n2,n3),dym(n2,n3)  &
         ,xth(maxi,*),xmh(maxi,*),yth(maxj,*),ymh(maxj,*),zmh(maxk,*)  &
         ,hwh(maxk,*),hth(maxk,*),zth(maxk,*)

jdim=1

! this routine computes the coordinate transformation constants
! based on the topographical values of topt.

do j=1,n3
   do i=1,n2-1
      topu(i,j)=topt(i,j)+(topt(i+1,j)-topt(i,j))  &
           *(xmh(i,ngr)-xth(i,ngr))/(xth(i+1,ngr)-xth(i,ngr))
   enddo
   topu(n2,j)=topt(n2,j)+(topt(n2,j)-topt(n2-1,j))  &
        *(xmh(n2,ngr)-xth(n2,ngr))/(xth(n2,ngr)-xth(n2-1,ngr))
enddo

do i=1,n2
   do j=1,n3-1
      topv(i,j)=topt(i,j)+(topt(i,j+1)-topt(i,j))  &
           *(ymh(j,ngr)-yth(j,ngr))/(yth(j+1,ngr)-yth(j,ngr))
      topm(i,j)=topu(i,j)+(topu(i,j+1)-topu(i,j))  &
           *(ymh(j,ngr)-yth(j,ngr))/(yth(j+1,ngr)-yth(j,ngr))
   enddo
   topv(i,n3)=topt(i,n3)+(topt(i,n3)-topt(i,n3-1))  &
        *(ymh(n3,ngr)-yth(n3,ngr))/(yth(n3,ngr)-yth(n3-1,ngr))
   topm(i,n3)=topu(i,n3)+(topu(i,n3)-topu(i,n3-1))  &
        *(ymh(n3,ngr)-yth(n3,ngr))/(yth(n3,ngr)-yth(n3-1,ngr))
enddo

do k=1,n1
   hth(k,ngr)=zth(k,ngr)/ztoph-1.0
   hwh(k,ngr)=zmh(k,ngr)/ztoph-1.0
enddo


do j=1,n3
   do i=1,n2
      rtgt(i,j)=1.-topt(i,j)/ztoph
      rtgu(i,j)=1.-topu(i,j)/ztoph
      rtgv(i,j)=1.-topv(i,j)/ztoph
      rtgm(i,j)=1.-topm(i,j)/ztoph
   enddo
enddo

do j=1,n3
   do i=2,n2
      f13t(i,j)=(topu(i,j)-topu(i-1,j))*dxt(i,j)/rtgt(i,j)
      f13v(i,j)=(topm(i,j)-topm(i-1,j))*dxv(i,j)/rtgv(i,j)
   enddo
enddo
do j=1,n3
   do i=1,n2-1
      f13u(i,j)=(topt(i+1,j)-topt(i,j))*dxu(i,j)/rtgu(i,j)
      f13m(i,j)=(topv(i+1,j)-topv(i,j))*dxm(i,j)/rtgm(i,j)
   enddo
enddo

do j=2,n3
   do i=1,n2
      f23t(i,j)=(topv(i,j)-topv(i,j-jdim))*dyt(i,j)/rtgt(i,j)
      f23u(i,j)=(topm(i,j)-topm(i,j-jdim))*dyu(i,j)/rtgu(i,j)
   enddo
enddo
do j=1,n3-1
   do i=1,n2
      f23v(i,j)=(topt(i,j+jdim)-topt(i,j))*dyv(i,j)/rtgv(i,j)
      f23m(i,j)=(topu(i,j+jdim)-topu(i,j))*dym(i,j)/rtgm(i,j)
   enddo
enddo

do j=1,n3
   f13t(1,j)=f13u(1,j)
   f13v(1,j)=f13m(1,j)
   f13u(n2,j)=f13t(n2,j)
   f13m(n2,j)=f13v(n2,j)
enddo
do i=1,n2
   f23t(i,1)=f23v(i,1)
   f23u(i,1)=f23m(i,1)
   f23v(i,n3)=f23t(i,n3)
   f23m(i,n3)=f23u(i,n3)
enddo

return
end

!***************************************************************************

SUBROUTINE hyp_nestcof (ngrh,nnzh,nnxh,nnyh,maxk,maxi,maxj,kpmh  &
                       ,ei1h,ei2h,ei3h,ej1h,ej2h,ej3h,ek1h,ek2h,ek3h  &
                       ,nxtnest,nstratx,nstraty,zth,dzth,zmh)
                       
dimension nnzh(*),nnxh(*),nnyh(*),kpmh(maxk,*)  &
         ,ei1h(maxi,*),ei2h(maxi,*),ei3h(maxi,*)  &
         ,ej1h(maxj,*),ej2h(maxj,*),ej3h(maxj,*)  &
         ,ek1h(maxk,*),ek2h(maxk,*),ek3h(maxk,*)  &
         ,nxtnest(*),nstratx(*),nstraty(*),zth(maxk,*),dzth(maxk,*)  &
         ,zmh(maxk,*)
dimension vctr1(200),vctr2(200),vctr3(200)

jdim=1
do nf=2,ngrh
   nc=nxtnest(nf)
   nrat=nstratx(nf)
   alpha=((1./float(nrat))**2-1.)/24.
   do if=1,nnxh(nf)
      et=-0.5+float(2*mod(if+nrat-2,nrat)+1)/(2.0*float(nrat))

      ei1h(if,nf)=et*(et-1.)/2.+alpha
      ei2h(if,nf)=(1.-et*et)-2.*alpha
      ei3h(if,nf)=et*(et+1.)/2.+alpha
   enddo

   if(jdim.eq.1)then
      nrat=nstraty(nf)
      alpha=((1./float(nrat))**2-1.)/24.
      do jf=1,nnyh(nf)
         et=-0.5+float(2*mod(jf+nrat-2,nrat)+1)/(2.0*float(nrat))

         ej1h(jf,nf)=et*(et-1.)/2.+alpha
         ej2h(jf,nf)=(1.-et*et)-2.*alpha
         ej3h(jf,nf)=et*(et+1.)/2.+alpha
      enddo
   endif

   do kc=1,nnzh(nc)
      vctr1(kc)=0.
      vctr2(kc)=0.
      vctr3(kc)=0.
   enddo

   do kf=1,nnzh(nf)
      kc=kpmh(kf,nf)
      ek1h(kf,nf)=  &
           (zth(kf,nf)-zth(kc,nc))*(zth(kf,nf)-zth(kc+1,nc))  &
           /((zth(kc-1,nc)-zth(kc,nc))  &
           *(zth(kc-1,nc)-zth(kc+1,nc)))
      ek2h(kf,nf)=  &
           (zth(kf,nf)-zth(kc-1,nc))*(zth(kf,nf)-zth(kc+1,nc))  &
          /((zth(kc,nc)-zth(kc-1,nc))  &
           *(zth(kc,nc)-zth(kc+1,nc)))
      ek3h(kf,nf)=  &
           (zth(kf,nf)-zth(kc-1,nc))*(zth(kf,nf)-zth(kc,nc))  &
           /((zth(kc+1,nc)-zth(kc-1,nc))  &
           *(zth(kc+1,nc)-zth(kc,nc)))
      if(kf.ge.2.and.kf.le.nnzh(nf)-1)then
         vctr1(kc)=vctr1(kc)+(zmh(kf,nf)-zmh(kf-1,nf))*ek1h(kf,nf)
         vctr2(kc)=vctr2(kc)+(zmh(kf,nf)-zmh(kf-1,nf))*ek2h(kf,nf)
         vctr3(kc)=vctr3(kc)+(zmh(kf,nf)-zmh(kf-1,nf))*ek3h(kf,nf)
      endif
   enddo


   do kf=2,nnzh(nf)-1
      kc=kpmh(kf,nf)
      ek1h(kf,nf)=ek1h(kf,nf)-dzth(kc,nc)*vctr1(kc)
      ek2h(kf,nf)=ek2h(kf,nf)-dzth(kc,nc)*vctr2(kc)+1.
      ek3h(kf,nf)=ek3h(kf,nf)-dzth(kc,nc)*vctr3(kc)
   enddo

enddo

return
end
