!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine emission_spec (nsumpart)

use hyp_coms

implicit none

integer :: cmu,nsumpart,iyear,imon,idate,nsrc,ierr,inst,idim  &
       ,i,ii,ip,ip2,ireject,lastchar,iy,imn,id,ihms,ih,im,is,nc,iun,iver  &
       ,ist,isr,isp,isu,indx1,indx2,itime
real :: zasl(maxem),tsumpart,durtot,x1,y1  &
       ,atp1,atp2,atp3,hifromx,hg_z,htopo,rtg,a1,a2,x2,x3,y2,y3,y4  &
       ,fa1,fa2,xmax,ymax,xmin,ymin,xgeom,ygeom,xmean,ymean,rpart  &
       ,sumpart,xavg,yavg,zavg
character(len=80) :: flnm,spname
character(len=3) :: stype
character(len=2) :: chdim


print*
print*,'------------------------------------------------------'
print*
print*,'HYPACT Source Summary'
print*
print*,'------------------------------------------------------'

tsumpart=0.
do nsrc=1,nsources
   print*
   write(*,'(a,i3)') 'Model Emission Source: ',nsrc
   print*

   ! can have a point source if the center location is specified by lat lon
   ! hgt (ie 'rec' or 'ell') and the source has 0 size in each dimension
   ! - if the source is defined by 'spe' or 'rat' scale is set 1 one dim unit
   ! (ie dimensionality is lost - 'abs' is effectively treated the same as 'spe')
   ! - in any case all the parts will be place at the lat lon hgt location

   scale(nsrc)=1.
   chdim='pt'
   ierr=0

   ! inst. emission is made over 1 timestep

   inst=0
   if (relend(nsrc).eq.relstrt(nsrc)) then ! inst. emission
      inst=1
      relend(nsrc)=relstrt(nsrc)+abs(dtpart)
      durtot=abs(dtpart)
      !emission(nsrc)=emission(nsrc)/abs(dtpart)
      !numparts(nsrc)=numparts(nsrc)*abs(dtpart)
   else
      durtot=relend(nsrc)-relstrt(nsrc)
   endif

   ! if calc by total parts, reset numparts and then do things by rate

   stype=scaling(nsrc)(1:3)
   if(scaling(nsrc)(1:3).eq.'tot') then
      rparts(nsrc)=float(numparts(nsrc))*abs(dtpart)/durtot
      scaling(nsrc)(1:3)='rat'
   else
      rparts(nsrc)=float(numparts(nsrc))
   endif

   ! RECTANGLULAR SOURCE

   if(sshape(nsrc)(1:3).eq.'rec')then

      ! convert lat/lon coordinates to x/y

      call ll_xy(srcy(nsrc),srcx(nsrc),pslat,pslon,x1,y1)
      xy(1,nsrc)=x1
      xy(2,nsrc)=y1

      ! source dimension

      idim=0
      if(xsize(nsrc).gt.0.) idim=idim+1
      if(ysize(nsrc).gt.0.) idim=idim+1
      if(zsize(nsrc).gt.0.) idim=idim+1
      if(idim.eq.0) then
         chdim=' '
      elseif(idim.eq.1) then
         chdim='m'
      elseif(idim.eq.2) then
         chdim='m2'
      elseif(idim.eq.3) then
         chdim='m3'
      endif

      if(xsize(nsrc).gt.0.) scale(nsrc)=scale(nsrc)*xsize(nsrc)
      if(ysize(nsrc).gt.0.) scale(nsrc)=scale(nsrc)*ysize(nsrc)
      if(zsize(nsrc).gt.0.) scale(nsrc)=scale(nsrc)*zsize(nsrc)

      ! recalc the vertical extent and height of release in terrain
      ! following coords
      atp1=hifromx (nih,xh,xy(1,nsrc))
      atp2=hifromx (njh,yh,xy(2,nsrc))
      atp3=hg_z (nih,njh,nkh,xh,yh,zh,atp1,atp2,1.,hg%topthg(1,1)  &
                ,nih,njh,ztoph,htopo)
      rtg=1.-htopo/zh(nkh)
      srcz(nsrc)=srcz(nsrc)/rtg
      zsize(nsrc)=zsize(nsrc)/rtg

      ! calc the release ASL in cart coords

      zasl(nsrc)=srcz(nsrc)+htopo

      ! emissions and the real number of particles to release

      if(scaling(nsrc)(1:3).eq.'rat') then
         emission(nsrc)=rparts(nsrc)*ratio(nsrc)/(scale(nsrc)*abs(dtpart))
      elseif(scaling(nsrc)(1:3).eq.'abs') then
         if(ratio(nsrc).gt.0.) then
            rparts(nsrc)=emission(nsrc)*abs(dtpart)/ratio(nsrc)
         else
            ratio(nsrc)=emission(nsrc)*abs(dtpart)/rparts(nsrc)
         endif
      elseif(scaling(nsrc)(1:3).eq.'spe') then
         if(ratio(nsrc).gt.0.) then
            rparts(nsrc)=emission(nsrc)*scale(nsrc)*abs(dtpart)/ratio(nsrc)
         else
            ratio(nsrc)=emission(nsrc)*scale(nsrc)*abs(dtpart)/rparts(nsrc)
         endif
      else
         print*,'emission_spec: ',nsrc,' - ',scaling(nsrc)
         stop 'source spec err'
      endif

   ! TRIANGLULAR SOURCE

   elseif(sshape(nsrc)(1:3).eq.'tri') then

      ! source dimension

      if(polylat(4,nsrc).ne.-999..or.  &
         polylon(4,nsrc).ne.-999.) then
         print*,'emission_spec: ',nsrc,' - ',sshape(nsrc)
         print*,polylat(4,nsrc),polylon(4,nsrc)
         stop 'source spec err - too many vertices for triangle'
      endif
      if(polylat(1,nsrc).eq.polylat(2,nsrc).and.  &
         polylon(1,nsrc).eq.polylon(2,nsrc)) then
         print*,'emission_spec: ',nsrc,' - ',sshape(nsrc)
         print*,'polylat- 1',polylat(1,nsrc),' 2',polylat(2,nsrc)
         print*,'polylon- 1',polylon(1,nsrc),' 2',polylon(2,nsrc)
         stop 'source spec err - vertices 1 and 2 identical'
      endif
      if(polylat_db(1,nsrc).eq.polylat(3,nsrc).and.  &
         polylon_db(1,nsrc).eq.polylon(3,nsrc)) then
         print*,'emission_spec: ',nsrc,' - ',sshape(nsrc)
         print*,'polylat- 1',polylat(1,nsrc),' 3',polylat(3,nsrc)
         print*,'polylon- 1',polylon(1,nsrc),' 3',polylon(3,nsrc)
         stop 'source spec err - vertices 1 and 3 identical'
      endif
      if(polylat(2,nsrc).eq.polylat(3,nsrc).and.  &
         polylon(2,nsrc).eq.polylon(3,nsrc)) then
         print*,'emission_spec: ',nsrc,' - ',sshape(nsrc)
         print*,'polylat- 2',polylat(2,nsrc),' 3',polylat(3,nsrc)
         print*,'polylon- 2',polylon(2,nsrc),' 3',polylon(3,nsrc)
         stop 'source spec err - vertices 2 and 3 identical'
      endif

      chdim='m2'
      if(zsize(nsrc).gt.0.)then
         chdim='m3'
      endif

      ! convert lat/lon coordinates to x/y

      call ll_xy(polylat(1,nsrc),polylon(1,nsrc)  &
         ,pslat,pslon,xy(1,nsrc),xy(2,nsrc))
      call ll_xy(polylat(2,nsrc),polylon(2,nsrc)  &
         ,pslat,pslon,xy(3,nsrc),xy(4,nsrc))
      call ll_xy(polylat(3,nsrc),polylon(3,nsrc)  &
         ,pslat,pslon,xy(5,nsrc),xy(6,nsrc))

      ! re-order vertices if necessary

      do ii=1,3
         i=mod(2*ii,4)+1
         if(xy(i,nsrc).gt.xy(i+2,nsrc))then
            a1=xy(i,nsrc)
            xy(i,nsrc)=xy(i+2,nsrc)
            xy(i+2,nsrc)=a1
            a1=xy(i+1,nsrc)
            xy(i+1,nsrc)=xy(i+3,nsrc)
            xy(i+3,nsrc)=a1
         endif
      enddo

      x1=xy(1,nsrc)
      x2=xy(3,nsrc)
      x3=xy(5,nsrc)
      y1=xy(2,nsrc)
      y2=xy(4,nsrc)
      y3=xy(6,nsrc)

      y4=y1+(y3-y1)*(x2-x1)/(x3-x1)
      a1=0.5*(x2-x1)*abs(y4-y2)
      a2=0.5*(x3-x2)*abs(y4-y2)
      fa1=max(1.e-10,a1/(a1+a2))
      fa2=max(1.e-10,1.-fa1)

      scale(nsrc)=a1+a2
      if(zsize(nsrc).gt.0.) scale(nsrc)=scale(nsrc)*zsize(nsrc)

      ! recalc the vertical extent and height of release in terrain
      ! following coords

      atp1=hifromx (nih,xh,xy(1,nsrc))
      atp2=hifromx (njh,yh,xy(2,nsrc))
      atp3=hg_z (nih,njh,nkh,xh,yh,zh,atp1,atp2,1.,hg%topthg(1,1)  &
                ,nih,njh,ztoph,htopo)
      rtg=1.-htopo/zh(nkh)
      srcz(nsrc)=srcz(nsrc)/rtg
      zsize(nsrc)=zsize(nsrc)/rtg

      ! calc the release ASL in cart coords

      zasl(nsrc)=srcz(nsrc)+htopo

      ! emissions and the real number of particles to release

      if(scaling(nsrc)(1:3).eq.'rat') then
         emission(nsrc)=rparts(nsrc)*ratio(nsrc)/(scale(nsrc)*abs(dtpart))
      elseif(scaling(nsrc)(1:3).eq.'abs') then
         if(ratio(nsrc).gt.0.) then
            rparts(nsrc)=emission(nsrc)*abs(dtpart)/ratio(nsrc)
         else
            ratio(nsrc)=emission(nsrc)*abs(dtpart)/rparts(nsrc)
         endif
      elseif(scaling(nsrc)(1:3).eq.'spe') then
         if(ratio(nsrc).gt.0.) then
            rparts(nsrc)=emission(nsrc)*scale(nsrc)*abs(dtpart)/ratio(nsrc)
         else
            ratio(nsrc)=emission(nsrc)*scale(nsrc)*abs(dtpart)/rparts(nsrc)
         endif
      else
         print*,'emission_spec: ',nsrc,' - ',scaling(nsrc)
         stop 'source spec err'
      endif

   ! IRREGULAR POLYGON SOURCE

   elseif(sshape(nsrc)(1:3).eq.'pol') then

      ! source dimension

      do ip=1,npolypts(nsrc)
         do ip2=1,npolypts(nsrc)
            if(polylat(ip,nsrc).eq.polylat(ip2,nsrc).and.  &
               polylon(ip,nsrc).eq.polylon(ip2,nsrc).and.  &
               ip.ne.ip2) then
               print*,'emission_spec: ',nsrc,' - ',sshape(nsrc)
               print*,'polylat- 1',polylat(1,nsrc)  &
                             ,' 2',polylat(2,nsrc)
               print*,'polylon- 1',polylon(1,nsrc)  &
                             ,' 2',polylon(2,nsrc)
               stop 'source spec err - two vertices are identical'
            endif
         enddo
      enddo

      chdim='m2'
      if(zsize(nsrc).gt.0.)then
         chdim='m3'
      endif

      ! convert lat/lon coordinates to x/y and set center point options

      xmax=-1e20
      ymax=-1e20
      xmin=1e20
      ymin=1e20
      xgeom=0.
      ygeom=0.
      xmean=0.
      ymean=0.
      do ip=1,npolypts(nsrc)
         call ll_xy (polylat(ip,nsrc),polylon(ip,nsrc)  &
                    ,pslat,pslon,xy(6*ip-5,nsrc),xy(6*ip-4,nsrc))
         xmax=max(xmax,xy(6*ip-5,nsrc))
         ymax=max(ymax,xy(6*ip-4,nsrc))
         xmin=min(xmin,xy(6*ip-5,nsrc))
         ymin=min(ymin,xy(6*ip-4,nsrc))
         xmean=xmean+xy(6*ip-5,nsrc)
         ymean=ymean+xy(6*ip-4,nsrc)
      enddo
      xgeom=(xmax+xmin)/2
      ygeom=(ymax+ymin)/2
      xmean=xmean/npolypts(nsrc)
      ymean=ymean/npolypts(nsrc)

      ! break polygon up into triangles

      ! first try to use center defined by srcx and srcy
      ! if that fails try the geometric center of the points
      ! if that fails try the mean center of the points

      call ll_xy (srcy(nsrc),srcx(nsrc),pslat,pslon,x1,y1)
      call test_poly (ireject,nsrc,x1,y1)
      if(ireject.eq.0) goto 130
      x1=xgeom
      y1=ygeom
      call test_poly (ireject,nsrc,x1,y1)
      if(ireject.eq.0) goto 130
      x1=xmean
      y1=ymean
      call test_poly (ireject,nsrc,x1,y1)
      if(ireject.eq.0) goto 130
      print*,'failed to split up polygon into triangles'  &
            ,' for source:',nsrc
      stop 're-examine polygon source'
      130 continue

      ! assign corners of triangles (there will be npolypts triangles)

      call xy_ll (srcy(nsrc),srcx(nsrc),pslat,pslon,x1,y1)
      xy(6*npolypts(nsrc)+1,nsrc)=xy(1,nsrc)
      xy(6*npolypts(nsrc)+2,nsrc)=xy(2,nsrc)
      do ip=1,npolypts(nsrc)
         xy(6*ip-3,nsrc)=xy(6*(ip+1)-5,nsrc)
         xy(6*ip-2,nsrc)=xy(6*(ip+1)-4,nsrc)
         xy(6*ip-1,nsrc)=x1
         xy(6*ip,nsrc)=y1
      enddo

      ! re-order vertices if necessary

      scale(nsrc)=0.
      do ip=1,npolypts(nsrc)
         do ii=1,3
            i=mod(2*ii,4)+1
            if(xy(6*(ip-1)+i,nsrc).gt.xy(6*(ip-1)+i+2,nsrc)) then
               a1=xy(6*(ip-1)+i,nsrc)
               xy(6*(ip-1)+i,nsrc)=xy(6*(ip-1)+i+2,nsrc)
               xy(6*(ip-1)+i+2,nsrc)=a1
               a1=xy(6*(ip-1)+i+1,nsrc)
               xy(6*(ip-1)+i+1,nsrc)=xy(6*(ip-1)+i+3,nsrc)
               xy(6*(ip-1)+i+3,nsrc)=a1
            endif
         enddo

         ! size of source

         x1=xy(6*(ip-1)+1,nsrc)
         x2=xy(6*(ip-1)+3,nsrc)
         x3=xy(6*(ip-1)+5,nsrc)
         y1=xy(6*(ip-1)+2,nsrc)
         y2=xy(6*(ip-1)+4,nsrc)
         y3=xy(6*(ip-1)+6,nsrc)

         y4=y1+(y3-y1)*(x2-x1)/(x3-x1)
         a1=0.5*(x2-x1)*abs(y4-y2)
         a2=0.5*(x3-x2)*abs(y4-y2)
         fa1=max(1.e-10,a1/(a1+a2))
         fa2=max(1.e-10,1.-fa1)

         scalep(ip,nsrc)=a1+a2
         if(zsize(nsrc).gt.0.)  &
            scalep(ip,nsrc)=scalep(ip,nsrc)*zsize(nsrc)
         scale(nsrc)=scale(nsrc)+scalep(ip,nsrc)

         ! recalc the vertical extent and height of release in terrain
         ! following coords

         atp1=hifromx (nih,xh,xy(1,nsrc))
         atp2=hifromx (njh,yh,xy(2,nsrc))
         atp3=hg_z (nih,njh,nkh,xh,yh,zh,atp1,atp2,1.,hg%topthg(1,1)  &
                ,nih,njh,ztoph,htopo)
         rtg=1.-htopo/zh(nkh)
         zasl(nsrc)=srcz(nsrc)+htopo
         srcz(nsrc)=srcz(nsrc)/rtg
         zsize(nsrc)=zsize(nsrc)/rtg

         ! calc the release ASL in cart coords

         zasl(nsrc)=srcz(nsrc)+htopo

      enddo

      ! emissions and the real number of particles to release

      rpart=0
      do ip=1,npolypts(nsrc)

         if(scaling(nsrc)(1:3).eq.'rat') then
           ratiop(ip,nsrc)=ratio(nsrc)
           rpartsp(ip,nsrc)=rparts(nsrc)*scalep(ip,nsrc)/scale(nsrc)
           emissionp(ip,nsrc)=rpartsp(ip,nsrc)*ratio(nsrc)  &
                             /(scalep(ip,nsrc)*abs(dtpart))
         elseif(scaling(nsrc)(1:3).eq.'abs') then
           emissionp(ip,nsrc)=emission(nsrc)*scalep(ip,nsrc)  &
                             /scale(nsrc)
           if(ratio(nsrc).gt.0.) then
            rpartsp(ip,nsrc)=emissionp(ip,nsrc)*abs(dtpart)/ratio(nsrc)
            ratiop(ip,nsrc)=ratio(nsrc)
           else
            rpartsp(ip,nsrc)=rparts(nsrc)*scalep(ip,nsrc)/scale(nsrc)
            ratiop(ip,nsrc)=emissionp(ip,nsrc)*abs(dtpart)/rpartsp(ip,nsrc)
           endif
         elseif(scaling(nsrc)(1:3).eq.'spe') then
           emissionp(ip,nsrc)=emission(nsrc)
           if(ratio(nsrc).gt.0.) then
            rpartsp(ip,nsrc)=emission(nsrc)*scalep(ip,nsrc)*abs(dtpart)  &
                           /ratio(nsrc)
            ratiop(ip,nsrc)=ratio(nsrc)
           else
            rpartsp(ip,nsrc)=rparts(nsrc)*scalep(ip,nsrc)/scale(nsrc)
            ratiop(ip,nsrc)=emission(nsrc)*scalep(ip,nsrc)*abs(dtpart)  &
                           /rpartsp(ip,nsrc)
           endif
         else
           print*,'emission_spec: ',nsrc,' - ',scaling(nsrc)
           stop 'source spec err'
         endif

         rpart=rpart+rpartsp(ip,nsrc)
      enddo
      rparts(nsrc)=rpart

   ! ELLIPTICAL SOURCE

   elseif(sshape(nsrc)(1:3).eq.'ell') then

      ! convert lat/lon coordinates to x/y

      call ll_xy (srcy(nsrc),srcx(nsrc),pslat,pslon,x1,y1)
      xy(1,nsrc)=x1
      xy(2,nsrc)=y1

      ! source dimension

      if(xsize(nsrc).eq.0.or.ysize(nsrc).eq.0) then
         print*,'emission_spec: ',nsrc,' - ',sshape(nsrc)
         print*,'xsize- ',xsize(nsrc),' ysize-',ysize(nsrc)
         stop 'source spec err - zero horizontal dimension'
      endif
      chdim='m2'
      if(zsize(nsrc).gt.0.)then
         chdim='m3'
      endif

      scale(nsrc)=0.25*3.14159*xsize(nsrc)*ysize(nsrc)
      if(zsize(nsrc).gt.0.) scale(nsrc)=scale(nsrc)*zsize(nsrc)

      ! recalc the vertical extent and height of release in terrain
      ! following coords

      atp1=hifromx (nih,xh,xy(1,nsrc))
      atp2=hifromx (njh,yh,xy(2,nsrc))
      atp3=hg_z (nih,njh,nkh,xh,yh,zh,atp1,atp2,1.,hg%topthg(1,1)  &
                ,nih,njh,ztoph,htopo)
      rtg=1.-htopo/zh(nkh)
      srcz(nsrc)=srcz(nsrc)/rtg
      zsize(nsrc)=zsize(nsrc)/rtg

      ! calc the release ASL in cart coords

      zasl(nsrc)=srcz(nsrc)+htopo

      ! emissions and the real number of particles to release

      if(scaling(nsrc)(1:3).eq.'rat') then
         emission(nsrc)=rparts(nsrc)*ratio(nsrc)/(scale(nsrc)*abs(dtpart))
      elseif(scaling(nsrc)(1:3).eq.'abs') then
         if(ratio(nsrc).gt.0.) then
            rparts(nsrc)=emission(nsrc)*abs(dtpart)/ratio(nsrc)
         else
            ratio(nsrc)=emission(nsrc)*abs(dtpart)/rparts(nsrc)
         endif
      elseif(scaling(nsrc)(1:3).eq.'spe') then
         if(ratio(nsrc).gt.0.) then
            rparts(nsrc)=emission(nsrc)*scale(nsrc)*abs(dtpart)/ratio(nsrc)
         else
            ratio(nsrc)=emission(nsrc)*scale(nsrc)*abs(dtpart)/rparts(nsrc)
         endif
      else
         print*,'emission_spec: ',nsrc,' - ',scaling(nsrc)
         stop 'source spec err'
      endif

   ! GRIDDED SOURCE

   elseif (sshape(nsrc)(1:4).eq.'grid') then

      print*,'emission_spec: ',nsrc,' - ',sshape(nsrc)
      stop 'feature not complete'

      ! nothing to configure

   else
      print*,'emission_spec: ',nsrc,' - ',sshape(nsrc)
      stop 'source spec err'
   endif

   ! calculate the total particles released for this source and sum for all
   ! sources (this is used in the memory allocation) - note that Eulerian
   ! sources only makes 'temporary' use of the particles
   
   if(ilagran == 1) then
      sumpart=rparts(nsrc)*durtot/abs(dtpart)
      tsumpart=tsumpart+sumpart
   else
      sumpart=rparts(nsrc)
      tsumpart=max(tsumpart,sumpart)
   endif

   ! print a summary of this source

   cmu=lastchar(units(species(nsrc)))
   indx1=lastchar(srcname(nsrc))
   indx2=lastchar(specname(species(nsrc)))
   write(*,'(2(a,i3,a2,a))') ' Source: ',psource(nsrc),'  '  &
    ,srcname(nsrc)(1:indx1),'  Species: ',pspecies(species(nsrc))  &
    ,'  ',specname(species(nsrc))(1:indx2)
   print*,'Type definition       -  ',type(nsrc),stype

   print*
   if(sshape(nsrc)(1:3).eq.'rec'.and.chdim(1:1).eq.' ') then
      print*,'Source shape          -  ','point'
   elseif(sshape(nsrc)(1:3).eq.'rec'.and.  &
          xsize(nsrc).eq.ysize(nsrc)) then
      print*,'Source shape          -  ','square'
   elseif(sshape(nsrc)(1:3).eq.'rec'.and.  &
          xsize(nsrc).ne.ysize(nsrc)) then
      print*,'Source shape          -  ','rectangular'
   elseif(sshape(nsrc)(1:3).eq.'tri') then
      print*,'Source shape          -  ','triangular'
   elseif(sshape(nsrc)(1:3).eq.'pol'.and.npolypts(nsrc).eq.3) then
      print*,'Source shape          -  ','triangular'
   elseif(sshape(nsrc)(1:3).eq.'pol'.and.  &
          npolypts(nsrc).gt.3) then
      print*,'Source shape          -  ','polygonal'
   elseif(sshape(nsrc)(1:3).eq.'pol'.and.  &
          npolypts(nsrc).gt.3) then
      print*,'Number of sides       -  ',npolypts(nsrc)
   elseif(sshape(nsrc)(1:3).eq.'ell'.and.  &
          xsize(nsrc).eq.ysize(nsrc)) then
      print*,'Source shape          -  ','circular'
   elseif(sshape(nsrc)(1:3).eq.'ell'.and.  &
          xsize(nsrc).ne.ysize(nsrc)) then
      print*,'Source shape          -  ','elliptical'
   elseif(sshape(nsrc)(1:4).eq.'grid') then
      print*,'Source shape          -  ','gridded'
   endif

   print*,'Location(lat/long)    - ',srcy(nsrc),srcx(nsrc)
   call ll_xy(srcy(nsrc),srcx(nsrc),pslat,pslon,x1,y1)
   print*,'Centre (x,y,z)        - ',x1,y1,srcz(nsrc)*rtg,' m'
   if(nint(rotation(nsrc)).gt.0)  &
   print*,'Rotation              - ',rotation(nsrc),' deg'
   if(chdim(1:1).ne.' ') then
      print*,'Height Dimension      - ',zsize(nsrc)*rtg,' m'
      if(sshape(nsrc).eq.'rec'.or.sshape(nsrc).eq.'ell')  &
      print*,'Horizontal Dimensions - ',xsize(nsrc),' x'  &
            ,ysize(nsrc),' m2'
      if(sshape(nsrc).eq.'tri'.or.sshape(nsrc).eq.'pol')  &
      print*,'Horizontal Dimensions - '  &
            ,'defined by lat/long coords'
      print*,'Size                  - ',scale(nsrc),' ',chdim
   endif

   print*
   if (inst == 1) then ! inst. emission

      print*,'Emission time         - ',relstrt(nsrc),' s'
      print*,'Particles to release  - ',rparts(nsrc)  &
            ,' (total for this source)'
      print*,'Mass per particle     - ',ratio(nsrc),' '  &
            ,units(species(nsrc))(1:cmu),'/particle'

   else

      print*,'Emission times:xxxxxxx'
      print*,'  start               - ',relstrt(nsrc),' s'
      print*,'  end                 - ',relend(nsrc),' s'
      print*,'  duration            - ',relend(nsrc)-relstrt(nsrc),' s'
      print*,'Timestep              - ',abs(dtpart),' s'
      print*,'Timesteps             - ',durtot/abs(dtpart)

      print*,'Particles to release  - ',rparts(nsrc),' per timestep'
      print*,'                      - ',sumpart  &
            ,' total for this source'
      print*,'Mass per particle     - ',ratio(nsrc),' '  &
            ,units(species(nsrc))(1:cmu),'/particle'

   endif

   if(scaling(nsrc)(1:3).eq.'abs') then

      if(lastchar(chdim).eq.0) then
         print*,'Emission              - ',emission(nsrc),' '  &
               ,units(species(nsrc))(1:cmu),'/s'
      else
         print*,'Emission              - ',emission(nsrc),' '  &
               ,units(species(nsrc))(1:cmu),'/s'
         print*,'                      - '  &
               ,emission(nsrc)/scale(nsrc)  &
               ,' ',units(species(nsrc))(1:cmu),'/',chdim,'/s'
      endif
      print*,'                      - '  &
            ,abs(dtpart)*emission(nsrc)  &
            ,' ',units(species(nsrc))(1:cmu),'/timestep'
      print*,'                      - '  &
            ,durtot*emission(nsrc)  &
            ,' ',units(species(nsrc))(1:cmu)  &
            ,' (total for this source)'

   elseif(scaling(nsrc)(1:3).eq.'spe'.or.  &
          scaling(nsrc)(1:3).eq.'rat') then

      if(lastchar(chdim).eq.0) then
         print*,'Emission rate         - ',emission(nsrc),' '  &
               ,units(species(nsrc))(1:cmu),'/s'
      else
         print*,'Emission rate         - ',emission(nsrc),' '  &
               ,units(species(nsrc))(1:cmu),'/',chdim,'/s'
         print*,'                      - '  &
               ,emission(nsrc)*scale(nsrc)  &
               ,' ',units(species(nsrc))(1:cmu),'/s'
      endif
      print*,'                      - '  &
            ,abs(dtpart)*emission(nsrc)*scale(nsrc)  &
            ,' ',units(species(nsrc))(1:cmu),'/timestep'
      print*,'                      - '  &
            ,durtot*emission(nsrc)*scale(nsrc)  &
            ,' ',units(species(nsrc))(1:cmu)  &
            ,' (total for this source)'

   endif

   print*
   print*,'------------------------------------------------------'
   1 format(2(a,i3))

enddo

! check total particles released less than specified by maxpart

if(ilagran == 1) then
   nsumpart=int(tsumpart+1.)
else
   nsumpart=1
endif

print*,'Total particles       - ',tsumpart, '(all sources)'
if(nsumpart.gt.maxpart) then
   print*
   print*,'Maximum particles specified by maxpart-',maxpart
   print*
   stop 'too many particles'
endif

print*
print*,'------------------------------------------------------'
print*
print*,' Source analysis complete'
print*
print*,'------------------------------------------------------'
print*

! get the UTC start date of the HYPACT run
read(startdate(1:4),*) iyear ; read(startdate(6:7),*) imon
read(startdate(9:10),*) idate ; read(startdate(12:15),*) itime
call date_add_to(iyear,imon,idate,itime*100  &
     ,hytime,'s',iy,imn,id,ihms)
ih=int(float(ihms)/10000.)
im=int(float(ihms-ih*10000)/100.)
is=ihms-ih*10000-im*100

! get the average source center (for easy positioning of cgrid)

xavg=0.
yavg=0.
zavg=0.
do nsrc=1,nsources
   xavg=xavg+srcx(nsrc)/nsources
   yavg=yavg+srcy(nsrc)/nsources
   zavg=zavg+zasl(nsrc)/nsources
enddo

! output to SPEC file

nc=lastchar(hyppref)
flnm=hyppref(1:nc)//'-spec.txt'
iun=78
open(iun,file=flnm,form='formatted',status='unknown')
iver=1
write(iun,60) iver,nsources,nspecies,xavg,yavg,zavg  &
             ,iy,imn,id,ih,im
do nsrc=1,nsources
   ist=index(type(nsrc),' ')-1
   isr=index(srcname(nsrc),' ')-1
   isp=index(specname(species(nsrc)),' ')-1
   spname="'"//specname(species(nsrc))(1:isp)//"'"  ! because :'s terminate format control
   isu=index(units(species(nsrc)),' ')-1
   write(iun,61) psource(nsrc),' ',type(nsrc)(1:ist+1)  &
        ,srcname(nsrc)(1:isr+1)  &
        ,srcy(nsrc),srcx(nsrc),srcz(nsrc),zasl(nsrc)  &
        ,pspecies(species(nsrc)),' ',spname(1:isp+3)  &
        ,units(species(nsrc))(1:isu+1),wgtmol(species(nsrc))
enddo
close(iun)
60 format(3i3,2f12.5,f10.1,i5,4i3)
61 format(i3,3a,2f12.5,2f10.1,i3,3a,f10.5)

return
end

!***************************************************************************

subroutine test_poly (ireject,nsrc,x1,y1)

! two tests are required to ensure triangles do not extend outside source:
! 1. each point to center line must sequentially move around the polygon
! - a backward 'movement' indicates that the current point to center line
! has crossed back over the previous triangle
! 2. the maximum change in angle between point to center lines is 180 deg
! - any more indicates that the center is outside the polygon

use hyp_coms
dimension theta1(polypts+1),theta2(polypts+1)

ireject=0

do ip=1,npolypts(nsrc)
   a=xy(6*ip-5,nsrc)-x1
   b=xy(6*ip-4,nsrc)-y1
   if(a.eq.0..and.b.eq.0.) then
      print*,'Center on polgon point for source:',nsrc
    stop 'Redefine polygon'
 else
    theta1(ip)=180.-atan2(a,-b)*180/3.1415927
    theta2(ip)=180.-atan2(-a,b)*180/3.1415927
   endif
enddo

theta1(npolypts(nsrc)+1)=theta1(1)
theta2(npolypts(nsrc)+1)=theta2(1)
do ip=1,npolypts(nsrc)
   if(theta1(ip).le.theta1(ip+1).and.  &
      theta2(ip).le.theta2(ip+1)) ireject=1
enddo

do ip=1,npolypts(nsrc)
   if(theta1(ip)-theta1(ip+1).lt.0.) then
      if(theta1(ip)-theta1(ip+1)+360..ge.180.) ireject=1
   else
      if(theta1(ip)-theta1(ip+1).ge.180.) ireject=1
   endif
enddo

return
end

