!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine DIAGT4A(n1,n2,n3,u,v,theta,tke  &
                 ,uu,vv,ww,ds2,tlx,tly,tlz,zt,zm,rtgt)


!      -------------------------------------------------------------
!       modified turbulence closure scheme level 2.5
!       (Mellor and Yamada, 1982, Helfand and Labraga, 1988)
!      diagnostic calculation of wind velocity variances (uu,vv,ww)
!      and Lagrangian time scales (tlx,tly,tlz)
!      -------------------------------------------------------------

parameter ( rkar=0.4,      alfl=0.1,      zl=0.035,  &
                c2=0.75,      tkemin=1.e-7,      b1=16.6 )
parameter ( cc2=-c2**2, tkemin2=2.*tkemin )

dimension u(n1,n2,n3),v(n1,n2,n3),theta(n1,n2,n3),tke(n1,n2,n3)  &
           ,zt(n1),zm(n1),rtgt(n2,n3)
dimension uu(n1,n2,n3),vv(n1,n2,n3),ww(n1,n2,n3),ds2(n1,n2,n3)  &
                ,tlx(n1,n2,n3),tly(n1,n2,n3),tlz(n1,n2,n3)

!      -------------------------------------------------------------

do j=1,n3
do i=1,n2

!       * asymptotic mixing length .......................................

    ccc1=0.
    ccc2=0.
    do k=2,n1-1
       cc=amax1(tke(k,i,j),tkemin)
       e1=sqrt(cc)*(zm(k+1)-zm(k))*rtgt(i,j)
       ccc1=ccc1+zt(k)*rtgt(i,j)*e1
       ccc2=ccc2+e1
    enddo
    cln=alfl*ccc1/ccc2

    do k=2,n1-1

       cc=rkar*zt(k)*rtgt(i,j)
       xl=cc/(1.+cc/cln)
       !if(xl.gt.10000.)print*,'xl-',k,i,j,xl,cc,cln,ccc1,ccc2
       !print*,'xl-',k,i,j,rtgt(i,j),zt(k),rkar
       !print*,'xl-',k,i,j,xl,cc,cln,ccc1,ccc2
       q2=2.*max(tke(k,i,j),tkemin)
       q1=sqrt(q2)

       dz2=(zt(k+1)-zt(k-1))*rtgt(i,j)
       dtemp=-zl*(theta(k+1,i,j)-theta(k-1,i,j))/dz2
       du=(u(k+1,i,j)-u(k-1,i,j))/dz2
       dv=(v(k+1,i,j)-v(k-1,i,j))/dz2
       duv=du*du+dv*dv
       if (dtemp.lt.0..and.k.gt.3) then
           xl=min( xl, c2*sqrt(-q2/dtemp))
           xlq=xl*xl/q2
           gh=dtemp*xlq
         gh=max( gh, cc2)
       else
           xlq=xl*xl/q2
           gh=dtemp*xlq
     endif
     xq=sqrt(xlq)
     gu=xq*du
     gv=xq*dv
       gm=duv*xlq

!          * equilibrium value of TKE from diagnostic equation ..........

     ri=-dtemp/max(duv,1.e-11)
       call LEVEL2 (ri,sme,she)
       qe2=max( b1*xl*xl*(sme*duv+she*dtemp) , tkemin2 )
     qq=q2/qe2
!             if(qe2.gt.1000.)print*,'qe2-',k,i,j,qe2,ri,sme,she
!             if(qe2.gt.1000.)print*,'qe2-',k,i,j,xl,duv,dtemp

       if(qe2.gt.q2) then

!              * growing turbulence case

           qqq=sqrt(qq)
           sm=qqq*sme
           sh=qqq*she

         else

!              * decaying turbulence case

           call LEVEL25 (gm,gh,sm,sh)

       endif

!          * variances and covariances of wind velocity and
!            Lagrangian times scales

       qlk=xl*q1
     call DIF4A(qq,gu,gv,gm,gh,sm,sh,q2,qlk,uu(k,i,j),  &
            vv(k,i,j),ww(k,i,j),  &
            tlx(k,i,j),tly(k,i,j),tlz(k,i,j))

      enddo

!NG SINCE THE TURBULENT VELOCITY COMPONENTS ARE ALREADY
! VARIANCES, THEY SHOULD NOT BE SQUARED.


      do k=2,n1-1

! THIS IS INCORRECT.
!               ds2(k,i,j)=(ww(k+1,i,j)**2-ww(k-1,i,j)**2)
!     +              /((zt(k+1)-zt(k-1))*rtgt(i,j))

! THIS IS CORRECT.
!
!                ds2(k,i,j)=(ww(k+1,i,j)-ww(k-1,i,j))
!     +              /((zt(k+1)-zt(k-1))*rtgt(i,j))

! HOWEVER, THINGS SEEM TO WORK BETTER WITH sig.d(sig)/dz RATHER THAN
! 1/2.d(sig**2)/dz. HENCE THE FOLLOWING (WITH CORRESPONDING CHANGE IN
! HADVECT.F
         ds2(k,i,j)=(sqrt(ww(k+1,i,j))-sqrt(ww(k-1,i,j)))  &
              /((zt(k+1)-zt(k-1))*rtgt(i,j))


      enddo


   enddo
enddo


return
end


!       ************************************************************
!
       subroutine LEVEL2 (ri,sme,she)

!      -----------------------------------------------------------
!       turbulence closure scheme level 2 (Mellor and Yamada, 1982)
!      nondimensional eddy diffusivities: sme, she
!      -----------------------------------------------------------

parameter (a1=0.92, a2=0.74, b1=16.6, b2=10.1, c1=0.08)

 parameter (  &
     sh1=a2*(3.*b2+b1+12.*a1)/b1,  &
     sm1=a1*(-3.*c1*b1+b1+9.*a2+12.*a1)/(a2*(3.*b2+b1+3.*a1)),  &
     rf2=(b1-6.*a1)/(3.*b2+b1+12.*a1),  &
     rf3=(b1-6.*a1)/(3.*b2+b1+3.*a1),  &
     rf4=(3.*c1*b1-b1+6.*a1)/(3.*c1*b1-b1-9.*a2-12.*a1)    )

       ri=min(ri, rf2 )
       ccc=max( ri*(ri-0.3221)+0.03156 , 0.)
       rf=min(0.6588*(ri+0.1776-sqrt(ccc)), rf2 )
       she=sh1*(rf-rf2)/(rf-1.)
       sme=sm1*(rf-rf4)/(rf-rf3)*she

       return
       end


!      **********************************************************


       subroutine LEVEL25 (gm,gh,sm,sh)

!      -------------------------------------------------------------
!       modified turbulence closure scheme level 2.5
!       (Mellor and Yamada, 1982, Helfand and Labraga, 1988)
!      nondimensional eddy diffusivities: sm, sh
!      -------------------------------------------------------------

parameter (a1=0.92, a2=0.74, b1=16.6, b2=10.1, c1=0.08)

  parameter(   cc2=a1*(1.-3.*c1),  &
               d1=6.*a1*a2,  &
               d2=a2*(3.*b2+12.*a1),  &
               d3=6.*a1*a1,  &
               d4=9.*a1*a2,  &
               d5=2.*d3+d4            )


  aa1=d1*gm
  bb1=1.-gh*d2
  aa2=1.+gm*d3-gh*d4
  bb2=-gh*d5
  w=aa1*bb2-aa2*bb1
  sm=(bb2*a2-bb1*cc2)/w
  sh=(aa1*cc2-aa2*a2)/w

  return
  end

!       ************************************************************
subroutine DIF4A(qq,gu,gv,gm,gh,sm,sh,q2,ql,  &
                   uu,vv,ww,tlx,tly,tlz)

!      -------------------------------------------------------------
!       modified turbulence closure scheme level 2.5
!       (Mellor and Yamada, 1982, Helfand and Labraga, 1988)
!      wind velocity variances (uu,vv,ww)
!      and Lagrangian time scales (tlx,tly,tlz)
!      -------------------------------------------------------------

parameter (a1=0.92, a2=0.74, b2=10.1)
parameter (d=1.5*a2*b2, a23=3*a2, a16=6*a1)
parameter (ctl=2.)


cc3=q2/3.

if(q2.gt.2.e-7) then

   cqq=amin1(sqrt(qq),1.)
   guu=gu*gu
   gvv=gv*gv
   sig=-2.*(gm*sm+gh*sh)
   cc1=a1*q2*cqq
   uu=amax1( cc3+cc1*(sig+6.*guu*sm), cc3)
   vv=amax1( cc3+cc1*(sig+6.*gvv*sm) , cc3)
   ww=amax1( cc3+cc1*(sig+6.*gh*sh) , cc3)

else

   uu=cc3
   vv=cc3
   ww=cc3

endif

tlx=ctl*ql/sqrt(uu*q2)
tly=ctl*ql/sqrt(vv*q2)
tlz=ctl*ql/sqrt(ww*q2)

return
end




