!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================


Module mem_euler
   
   ! Added scalar variables and tendencies

   Type eul_vars   
      real, pointer, dimension(:,:,:) ::  &
                              spec,specavg,specl,speclavg,speclprob
      real, pointer, dimension(:,:) :: drydep
   End Type
   
   !    allocated by (nspecies,ngrids)
   type (eul_vars), allocatable :: eul(:,:)
  
Contains
!---------------------------------------------------------------

   subroutine alloc_euler(es,n1,n2,n3,nspecies)
   
   use hyp_coms, only: ieuler,ieulout,ilagran,ilagout,ilagprob

   implicit none

   type (eul_vars) :: es(*)
   integer :: n1,n2,n3
   integer :: nspecies
   
   integer :: nsc

   ! Allocate arrays based on options (if necessary).
print*,'alloc_eul:',nspecies,ieuler,ilagran,ilagout
   do nsc=1,nspecies
      if (ieuler == 1) then
         allocate (es(nsc)%spec     (n1,n2,n3))
         if(ieulout >= 2) allocate (es(nsc)%specavg  (n1,n2,n3))
         allocate (es(nsc)%drydep(n2,n3))
      endif
      if (ilagran == 1) then
         if(ilagout >= 1)  allocate (es(nsc)%specl  (n1,n2,n3))
         if(ilagout >= 2)  allocate (es(nsc)%speclavg (n1,n2,n3))
         if(ilagprob == 1) allocate (es(nsc)%speclprob(n1,n2,n3))
      endif
   enddo
print*,'alloc_eul:',associated(eul(1,1)%specl)
          
   return
   end subroutine

!---------------------------------------------------------------

   subroutine dealloc_euler(es,nspecies)

   implicit none

   type (eul_vars) :: es(*)

   integer :: nspecies
   integer :: nsc

   !  Deallocate arrays

   do nsc=1,nspecies
      if (associated(es(nsc)%spec     )) deallocate (es(nsc)%spec     )
      if (associated(es(nsc)%specavg  )) deallocate (es(nsc)%specavg  )
      if (associated(es(nsc)%specl    )) deallocate (es(nsc)%specl  )
      if (associated(es(nsc)%speclavg )) deallocate (es(nsc)%speclavg )
      if (associated(es(nsc)%speclprob)) deallocate (es(nsc)%speclprob)
      if (associated(es(nsc)%drydep)) deallocate (es(nsc)%drydep)
   enddo
           
   return
   end subroutine

!---------------------------------------------------------------

   subroutine nullify_euler(es,nspecies)

   implicit none

   type (eul_vars) :: es(*)

   integer :: nspecies
   integer :: nsc

   !  nullify pointers

   do nsc=1,nspecies
      if (associated(es(nsc)%spec     ))   nullify (es(nsc)%spec     )
      if (associated(es(nsc)%specavg  ))   nullify (es(nsc)%specavg  )
      if (associated(es(nsc)%specl    ))   nullify (es(nsc)%specl  )
      if (associated(es(nsc)%speclavg ))   nullify (es(nsc)%speclavg )
      if (associated(es(nsc)%speclprob))   nullify (es(nsc)%speclprob)
      if (associated(es(nsc)%drydep)) nullify (es(nsc)%drydep)
   enddo
           
   return
   end subroutine

End Module
