!############################# Change Log ##################################
! 1.0.0
!
!
!###########################################################################
!  Copyright (C)  1990, 1995, 1999, 2000 - All Rights Reserved
!  Regional Atmospheric Modeling System - RAMS
!  Mission Research Corporation / *ASTeR Division
!###########################################################################

subroutine landmks (xx1,xx2,yy1,yy2  &
                   ,idate,itime,ngd,ngrids &
                   ,icfile,nfile,dx,ztn,nxp,nyp,topo)

! Draws landmarks at the location indicated in the LANDMARKS file
! (which must be in the current runtime directory)# Limited to at most a 16 character identifier
! LANDMARKS file format (name can be up 16 characters):
!   name lat(deg min sec) lon(deg min sec) 
!   POU       41 37 32        -73 52 55
! OR
!   name    lat       lon
!   POU    41.62556  -73.88194
! Not all landmarks get plotted (if they are too close)
! The first in the list have plotting priority

!!!!!   Not any more !!!!! This version requires a Ralph II files

use pl_params
use pl_arrays
use define_colors
use gdf_input

implicit none

integer :: ngd,ngrids,icfile,nfile,idate,itime,nxp,nyp
real :: xx1,xx2,yy1,yy2,dx,ztn
real :: topo(nxp,nyp)

integer :: l,nums,ic,nt,ntok,lc,idraw,m,lcm  &
          ,nval,n,ng,ing,ic1,ic2
real :: xwl,xel,ysl,ynl,xwm,xem,ysm,ynm  &
       ,xsw,xne,ysw,yne,sx,sy,textoff
integer, parameter         :: numsta=1000,maxlevels=500
character(len=256)         :: fname,line
character(len=32)          :: tokens(20)
character(len=15)          :: curdate,name
character(len=1)           :: cng
integer, dimension(numsta) :: iplotlm
real, dimension(numsta)    :: xsta,ysta,alatd,alatm,alats,alond,alonm,alons
real, allocatable          :: locsh(:),locsv(:)
logical :: there

integer, parameter :: maxg=10
real :: swx,swy,nex,ney,dlx,dly,plat,plon,stx,sty,ztop,xista,yjsta,topom  &
         ,zloc,ueee,veee
integer :: npts(5),nv,iyr,imon,iday,ifile,ns,ierr,nlevels,ipixelx,ipixely

type obs_info_type
   character(len=8) :: id
   integer :: intid
   integer :: ntimes
   real :: xlat,xlon,xsta,ysta,stopo
   real :: temp, dewpt, us, vs, ps,u,v,time
end type   
   
type(obs_info_type) :: obs_info(numsta)

integer, parameter :: num_convars=2
character(len=8) :: varn(num_convars),ftype
data varn/'ue','ve'/
real :: vars(num_convars),usnd(maxlevels),vsnd(maxlevels),zsnd(maxlevels)



! Construct standard date string
iyr=idate/10000
imon=mod(idate,10000)/100
iday=mod(idate,100)
write (curdate,'(i4.4,a1,i2.2,a1,i2.2,a1,i4.4)')  &
  iyr,'-',imon,'-',iday,'-',itime/100
write(cng,'(i1)') ngd

fname=trim(landmark)//curdate
!fname=trim(landmark)

print*,'LLLLLLLLLANDMARKS fname:',trim(fname)


! Need to determine if this is an upper air file or surface file.

inquire(file=fname,exist=there)
if(.not.there) then
   print*,'landmarks: no file found for this time'
   print*,'landmarks: ',trim(fname)
   return
endif


ifile=1
header(ifile)%iun=31
open(unit=header(ifile)%iun,file=fname,status='old')

call gdf_file_type(header(ifile)%iun,ftype)

rewind(header(ifile)%iun)

print*,'+++++++++++',ftype

if( trim(ftype) == 'sfc') then
   call gdf_read_sfc_ver (ifile)
elseif( trim(ftype) == 'upa') then
   call gdf_read_upa_ver (ifile)
else
   print*,'landmarks: Unknown file type for file:',trim(fname)
   print*,'landmarks: No landmarks drawn'
   return
endif

print*,'landmarks: file type:',ftype

xsw=f(1)%xtn(1,f(1)%nvgr)
ysw=f(1)%ytn(1,f(1)%nvgr)
xne=f(1)%xtn(f(1)%nnxp(f(1)%nvgr)-1,f(1)%nvgr)
yne=f(1)%ytn(f(1)%nnyp(f(1)%nvgr)-1,f(1)%nvgr)
ztop=f(1)%zmn(f(1)%nnzp(f(1)%nvgr)-1,f(1)%nvgr)
plat=f(1)%platn(f(1)%nvgr)
plon=f(1)%plonn(f(1)%nvgr)
!print*,plon,nnxp,xsw,xne,delx
!print*,plat,nnyp,ysw,yne,dely

ns=0
do while (.TRUE.)
   if( trim(ftype) == 'sfc') then
      call gdf_read_sfc_obs (ifile,'no',ierr)
      if(ierr==1) exit
   
      call ll_xy (rsfc_obs%lat,rsfc_obs%lon,plat,plon,stx,sty)
   
      if (stx > xsw .and. stx < xne .and. sty > ysw .and. sty < yne) then
         !!!!!! should do time filter also
         ns=ns+1
         obs_info(ns)%id=rsfc_obs%id
         obs_info(ns)%xsta=stx
         obs_info(ns)%ysta=sty
         obs_info(ns)%xlat=rsfc_obs%lat
         obs_info(ns)%xlon=rsfc_obs%lon
         obs_info(ns)%stopo=rsfc_obs%elev


         !   Fill data arrays
      
         call gdf_sfc_data_convert(vars,varn,num_convars)
      
         obs_info(ns)%temp=rsfc_obs%t
         obs_info(ns)%dewpt=rsfc_obs%td
         obs_info(ns)%us=vars(1)
         obs_info(ns)%vs=vars(2)
         obs_info(ns)%ps=rsfc_obs%p
      
         obs_info(ns)%u=-999.
         obs_info(ns)%v=-999.
         if(vars(1) > -998. .and. vars(2) > -998.) then
            call uevetouv(obs_info(ns)%u  &
                        ,obs_info(ns)%v  &
                        ,vars(1),vars(2)  &
                        ,rsfc_obs%lat,rsfc_obs%lon,plat,plon)
         endif
      endif
   elseif ( trim(ftype) == 'upa') then
      call gdf_read_upa_obs (ifile,'no',ierr)
      if(ierr==1) exit
   
      call ll_xy (rupa_obs%lat,rupa_obs%lon,plat,plon,stx,sty)
   
      if (stx > xsw .and. stx < xne .and. sty > ysw .and. sty < yne) then
         !!!!!! should do time filter also
         ns=ns+1
         obs_info(ns)%id=rupa_obs%id
         obs_info(ns)%xsta=stx
         obs_info(ns)%ysta=sty
         obs_info(ns)%xlat=rupa_obs%lat
         obs_info(ns)%xlon=rupa_obs%lon
         obs_info(ns)%stopo=rupa_obs%elev
         
         ! Find plotted model height at obs location (assume dx=dy)
         xista=(stx-xsw)/dx
         yjsta=(sty-ysw)/dx
         call gdtost2(topo,nxp,nyp,xista,yjsta,topom)
         zloc=topom+ztn*(1.-topom/ztop)
         print*,'******:',rupa_obs%id,zloc,xista,yjsta,topom
         
         ! Interpolate from sounding to level being plotted
         
         call gdf_upa_get_profile(usnd,nlevels,'ue','z')
         call gdf_upa_get_profile(vsnd,nlevels,'ve','z')
         call gdf_upa_get_profile(zsnd,nlevels,'zz','z')
         call sound_interp(usnd,zsnd,nlevels,zloc,500.,ueee)
         call sound_interp(vsnd,zsnd,nlevels,zloc,500.,veee)
         if(ueee > -998. .and. veee > -998.) then
            call uevetouv(obs_info(ns)%u  &
                        ,obs_info(ns)%v  &
                        ,ueee,veee  &
                        ,rupa_obs%lat,rupa_obs%lon,plat,plon)
         endif
         
      endif         
         
   endif
enddo

close(header(ifile)%iun)
nums=ns
print*,'landmarks:',nums

ic1=31
ic2=32
call gsmksc(sizelandmk)     ! marker size
call gsmk(itypelandmk)      ! marker type
call coloralias_set (1,collandmk,ic1)
call coloralias_set (1,collandlab,ic2)

!if(lmarkers(1:4) == 'name') then
!elseif(lmarkers(1:4) == 'vect') then
   call gslwsc(2.)  ! line and label thickness
!endif

call gspmci(ic1)  ! marker colour
call gsplci(ic2)  ! label and line/vector colour
call gstxci(ic2)  ! label colour

open(97,file='LANDMARKS.g'//cng,status='unknown')
rewind(97)

do ns=1,nums
     
      call set(px1,px2,py1,py2,xx1,xx2,yy1,yy2,1)
      !print*,'xxyy:',xx1,xx2,yy1,yy2,px1,px2,py1,py2
      
      name=obs_info(ns)%id

      call maptra(obs_info(ns)%xlat,obs_info(ns)%xlon,xsta(ns),ysta(ns))
      
      !print*,'loc:',ns,obs_info(ns)%xlat,obs_info(ns)%xlon,xsta(ns),ysta(ns)
   
      if(xsta(ns).ge.xx1.and.xsta(ns).le.xx2.and.  &
         ysta(ns).ge.yy1.and.ysta(ns).le.yy2) then

         idraw=1
         
         
         !lc=len_trim(name)
         !xwl=xsta(ns)-bufflandmk*(xx2-xx1)
         !xel=xsta(ns)+bufflandmk*(xx2-xx1)*(lc+1)
         !ysl=ysta(ns)-bufflandmk*(yy2-yy1)
         !ynl=ysta(ns)+bufflandmk*(yy2-yy1)

         !do m=1,ns-1
         !   lcm=len_trim(name(m))
         !   call maptra(alatd(m),alond(m),xsta(m),ysta(m))
         !   xwm=xsta(m)-bufflandmk*(xx2-xx1)
         !   xem=xsta(m)+bufflandmk*(xx2-xx1)*(lcm+1)
         !   ysm=ysta(m)-bufflandmk*(yy2-yy1)
         !   ynm=ysta(m)+bufflandmk*(yy2-yy1)
         !   if(((xwm.le.xel.and.xel.le.xem).or.  &
         !       (xwm.le.xwl.and.xwl.le.xem)).and.  &
         !       ((ysm.le.ynl.and.ynl.le.ynm).or.  &
         !       (ysm.le.ysl.and.ysl.le.ynm)).and.  &
         !        iplotlm(m).eq.1) idraw=0
         !enddo

         if(idraw.eq.1) then
            !print*,'plotting:',ns,trim(name),xsta(ns),ysta(ns)

            call gpm (1,xsta(ns),ysta(ns))
            if(lmarkers(1:4) == 'name') then
               call plchlq (xsta(ns),ysta(ns),' '//trim(name)  &
                              ,sizelandlab,0.,-1.)
            elseif(lmarkers(1:4) == 'vect') then
               call wvector (obs_info(ns)%xsta,obs_info(ns)%ysta  &
                            ,obs_info(ns)%u,obs_info(ns)%v &
                            ,dx  &
                            ,px1,px2,py1,py2,xmin,xmax,ymin,ymax  &
                   ,stemleng,headleng0,headleng,headang,velomax)
                      !      ,6.,.6,.6,25.,5.)
            endif
            
            ! Compute image locate of polymarker, assuming 800x800 res.
            ipixelx=nint(800.*(px1+(px2-px1)*(xsta(ns)-xx1)/(xx2-xx1)))
            ipixely=800-nint(800.*(py1+(py2-py1)*(ysta(ns)-yy1)/(yy2-yy1)))
            write(97,101) ns,trim(name)  &
                        ,obs_info(ns)%xlat,obs_info(ns)%xlon  &
                        ,ipixelx,ipixely
            101 format(i4,a24,2f14.4,2i6)
         endif
      endif
   
   !endif
   
   
enddo

close(97)
call sflush()
      
return
end
