Module pl_arrays

use var_info
use ra_file_info

type fields
   real, pointer, dimension(:) :: arr,slab
   integer :: ivartype
   character(len=24) :: cdname,cdunits
end type

!---------------------

type scratch
   real, pointer, dimension(:) :: scr1,scr2
end type

!---------------------

integer, parameter :: maxplvar=2, maxvar=maxplvar+11, maxplgrids=10

type plarrays
   
   ! "vars" holds all fields that will possibly be used in a given plot
   !    The user-requested variables come first, then wind components, topo,
   !    pctlnd, etc.come after. These are identified by their variable id "iv",
   !    which is assigned in pl_comp_mem.  
   
   ! maxplvar is the max number of user variables, maxvar is the max total number
   
   type(fields) :: vars(maxvar),windu,windv,windw ,windtu,windtv,windtw ,volt             

   type(fields) :: topo,pctlnd,topta
   
   integer :: iv_u, iv_v, iv_w, iv_tu, iv_tv, iv_tw, iv_volt
   integer :: iv_topo, iv_topta, iv_pctlnd, iv_parea
   

   ! Analysis file coordinate info and options
   integer :: ifile, ngrids, nfiles, nzg, nzs, npatch, if_adap, ihtran
   integer, dimension(maxplgrids) :: nnxp,nnyp,nnzp
   real, dimension(maxplgrids) :: platn,plonn,deltaxn,deltayn
   real, pointer, dimension(:,:) :: xtn,ytn,ztn,xmn,ymn,zmn
   real, pointer, dimension(:) :: slz

   integer :: nvgr       ! Current grid number

   integer :: maxmem, maxslab, numvars

   character(len=128) :: flnm,flnm_base

end type


!---------------------

type(plarrays) :: f(1)

type(scratch) :: scr


Contains

!---------------------------------------------------------------------------

subroutine pl_fill_coords_rams(nf)

implicit none

integer :: nf

integer :: ipts(6),n,npt
integer :: maxx,maxy,maxz

logical, save :: first_call = .t.

   !  Get number of grids and grid points
call ra_get_idata(1,1,f(nf)%ngrids,npt)
call ra_get_idata(5,1,f(nf)%ihtran,npt)
call ra_get_idata(6,1,f(nf)%if_adap,npt)

do n=1,f(nf)%ngrids
   call ra_get_idata(0, n, ipts(1), npt)
   f(nf)%nnxp(n)=ipts(1)
   f(nf)%nnyp(n)=ipts(2)
   f(nf)%nnzp(n)=ipts(3)
   f(nf)%nzg=ipts(4)
   f(nf)%nzs=ipts(5)
   f(nf)%npatch=ipts(6)
enddo

! Allocate and fill coordinate arrays
maxx=maxval(f(nf)%nnxp(1:f(nf)%ngrids))
maxy=maxval(f(nf)%nnyp(1:f(nf)%ngrids))
maxz=maxval(f(nf)%nnzp(1:f(nf)%ngrids))

if(first_call) then
   first_call = .f.
   if (associated(f(nf)%xtn)) nullify(f(nf)%xtn)
   if (associated(f(nf)%ytn)) nullify(f(nf)%ytn)
   if (associated(f(nf)%ztn)) nullify(f(nf)%ztn)
   if (associated(f(nf)%xmn)) nullify(f(nf)%xmn)
   if (associated(f(nf)%ymn)) nullify(f(nf)%ymn)
   if (associated(f(nf)%zmn)) nullify(f(nf)%zmn)
   if (associated(f(nf)%slz)) nullify(f(nf)%slz)
endif

if(associated(f(nf)%xtn)) deallocate(f(nf)%xtn) 
allocate(f(nf)%xtn(maxx,f(nf)%ngrids))
if(associated(f(nf)%ytn)) deallocate(f(nf)%ytn) 
allocate(f(nf)%ytn(maxy,f(nf)%ngrids))
if(associated(f(nf)%ztn)) deallocate(f(nf)%ztn) 
allocate(f(nf)%ztn(maxz,f(nf)%ngrids))
if(associated(f(nf)%xmn)) deallocate(f(nf)%xmn) 
allocate(f(nf)%xmn(maxx,f(nf)%ngrids))
if(associated(f(nf)%ymn)) deallocate(f(nf)%ymn) 
allocate(f(nf)%ymn(maxy,f(nf)%ngrids))
if(associated(f(nf)%zmn)) deallocate(f(nf)%zmn) 
allocate(f(nf)%zmn(maxz,f(nf)%ngrids))
if(associated(f(nf)%slz)) deallocate(f(nf)%slz) 
allocate(f(nf)%slz(f(nf)%nzg))
do n=1,f(nf)%ngrids
   call ra_get_fdata(10,n,f(nf)%xtn(1,n),npt)
   call ra_get_fdata(11,n,f(nf)%xmn(1,n),npt)
   call ra_get_fdata(12,n,f(nf)%ytn(1,n),npt)
   call ra_get_fdata(13,n,f(nf)%ymn(1,n),npt)
   call ra_get_fdata( 0,n,f(nf)%ztn(1,n),npt)
   call ra_get_fdata(14,n,f(nf)%zmn(1,n),npt)
   call ra_get_fdata(15,n,f(nf)%slz(1),npt)
   call ra_get_fdata( 6,n,f(nf)%platn(n),npt)
   call ra_get_fdata( 7,n,f(nf)%plonn(n),npt)
   call ra_get_fdata( 4,n,f(nf)%deltaxn(n),npt)
   call ra_get_fdata( 5,n,f(nf)%deltayn(n),npt)
enddo

return
end subroutine

!---------------------------------------------------------------------------

subroutine pl_comp_mem(nf)

implicit none

integer :: nf

integer :: ng,maxxz

! Find memory sizes
f(nf)%maxmem=0
f(nf)%maxslab=0
do ng=1,f(nf)%ngrids
   maxxz = max(f(nf)%nnzp(ng),f(nf)%nzg*f(nf)%npatch,f(nf)%nzs*f(nf)%npatch)
   
   f(nf)%maxmem=max(f(nf)%maxmem,(f(nf)%nnxp(ng)+1)*(f(nf)%nnyp(ng)+1)*maxxz)
   
   f(nf)%maxslab=max(f(nf)%maxslab  &
               ,(f(nf)%nnxp(ng)+1)*(f(nf)%nnyp(ng)+1)*f(nf)%npatch  &
               ,(f(nf)%nnxp(ng)+1)*maxxz &
               ,(f(nf)%nnyp(ng)+1)*maxxz)
enddo


! Fill variable id numbers

f(nf)%numvars   = maxvar
f(nf)%iv_u      = maxplvar+1 
f(nf)%iv_v      = maxplvar+2 
f(nf)%iv_w      = maxplvar+3
f(nf)%iv_tu     = maxplvar+4 
f(nf)%iv_tv     = maxplvar+5 
f(nf)%iv_tw     = maxplvar+6
f(nf)%iv_volt   = maxplvar+7
f(nf)%iv_topo   = maxplvar+8 
f(nf)%iv_topta  = maxplvar+9 
f(nf)%iv_pctlnd = maxplvar+10
f(nf)%iv_parea  = maxplvar+11

return
end subroutine

!---------------------------------------------------------------------------

subroutine alloc_pl_arrays(nf)

implicit none

integer :: nf

integer :: nv

logical, save :: first_call = .true.


if(first_call) then
   first_call = .false.
   do nv=1,f(nf)%numvars
      if (associated(f(nf)%vars(nv)%arr)) nullify(f(nf)%vars(nv)%arr)
      if (associated(f(nf)%vars(nv)%slab)) nullify(f(nf)%vars(nv)%slab)
   enddo
   if (associated(scr%scr1)) nullify(scr%scr1)
   if (associated(scr%scr2)) nullify(scr%scr2)
endif


! Allocate a new set of arrays for a file

do nv=1,f(nf)%numvars
   if(associated(f(nf)%vars(nv)%arr))  deallocate(f(nf)%vars(nv)%arr)
                                        allocate(f(nf)%vars(nv)%arr(f(nf)%maxmem))
   if(associated(f(nf)%vars(nv)%slab)) deallocate(f(nf)%vars(nv)%slab)
                                        allocate(f(nf)%vars(nv)%slab(f(nf)%maxslab))
enddo


! Allocate scratch

if (associated(scr%scr1)) deallocate (scr%scr1)
                            allocate(scr%scr1(f(nf)%maxmem))
if (associated(scr%scr2)) deallocate (scr%scr2)
                            allocate(scr%scr2(f(nf)%maxmem))

print*,'------allocate:',f(nf)%maxmem,f(nf)%maxslab

return
end subroutine



End Module pl_arrays
