!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================


subroutine hyp_mem_alloc()

use mem_hyp

implicit none 

integer, pointer :: nmzp(:),nmxp(:),nmyp(:)
integer :: ng,nv,imean,ntpts

! First, depending on type of process, define grid point pointers correctly...

print*,' %% mem_alloc %%:'

!  This is the call for either a single processor run
nmzp => nnzh
nmxp => nnxh
nmyp => nnyh
call dealloc_all()


! Allocate met variables data type
print*,'start basic alloc'
allocate(met(ngrids))
do ng=1,ngrh
   call nullify_met(met(ng))
   call alloc_basic(met(ng),nmzp(ng),nmxp(ng),nmyp(ng),ng) 
enddo


! Allocate grid variables data type. 

print*,'start grid alloc'
allocate(grid_g(ngrids))
do ng=1,ngrh
   call nullify_grid(grid_g(ng))
   call alloc_grid(grid_g(ng),nmzp(ng),nmxp(ng),nmyp(ng),ng) 
enddo

! Allocate any added Scalar types 
! NOT ALLOWING DIFFERENT NUMBERS OF SCALARS ON DIFFERENT NESTS

print*,'start scalar alloc'
!   Allocate length 1 of these datatypes by default
allocate(scalar_g(1,ngrh))
if (naddsc > 0) then
   ! deallocate datatypes, then re-alloc to correct length
   deallocate(scalar_g,scalarm_g)
   allocate(scalar_g(naddsc,ngrids))
   do ng=1,ngrids
      call nullify_scalar(scalar_g(:,ng),naddsc)
      call alloc_scalar(scalar_g(:,ng),nmzp(ng),nmxp(ng),nmyp(ng)  &
                       ,naddsc)      
   enddo
endif

! Allocate Tendency data type,  filltab_tendency is responsible 
!   for filling the main atmospheric model variables in the scalar table,
!   so make sure to call any routines that define scalar variables first.

! Assuming same scalars on all grids!!!!!

print*,'start tendency alloc'
call nullify_tend(naddsc)
call alloc_tend(nmzp,nmxp,nmyp,ngrids,naddsc,proc_type)
do ng=1,ngrids
   call filltab_tend(basic_g(ng),micro_g(ng),turb_g(ng)  &
                    ,scalar_g(:,ng),naddsc,ng) 
enddo

! Allocate Scratch data type, This also fills the max's that are needed
!    by nesting stuff.

print*,'start scratch alloc'
call nullify_scratch()
call alloc_scratch(nmzp,nmxp,nmyp,nnzp,nnxp,nnyp,ngrids  &
                  ,nzg,nzs,npatch,proc_type,maxnxp,maxnyp,maxnzp)
call filltab_scratch() 

! Allocate nested boundary interpolation arrays. All grids will be allocated.

print*,'start nestb alloc'
if (proc_type == 0 .or. proc_type == 2) then
   do ng=1,ngrids
      if(nxtnest(ng) == 0 ) then
         call alloc_nestb(ng,1,1,1)
      else
         call alloc_nestb(ng,nnxp(ng),nnyp(ng),nnzp(ng))
      endif
   enddo
endif

! Set "Lite" variable flags according to namelist input LITE_VARS.

if (proc_type == 0 .or. proc_type == 2) then
   call lite_varset()
endif


! Set ALL variables in the vtab_r variable table to zero by default. These
!  are variables processed in the filltab_* routines with a call to vtables2.
!  This does NOT include scratch arrays, tendencies, or mean arrays.

do ng = 1, ngrids
   do nv = 1,num_var(ng)
      call azero( vtab_r(nv,ng)%npts, vtab_r(nv,ng)%var_p)
   enddo
enddo

print*,'end alloc'

return
end subroutine

!---------------------------------------------------------------------

subroutine dealloc_all()

use mem_all

implicit none

! deallocate all model memory.  Used on dynamic balance

integer :: ng


call dealloc_tend(naddsc)
call dealloc_scratch()

do ng=1,ngrids
   call dealloc_basic(basic_g(ng)) 
   call dealloc_grid(grid_g(ng))    
 enddo
deallocate(basic_g)
deallocate(grid_g)

return
end
