!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine cluster ()

use hyp_coms

implicit none

real, allocatable :: xpc(:,:),xpc2(:,:),ypc(:,:),ypc2(:,:),zpc(:),zpc2(:)
integer, allocatable :: npc(:),imove(:)
real :: atp11,atp12,atp13,atp14,htopo,vol

integer :: ihp,jhp,khp,mgp,ipc1,ipc2,i,l,ipc,j,nnp,m
real :: xmean,ymean,zmean,sigmax,sigmay,sigmaz,aincr,hyp,ang,x,y

real :: pi=3.1415927

if(npart.eq.0.or.ncluster.eq.0) return

allocate (npc(ncluster),imove(ncluster)  &
         ,xpc(ncluster,18),xpc2(ncluster,18)  &
         ,ypc(ncluster,18),ypc2(ncluster,18)  &
         ,zpc(ncluster),zpc2(ncluster))

ipc1=1000000
ipc2=-ipc1

npc(1:ncluster)=0
imove(1:ncluster)=0
xpc(1:ncluster,1:18)=0.
xpc2(1:ncluster,1:18)=0.
ypc(1:ncluster,1:18)=0.
ypc2(1:ncluster,1:18)=0.
zpc(1:ncluster)=0.
zpc2(1:ncluster)=0.

! loop over particles to calculate cluster characteristics

do l=1,npart

   ipc=atp(l)%ncluster
   if(ipc.eq.0.or.ncfill(ipc).eq.0) cycle
      
   ! get particle locations
   call part_loc (atp(l)%hi,atp(l)%hj,atp(l)%hk,nih,njh,nkh,hg%mg,hg%topthg  &
                 ,ihp,jhp,khp,htopo,atp11,atp12,atp13,atp14,mgp)

   ! extend range of clusters
   ipc1=min(ipc,ipc1)
   ipc2=max(ipc,ipc2)
   
   ! number in clusters
   npc(ipc)=npc(ipc)+1

   !increment around 90 degrees
   do m=1,18
      aincr=float(m-1)*5.
   
      ! calc x and y
      hyp=sqrt(atp(l)%hi**2+atp(l)%hj**2)
      ang=atan(atp(l)%hi/atp(l)%hj)+aincr*pi/180.
      x=sin(ang)*hyp
      y=cos(ang)*hyp
      
      ! prepare horizontal stats
      xpc(ipc,m)=xpc(ipc,m)+x
      xpc2(ipc,m)=xpc2(ipc,m)+x**2
      ypc(ipc,m)=ypc(ipc,m)+y
      ypc2(ipc,m)=ypc2(ipc,m)+y**2
      
   enddo
      
   ! prepare vertical stats
   if(int(hybvert).ge.0) then
      zpc(ipc)=zpc(ipc)+atp(l)%hk
      zpc2(ipc)=zpc2(ipc)+atp(l)%hk**2
   else
      zpc(ipc)=zpc(ipc)+atp13
      zpc2(ipc)=zpc2(ipc)+atp13**2
   endif
         
enddo

if(ipc2.lt.ipc1) return

! loop over range of avtive clusters to calculate sigmax/y for each
! cluster and to check cluster maturity

do j=ipc1,ipc2

   nnp=npc(j)
   if(nnp.eq.0.or.ncfill(j).eq.0) cycle
   
   ! move to concs if too few parts
   if(nnp.lt.ihybmin) then
      imove(j)=1
      cycle
   endif
   
   ! get max horizontal sigmas
   sigmax=1.e20
   sigmay=1.e20
   do m=1,18
      xmean=xpc(j,m)/nnp
      ymean=ypc(j,m)/nnp
      sigmax=min(sigmax,sqrt(max(0.,xpc2(j,m)/nnp-xmean**2)))
      sigmay=min(sigmay,sqrt(max(0.,ypc2(j,m)/nnp-ymean**2)))
   enddo
   
   ! get vertical sigma
   zmean=zpc(j)/nnp
   sigmaz=sqrt(max(0.,zpc2(j)/nnp-zmean**2))

   ! move to concs if all 6*sigma's > criteria
   if(6*sigmax.ge.hybhoriz.and.6*sigmay.ge.hybhoriz.and.  &
      6*sigmaz.ge.hybvert) imove(j)=1
   
enddo

! loop over particles again to move particles from the marked clusters
! to Eulerian emission or concentration field

do l=1,npart
   if(atp(l)%ncluster /= 0) then
      if(imove(atp(l)%ncluster) == 1) then
   
         call lag_to_eul (atp(l)%hi,atp(l)%hj,atp(l)%hk,atp(l)%mass  &
                      ,-1,species(atp(l)%nsource),1)
                      
         atp(l)%ncluster=0
      endif
   endif
enddo

deallocate (npc,imove,xpc,xpc2,ypc,ypc2,zpc,zpc2)

return
end
