!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine emissions ()

use hyp_coms
use mem_euler

implicit none

integer, parameter :: nzef=1000,nb=1
integer, save :: idum_seed,ncall=0
integer, dimension(12) :: nrib,nrjb,nrkb,nrandb
integer, dimension(maxem) :: iwarnx=0,iwarny=0,iwarnz=0,ibwarn=0
real :: pii=3.14159265359
real, dimension(maxhk) :: ustak,thetstak,tlwstak,zhrtg,zstk,ustk,tpstk,tlstk
real, dimension(nzef) :: zef,wb
real, dimension(maxem) :: relcnt
real, dimension(polypts,maxem) :: relcntp
real :: pi180,sc1,sc3,sc4,denslp,duration,xdisp,ramran,ydisp,zdisp  &
       ,radius,hifromx,hkfromz,atp1,atp2,atp3,htopo,atp11,atp12,atp13  &
       ,atp14,vol,x1,x2,x3,angle,y1,y2,y3,y4,a1,a2,fa1,fa2,ri,rj,xp  &
       ,yp1,yp2,yp,rpos,azim,atp4
integer :: nsrc,ip,ng,ns,n,npartstart,nrelcnt,irelcnt,ierr,ihp,jhp  &
          ,khp,mgp,ngp,imp,jmp,kmp,indm,nrelcntp,i,j,k,nsp,latp2  &
          ,nnrelcnt,nnrelcntp

if(ncall == 0) then
   ncall=1
   pi180=pii/180.
   do nsrc=1,nsources
      ncpart(nsrc)=ihybpart
      relcnt(nsrc)=0.
      do ip=1,polypts
         relcntp(ip,nsrc)=0.
      enddo
      if(sshape(nsrc)(1:3).eq.'pol') then
         do ip=1,npolypts(nsrc)
            ncpartp(ip,nsrc)=ihybpart
         enddo
      endif
   enddo
endif

do nsrc=1,nsources
 if(hytime >= relstrt(nsrc).and.hytime < relend(nsrc)) then

   npartstart=npart

   sc1=szpwr(nsrc)
   sc3=szmin(nsrc)**(1./sc1)
   sc4=szmax(nsrc)**(1./sc1)
   denslp=2.0

   ! RECTANGLULAR SOURCE

   if(sshape(nsrc)(1:3).eq.'rec') then

      ! account for portion of a part left over from last timestep and
      ! portion of a timestep until completion of emission

      if(relend(nsrc).eq.relstrt(nsrc)) then ! inst. emission
         relcnt(nsrc)=rparts(nsrc)
      else
         duration=min(hytime+abs(dtpart),relend(nsrc))  &
                 -max(hytime,relstrt(nsrc))
         relcnt(nsrc)=relcnt(nsrc)+rparts(nsrc)*duration/abs(dtpart)
      endif
      nrelcnt=int(relcnt(nsrc))
      relcnt(nsrc)=relcnt(nsrc)-float(nrelcnt)

      nnrelcnt=nrelcnt
      do irelcnt=1,nrelcnt
         nrelpart=nrelpart+1
         npart=npart+1

         ! random particle placement within source

         xdisp=xsize(nsrc)*(ramran(idum_seed)-0.5)
         ydisp=ysize(nsrc)*(ramran(idum_seed)-0.5)
         zdisp=zsize(nsrc)*(ramran(idum_seed)-0.5)
         radius=sqrt(xdisp**2+ydisp**2)
         if (xsize(nsrc).eq.0..and.ysize(nsrc).eq.0.) then
            angle=0.
         else
            angle=atan2(ydisp,xdisp)
         endif
         angle=angle-rotation(nsrc)*pi180
         xdisp=radius*cos(angle)
         ydisp=radius*sin(angle)
         if(xy(1,nsrc)+xdisp < xh(1) .or. xy(1,nsrc)+xdisp > xh(nih) .or.  &
            xy(2,nsrc)+ydisp < yh(1) .or. xy(2,nsrc)+ydisp > yh(njh) .or.  &
            srcz(nsrc)+zdisp < zh(1) .or. srcz(nsrc)+zdisp > zh(nkh)) then
            if(ibwarn(nsrc) == 0)  &
               print*,'WARNING: source partially out of outer grid',nsrc
            ibwarn(nsrc)=ibwarn(nsrc)+1
            nrelpart=nrelpart-1
            npart=npart-1
            nnrelcnt=nnrelcnt-1
            cycle
         endif
         atp1=hifromx (nih,xh,xy(1,nsrc)+xdisp)
         atp2=hifromx (njh,yh,xy(2,nsrc)+ydisp)
         !atp4=hifromx (nkh,zh,srcz(nsrc)+zdisp) ! sigma-z height
         atp3=hkfromz (nkh,zh,srcz(nsrc)+zdisp) ! sigma-z height
         !print*,atp3,atp4

         ! assign particle attributes
         
         call process_lp(nsrc,npart,atp1,atp2,atp3,irelcnt,nnrelcnt  &
                     ,denslp,sc1,sc3,sc4,idum_seed)

         ! add Eularian source to concentration field
         
         if(type(nsrc)(1:3) == 'eul'.or.type(nsrc)(1:3) == 'bot')  &
            call lag_to_eul (atp1,atp2,atp3,ratio(nsrc)  &
                            ,-1,species(nsrc),1)

      enddo

  ! TRIANGULAR SOURCE

   elseif(sshape(nsrc)(1:3).eq.'tri') then

      x1=xy(1,nsrc)
      x2=xy(3,nsrc)
      x3=xy(5,nsrc)
      y1=xy(2,nsrc)
      y2=xy(4,nsrc)
      y3=xy(6,nsrc)

      y4=y1+(y3-y1)*(x2-x1)/(x3-x1)
      a1=0.5*(x2-x1)*abs(y4-y2)
      a2=0.5*(x3-x2)*abs(y4-y2)
      fa1=max(1.e-10,a1/(a1+a2))
      fa2=max(1.e-10,1.-fa1)

      ! account for portion of a part left over from last timestep and
      ! portion of a timestep until completion of emission

      if(relend(nsrc).eq.relstrt(nsrc)) then ! inst. emission
         relcnt(nsrc)=rparts(nsrc)
      else
         duration=min(hytime+abs(dtpart),relend(nsrc))  &
                 -max(hytime,relstrt(nsrc))
         relcnt(nsrc)=relcnt(nsrc)+rparts(nsrc)*duration/abs(dtpart)
      endif
      nrelcnt=int(relcnt(nsrc))
      relcnt(nsrc)=relcnt(nsrc)-float(nrelcnt)

      nnrelcnt=nrelcnt
      do irelcnt=1,nrelcnt
         nrelpart=nrelpart+1
         npart=npart+1

         ! random particle placement within source

         ri=ramran(idum_seed)
         rj=ramran(idum_seed)
         if(ri.le.fa1) then
            xp=x1+(x2-x1)*sqrt(ri/fa1)
            yp1=y1+(y2-y1)*(xp-x1)/(x2-x1)
            yp2=y1+(y3-y1)*(xp-x1)/(x3-x1)
         else
            xp=x3+(x2-x3)*sqrt((1.-ri)/fa2)
            yp1=y3+(y1-y3)*(xp-x3)/(x1-x3)
            yp2=y3+(y2-y3)*(xp-x3)/(x2-x3)
         endif
         yp=yp1+(yp2-yp1)*rj
         zdisp=zsize(nsrc)*(ramran(idum_seed)-0.5)
         if(xy(1,nsrc)+xdisp < xh(1) .or. xy(1,nsrc)+xdisp > xh(nih) .or.  &
            xy(2,nsrc)+ydisp < yh(1) .or. xy(2,nsrc)+ydisp > yh(njh) .or.  &
            srcz(nsrc)+zdisp < zh(1) .or. srcz(nsrc)+zdisp > zh(nkh)) then
            if(ibwarn(nsrc) == 0)  &
               print*,'WARNING: source partially out of outer grid',nsrc
            ibwarn(nsrc)=ibwarn(nsrc)+1
            nrelpart=nrelpart-1
            npart=npart-1
            nnrelcnt=nnrelcnt-1
            cycle
         endif
         atp1=hifromx (nih,xh,xy(1,nsrc)+xdisp)
         atp2=hifromx (njh,yh,xy(2,nsrc)+ydisp)
         atp3=hkfromz (nkh,zh,srcz(nsrc)+zdisp) ! sigma-z height

         ! assign particle attributes

         call process_lp(nsrc,npart,atp1,atp2,atp3,irelcnt,nnrelcnt  &
                     ,denslp,sc1,sc3,sc4,idum_seed)

          ! add Eularian source to concentration field
 
         if(type(nsrc)(1:3).eq.'eul'.or.type(nsrc)(1:3).eq.'bot')  &
            call lag_to_eul (atp1,atp2,atp3,ratio(nsrc)  &
                            ,-1,species(nsrc),1)
         
      enddo

   ! IRREGULAR POLYGON SOURCE

   elseif(sshape(nsrc)(1:3).eq.'pol') then

      nrelcnt=0
      do ip=1,npolypts(nsrc)

         x1=xy(6*(ip-1)+1,nsrc)
         x2=xy(6*(ip-1)+3,nsrc)
         x3=xy(6*(ip-1)+5,nsrc)
         y1=xy(6*(ip-1)+2,nsrc)
         y2=xy(6*(ip-1)+4,nsrc)
         y3=xy(6*(ip-1)+6,nsrc)

         y4=y1+(y3-y1)*(x2-x1)/(x3-x1)
         a1=0.5*(x2-x1)*abs(y4-y2)
         a2=0.5*(x3-x2)*abs(y4-y2)
         fa1=max(1.e-10,a1/(a1+a2))
         fa2=max(1.e-10,1.-fa1)

         ! account for portion of a part left over from last timestep and
         ! portion of a timestep until completion of emission

         if(relend(nsrc).eq.relstrt(nsrc)) then ! inst. emission
            relcntp(ip,nsrc)=rpartsp(ip,nsrc)
         else
            duration=min(hytime+abs(dtpart),relend(nsrc))  &
                    -max(hytime,relstrt(nsrc))
            relcntp(ip,nsrc)=relcntp(ip,nsrc)+rpartsp(ip,nsrc)*duration/abs(dtpart)
         endif
         nrelcntp=int(relcntp(ip,nsrc))
         nrelcnt=nrelcnt+nrelcntp
         relcntp(ip,nsrc)=relcntp(ip,nsrc)-float(nrelcntp)

         nnrelcntp=nrelcntp
         do irelcnt=1,nrelcntp
            nrelpart=nrelpart+1
            npart=npart+1

            ! random particle placement within source

            ri=ramran(idum_seed)
            rj=ramran(idum_seed)
            if(ri.le.fa1)then
               xp=x1+(x2-x1)*sqrt(ri/fa1)
               yp1=y1+(y2-y1)*(xp-x1)/(x2-x1)
               yp2=y1+(y3-y1)*(xp-x1)/(x3-x1)
            else
               xp=x3+(x2-x3)*sqrt((1.-ri)/fa2)
               yp1=y3+(y1-y3)*(xp-x3)/(x1-x3)
               yp2=y3+(y2-y3)*(xp-x3)/(x2-x3)
            endif
            yp=yp1+(yp2-yp1)*rj
            zdisp=zsize(nsrc)*(ramran(idum_seed)-0.5)
            if(xy(1,nsrc)+xdisp < xh(1) .or. xy(1,nsrc)+xdisp > xh(nih) .or.  &
               xy(2,nsrc)+ydisp < yh(1) .or. xy(2,nsrc)+ydisp > yh(njh) .or.  &
               srcz(nsrc)+zdisp < zh(1) .or. srcz(nsrc)+zdisp > zh(nkh)) then
               if(ibwarn(nsrc) == 0)  &
                  print*,'WARNING: source partially out of outer grid',nsrc
               ibwarn(nsrc)=ibwarn(nsrc)+1
               nrelpart=nrelpart-1
               npart=npart-1
               nnrelcntp=nnrelcntp-1
               cycle
            endif
            atp1=hifromx (nih,xh,xy(1,nsrc)+xdisp)
            atp2=hifromx (njh,yh,xy(2,nsrc)+ydisp)
            atp3=hkfromz (nkh,zh,srcz(nsrc)+zdisp) ! sigma-z height

            ! assign particle attributes - this needs more checking

!            if(type(nsrc)(1:3).eq.'lag'.or.  &
!               type(nsrc)(1:3).eq.'hyb'.or.  &
!               type(nsrc)(1:3).eq.'bot') then
!
            call process_lp(nsrc,npart,atp1,atp2,atp3,irelcnt,nnrelcnt  &
                     ,denslp,sc1,sc3,sc4,idum_seed)

             ! add Eularian source to concentration field
 
            if(type(nsrc)(1:3).eq.'eul'.or.type(nsrc)(1:3).eq.'bot')  &
               call lag_to_eul (atp1,atp2,atp3,ratio(nsrc)  &
                            ,-1,species(nsrc),1)
            
         enddo
      enddo

   ! ELLIPTICAL SOURCE

   elseif(sshape(nsrc)(1:3).eq.'ell') then

      ! account for portion of a part left over from last timestep and
      ! portion of a timestep until completion of emission

      if(relend(nsrc).eq.relstrt(nsrc)) then ! inst. emission
         relcnt(nsrc)=rparts(nsrc)
      else
         duration=min(hytime+abs(dtpart),relend(nsrc))  &
                 -max(hytime,relstrt(nsrc))
         relcnt(nsrc)=relcnt(nsrc)+rparts(nsrc)*duration/abs(dtpart)
      endif
      nrelcnt=int(relcnt(nsrc))
      relcnt(nsrc)=relcnt(nsrc)-float(nrelcnt)

      nnrelcnt=nrelcnt
      do irelcnt=1,nrelcnt
         nrelpart=nrelpart+1
         npart=npart+1

         ! random particle placement within source

         rpos=sqrt(ramran(idum_seed))
         azim=6.2831853*ramran(idum_seed)
         xdisp=rpos*cos(azim)*xsize(nsrc)/2.
         ydisp=rpos*sin(azim)*ysize(nsrc)/2.
         radius=sqrt(xdisp**2+ydisp**2)
         if (xsize(nsrc).eq.0..and.ysize(nsrc).eq.0.) then
            angle=0.
         else
            angle=atan2(ydisp,xdisp)
         endif
         angle=angle-rotation(nsrc)*pi180
         xdisp=radius*cos(angle)
         ydisp=radius*sin(angle)
         zdisp=zsize(nsrc)*(ramran(idum_seed)-0.5)
         if(xy(1,nsrc)+xdisp < xh(1) .or. xy(1,nsrc)+xdisp > xh(nih) .or.  &
            xy(2,nsrc)+ydisp < yh(1) .or. xy(2,nsrc)+ydisp > yh(njh) .or.  &
            srcz(nsrc)+zdisp < zh(1) .or. srcz(nsrc)+zdisp > zh(nkh)) then
            if(ibwarn(nsrc) == 0)  &
               print*,'WARNING: source partially out of outer grid',nsrc
            ibwarn(nsrc)=ibwarn(nsrc)+1
            nrelpart=nrelpart-1
            npart=npart-1
            nnrelcnt=nnrelcnt-1
            cycle
         endif
         atp1=hifromx (nih,xh,xy(1,nsrc)+xdisp)
         atp2=hifromx (njh,yh,xy(2,nsrc)+ydisp)
         atp3=hkfromz (nkh,zh,srcz(nsrc)+zdisp) ! sigma-z height

         ! assign particle attributes

         call process_lp(nsrc,npart,atp1,atp2,atp3,irelcnt,nnrelcnt  &
                     ,denslp,sc1,sc3,sc4,idum_seed)

          ! add Eularian source to concentration field
 
         if(type(nsrc)(1:3).eq.'eul'.or.type(nsrc)(1:3).eq.'bot')  &
            call lag_to_eul (atp1,atp2,atp3,ratio(nsrc)  &
                            ,-1,species(nsrc),1)
         
      enddo

   ! GRIDDED SOURCE - not finished yet

   elseif(sshape(nsrc)(1:4).eq.'grid') then

      do ng=1,ngrh
         do j=1,nnyh(ng)
            do i=1,nnxh(ng)
               do k=1,nnzh(ng)
                  eul(species(nsrc),ng)%spec(k,i,j)=emission(nsrc)
               enddo
            enddo
         enddo
      enddo
   endif

   ! don't want to add these particles to total if eul only

   if(type(nsrc)(1:3).eq.'eul') npart=npartstart

      write(*,200) nsrc,'  source:',psource(nsrc),'   ',srcname(nsrc)  &
               ,'  species:',pspecies(species(nsrc)),'   '  &
               ,specname(species(nsrc))  &
               ,'  parts used:',nrelcnt  &
               ,'  added parts:',npart-npartstart  &
               ,'  total parts:',npart
      200 format(i4,a,i4,a,a10,a,i4,a,a10,a,i5,a,i5,a,i7)

   elseif(type(nsrc)(1:3).eq.'hyb') then
      ncfill(ncclust(nsrc))=1
   endif

enddo

return
end

!------------------------------------------------------------------

subroutine process_lp(nsrc,npartt,atp1,atp2,atp3,irelcnt,nnrelcnt  &
                     ,denslp,sc1,sc3,sc4,idum_seed)

use hyp_coms

implicit none

integer :: nsrc,npartt,irelcnt,nnrelcnt,idum_seed
real :: atp1,atp2,atp3,denslp,sc1,sc3,sc4

real, external :: ramran
integer :: latp2,ihp,jhp,khp

! assign particle attributes

if(ilagran == 1) then

   atp(npartt)%hi=atp1
   atp(npartt)%hj=atp2
   atp(npartt)%hk=atp3
   atp(npartt)%nrelpart=nrelpart
   if(type(nsrc)(1:3).eq.'hyb') then
      if(ncpart(nsrc).eq.ihybpart) then
         ncpart(nsrc)=1
         ncluster=ncluster+1
         ncclust(nsrc)=ncluster
      else
         ncpart(nsrc)=ncpart(nsrc)+1
      endif
      atp(npartt)%ncluster=ncclust(nsrc)
      if(ncpart(nsrc)==ihybpart.or.irelcnt==nnrelcnt)  &
         ncfill(ncluster)=1
   else
      atp(npartt)%ncluster=1
   endif
   atp(npartt)%nsource=nsrc
   atp(npartt)%nspecies=species(nsrc)
   atp(npartt)%reltime=hytime
   if(ihfall(nsrc).eq.1) then
      atp(npartt)%fallvel=-4.e7*denslp*(sc3+(sc4-sc3)  &
                  *ramran(idum_seed))**(2.*sc1)
   else
      atp(npartt)%fallvel=0.
   endif
   atp(npartt)%upp=0.
   atp(npartt)%vpp=0.
   atp(npartt)%wpp=0.
   atp(npartt)%mass=ratio(nsrc)
   atp(npartt)%ppm=0.

endif

! check to see if particle within grid bounds

latp2=1
ihp=int(atp1)
jhp=int(atp2)
khp=int(atp3)
if(ihp.lt.1.or.jhp.lt.1.or.khp.lt.1.or.  &
   ihp.ge.nih.or.jhp.ge.njh.or.khp.ge.nkh) then
   latp2=0
   if(ilagran == 1) atp(npartt)%ncluster=latp2
   if(iwarn==1) print*,'warning - emissions outside domain',nsrc
   return
endif


return
end
