!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine es_scsets (ng)

use hyp_coms
use mem_grid
use mem_met
use mem_euler

implicit none

integer :: ng

integer :: n

! apply lateral, top, and bottom boundary conditions

do n=1,nspecies

   if(ng == 1)  &
      call latset (nnzh(ng),nnxh(ng),nnyh(ng)  &
                  ,eul(n,ng)%spec(1,1,1),metc(ng)%u,metc(ng)%v  &
                  ,grid(ng)%dxu,grid(ng)%dxm,grid(ng)%dyv  &
                  ,grid(ng)%dym,dtpart)
                  
   call es_bottop (nnzh(ng),nnxh(ng),nnyh(ng)  &
                  ,eul(n,ng)%spec(1,1,1),nnsttop(ng),nnstbot(ng),dzmh(1,ng))
                  
   call es_noneg (nnzh(ng),nnxh(ng),nnyh(ng),eul(n,ng)%spec(1,1,1),ng)
   
enddo

return
end

!***********************************************************************

subroutine es_noneg (n1,n2,n3,es,ng)

dimension es(n1,n2,n3)

do j=1,n3
   do i=1,n2
      do k=1,n1
         es(k,i,j)=max(0.,es(k,i,j))
      enddo
   enddo
enddo

return
end

!***********************************************************************

subroutine latset (n1,n2,n3,ap,uc,vc,dxu,dxm,dyv,dym,dtlt)

dimension ap(n1,n2,n3),uc(n1,n2,n3),vc(n1,n2,n3),dxu(n2,n3)  &
         ,dxm(n2,n3),dyv(n2,n3),dym(n2,n3)
dimension vctr17(100),vctr18(100)

thresh=0.
dtlx=dtlt
lsflg=0

! western and eastern boundaries for zero gradient option

if(lsflg == 0) then
   do j=1,n3
      do k=1,n1-1
         ap(k,1,j)=ap(k,2,j)
         ap(k,n2,j)=ap(k,n2-1,j)
      enddo
   enddo
else

! western boundary for lsflg = 1 or 2

   do j=1,n3
      dxr=dxu(2,j)/dxu(1,j)
      c1=dtlx*dxu(1,j)
      do k=1,n1-1
         vctr17(k)=-c1*uc(k,1,j)
      enddo
      do k=1,n1-1
         vctr18(k)=ap(k,2,j)+dxr*(ap(k,2,j)-ap(k,3,j))
      enddo
      do k=1,n1-1
         if(vctr17(k) >= thresh) then
            ap(k,1,j)=vctr18(k)
         elseif(lsflg == 1) then
            ap(k,1,j)=ap(k,2,j)
         endif
      enddo
   enddo

! eastern boundary for lsflg = 1 or 2

   do j=1,n3
      dxr=dxu(n2-2,j)/dxu(n2-1,j)
      c1=dtlx*dxu(n2-1,j)
      do k=1,n1-1
         vctr17(k)=c1*uc(k,n2-1,j)
      enddo
      do k=1,n1-1
         vctr18(k)=ap(k,n2-1,j)+dxr*(ap(k,n2-1,j)-ap(k,n2-2,j))
      enddo
      do k=1,n1-1
         if(vctr17(k) >= thresh) then
            ap(k,n2,j)=vctr18(k)
         elseif(lsflg == 1) then
            ap(k,n2,j)=ap(k,n2-1,j)
         endif
      enddo
   enddo
endif

! southern and northern boundaries for zero gradient option

if(lsflg==0) then
   do i=1,n2
      do k=1,n1-1
         ap(k,i,1)=ap(k,i,2)
         ap(k,i,n3)=ap(k,i,n3-1)
     enddo
  enddo
else

! southern boundary for lsflg = 1 or 2

   do i=1,n2
      dyr=dyv(i,2)/dyv(i,1)
      c1=dtlx*dyv(i,1)
      do k=1,n1-1
         vctr17(k)=-c1*vc(k,i,1)
      enddo
      do k=1,n1-1
         vctr18(k)=ap(k,i,2)+dyr*(ap(k,i,2)-ap(k,i,3))
      enddo
      do k=1,n1-1
         if(vctr17(k) >= thresh) then
            ap(k,i,1)=vctr18(k)
         elseif(lsflg == 1) then
            ap(k,i,1)=ap(k,i,2)
         endif
      enddo
   enddo

! northern boundary for lsflg = 1 or 2

   do i=1,n2
      dyr=dyv(i,n3-2)/dyv(i,n3-1)
      c1=dtlx*dyv(i,n3-1)
      do k=1,n1-1
         vctr17(k)=c1*vc(k,i,n3-1)
      enddo
      do k=1,n1-1
         vctr18(k)=ap(k,i,n3-1)+dyr*(ap(k,i,n3-1)-ap(k,i,n3-2))
      enddo
      do k=1,n1-1
         if(vctr17(k) >= thresh) then
            ap(k,i,n3)=vctr18(k)
         elseif(lsflg == 1) then
            ap(k,i,n3)=ap(k,i,n3-1)
         endif
      enddo
   enddo
endif

return
end

!***********************************************************************

subroutine es_bottop (n1,n2,n3,ap,itop,ibot,dzm)

dimension ap(n1,n2,n3),dzm(*)

if(itop == 1) then
   dzmr=dzm(n1-2)/dzm(n1-1)
   do j=1,n3
      do i=1,n2
         ap(n1,i,j)=max(0.,ap(n1-1,i,j)+dzmr*(ap(n1-1,i,j)-ap(n1-2,i,j)))
      enddo
   enddo
endif

if(ibot == 1) then
   do i=1,n2
      do j=1,n3
         ap(1,i,j)=ap(2,i,j)
      enddo
   enddo
endif

return
end

!***********************************************************************

subroutine es_nstbdriv (ng,isstp)

use hyp_coms
use mem_euler

implicit none

integer :: ng,isstp

integer :: n

do n=1,nspecies
   call nstbtnd (nnzh(ng),nnxh(ng),nnyh(ng),eul(n,ng)%spec(1,1,1)  &
                ,eb(ibsxh(n,ng)),eb(ibsyh(n,ng)),eb(ibszh(n,ng))  &
                ,nnsttop(ng),nnstbot(ng),nndtrat(ng),dtpart,isstp)
enddo

return
end

!***********************************************************************

subroutine nstbtnd (n1,n2,n3,scp,bx,by,bz,nsttop,nstbot,ndtrat,dtlx,isstp)

dimension scp(n1,n2,n3),bx(n1,n3,2),by(n1,n2,2),bz(n2,n3,2)

jdim=1
if(n3 == 1) jdim=0
nzfm=n1
nxfm=n2
nyfm=n3
fracupd = dtlx/(dtlx*float(ndtrat+1-isstp))

if(nstbot == 0) then
  do j=1,nyfm
    do i=1,nxfm
      scp(1,i,j)=scp(i,j,1)+fracupd*(bz(i,j,1)-scp(1,i,j))
    enddo
  enddo
endif

if(nsttop == 0) then
  do j=1,nyfm
    do i=1,nxfm
      scp(nzfm,i,j)=scp(nzfm,i,j)+fracupd*(bz(i,j,2)-scp(nzfm,i,j))
    enddo
  enddo
endif

do j=1,nyfm
  do k=1,nzfm
    scp(k,1,j)=scp(k,1,j)+fracupd*(bx(k,j,1)-scp(k,1,j))
    scp(k,nxfm,j)=scp(k,nxfm,j)+fracupd*(bx(k,j,2)-scp(k,nxfm,j))
  enddo
enddo

if(jdim == 1) then
  do i=1,nxfm
    do k=1,nzfm
      scp(k,i,1)=scp(k,i,1)+fracupd*(by(k,i,1)-scp(k,i,1))
      scp(k,i,nyfm)=scp(k,i,nyfm)+fracupd*(by(k,i,2)-scp(k,i,nyfm))
    enddo
  enddo
endif

return
end

!***********************************************************************

subroutine es_prgintrp (ng)

use hyp_coms
use mem_euler

implicit none

integer :: ng

integer :: ifm,icm,n

ifm=ng
icm=nxtnest(ifm)

do n=1,nspecies
   call fmint3 (nnzh(ifm),nnxh(ifm),nnyh(ifm)  &
               ,eul(n,icm)%spec(1,1,1),eul(n,ifm)%spec(1,1,1)  &
               ,ifm,icm ,eb(ibsxh(n,ifm)),eb(ibsyh(n,ifm))  &
               ,eb(ibszh(n,ifm)),nnsttop(ng),nnstbot(ng))
enddo

return
end

!***********************************************************************

subroutine fmint3 (n1,n2,n3,var1,var2,ifm,icm,bx,by,bz,itop,ibot)

use hyp_coms
use mem_scratch

implicit none

integer :: n1,n2,n3,ifm,icm,itop,ibot
real :: var1(*),var2(*),bx(*),by(*),bz(*)

call atob (n1*n2*n3,var1,scr%scr1)

call fill3d (scr%scr1,maxzp,maxxp,nnzh(icm),nnxh(icm),nnyh(icm))

call eintp (scr%scr1,scr%scr2,scr%scr3  &
           ,maxzp,maxxp,maxyp,n1,n2,n3,nnzh(icm),ifm,icm)

call nstb (n1,n2,n3,scr%scr3,bx,by,bz,itop,ibot)

return
end

!***********************************************************************

subroutine fill3d (rloc,n1m,n2m,n1,n2,n3)

dimension rloc (*)

if (n1 == n1m .and. n2 == n2m) return

do j=n3,1,-1
  lj=n1m*n2m*(j-1)
  mj=n1*n2*(j-1)
  do i=n2,1,-1
    lij=lj+n1m*(i-1)
    mij=mj+n1*(i-1)
    do k=n1,1,-1
      rloc(lij+k)=rloc(mij+k)
    enddo
  enddo
enddo

return
end

!***********************************************************************

subroutine eintp (ac,as,af,n1m,n2m,n3m,n1f,n2f,n3f,n1c,nf,nc)

use hyp_coms

dimension ac(n1m,n2m,n3m),as(n1m,n2m,n3m),af(n1f,n2f,n3f)

k1=max(1,kpmh(2,nf)-2)
k2=min(n1c,kpmh(n1f-1,nf)+2)

do jc=jpmh(1,nf)-1,jpmh(n3f,nf)+1
   do if=1,n2f
      do kc=k1,k2
         as(kc,if,jc)=ei1h(if,nf)*ac(kc,ipmh(if,nf)-1,jc)  &
                     +ei2h(if,nf)*ac(kc,ipmh(if,nf),  jc)  &
                     +ei3h(if,nf)*ac(kc,ipmh(if,nf)+1,jc)
      enddo
   enddo
enddo

do jf=1,n3f
   do if=1,n2f
      do kc=k1,k2
         ac(kc,if,jf)=ej1h(jf,nf)*as(kc,if,jpmh(jf,nf)-1)  &
                     +ej2h(jf,nf)*as(kc,if,jpmh(jf,nf)  )  &
                     +ej3h(jf,nf)*as(kc,if,jpmh(jf,nf)+1)
      enddo
   enddo
enddo

do jf=1,n3f
   do if=1,n2f
      do kf=1,n1f-1
         af(kf,if,jf)=ek1h(kf,nf)*ac(kpmh(kf,nf)-1,if,jf)  &
                     +ek2h(kf,nf)*ac(kpmh(kf,nf)  ,if,jf)  &
                     +ek3h(kf,nf)*ac(kpmh(kf,nf)+1,if,jf)
      enddo
   enddo
enddo

return
end

!***********************************************************************

subroutine nstb (n1,n2,n3,vt3da,bx,by,bz,itop,ibot)

dimension vt3da(n1,n2,n3),bx(n1,n3,2),by(n1,n2,2),bz(n2,n3,2)

do j=1,n3
   do k=1,n1
      bx(k,j,1)=vt3da(k,1,j)
      bx(k,j,2)=vt3da(k,n2,j)
   enddo
enddo

if(n3 > 1) then
   do i=1,n2
      do k=1,n1
         by(k,i,1)=vt3da(k,i,1)
         by(k,i,2)=vt3da(k,i,n3)
      enddo
   enddo
endif

if(ibot == 0) then
   do j=1,n3
      do i=1,n2
         bz(i,j,1)=vt3da(1,i,j)
      enddo
   enddo
endif

if(itop == 0) then
   do j=1,n3
      do i=1,n2
         bz(i,j,2)=vt3da(n1,i,j)
      enddo
   enddo
endif
return
end

!***********************************************************************

subroutine es_nstfeed (ng)

use hyp_coms
use mem_euler
use mem_scratch

implicit none

integer :: ng

integer :: ifm,icm,n
! feed back the finer mesh to the coarser mesh.

ifm=ng
icm=nxtnest(ifm)

do n=1,nspecies
   call es_fdback (eul(n,icm)%spec(1,1,1),eul(n,ifm)%spec(1,1,1)  &
                  ,nnzh(icm),nnxh(icm),nnyh(icm)  &
                  ,nnzh(ifm),nnxh(ifm),nnyh(ifm),ifm,icm,scr%scr2)
enddo

return
end

!***********************************************************************

subroutine es_fdback (ac,af,nzc,nxc,nyc,nzf,nxf,nyf,nf,nc,sumflg)
                     
use hyp_coms

implicit none

integer :: nzc,nxc,nyc,nzf,nxf,nyf,nf,nc
real :: ac(nzc,nxc,nyc),af(nzf,nxf,nyf),sumflg(nzc,nxc,nyc)
         
real :: vctr1(100)
integer :: ibeg,jbeg,kbeg,iend,jend,kend,iinc,jinc,ic,if,jc,jf,kc,kf &
          ,jdim
          
          
call azero(nzc*nxc*nyc,sumflg)

jdim = 1
if(nyc == 1) jdim = 0

ibeg=2
jbeg=1+jdim
kbeg=2
iend=nxf-1
jend=nyf-jdim
kend=nzf-1
iinc=1
jinc=1

do kf=1,nzf
   vctr1(kf)=dzth(kpmh(kf,nf),nc)  &
            /(float(nstratx(nf)*nstraty(nf))*dzth(kf,nf))
enddo

kf=kbeg
1 continue
   kc=kpmh(kf,nf)
   do jf=jbeg,jend,jinc
      jc=jpmh(jf,nf)
      do if=ibeg,iend,iinc
         ic=ipmh(if,nf)
         ac(kc,ic,jc)=ac(kc,ic,jc)*sumflg(kc,ic,jc)+af(kf,if,jf)*vctr1(kf)
         sumflg(kc,ic,jc)=1.
      enddo
   enddo
   kf=kf+1
if(kf <= kend) goto 1

return
end

