!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine hout ()

use an_header
use hyp_coms
use mem_all
use rams_head

implicit none

include 'interface.h'

integer :: nsrc,ls,iun,i,j,l,ln,iy,imn,id,ihms,ih,im,is,ng  &
          ,ntotparts,nsp,mgp,ihp,jhp,khp,irg,jrg,krg,ngmin,ngmax  &
          ,itab,npts,npointer,nv,iyear,imon,idate
real :: xavg,yavg,xclat,xclon,htopo,atp11,atp12,atp13,atp14,vol,xw,xe,ys,yn  &
       ,totmass,hswlat,hselat,hselon,hnwlat,hnwlon,hnelat,hnelon,hwlon  &
       ,helon,hslat,hnlat,hswlon,strtime
integer :: nspecparts(maxspec),nsrcparts(maxem)
real :: ietab(200,5)
character(len=128) :: dir
character(len=80) :: flnm
character(len=15) :: dstring
character(len=14) :: hcurdate
character(len=3) :: cspec
character(len=2) :: cgrid,csource,cspecies
integer, save :: ncall=0,nstart,nfinish
type (head_table), allocatable,save :: hyp_table(:)
integer, save:: nhypvars
real, allocatable :: dum_patch(:,:,:)
integer :: inyear,inmonth,indate,inhour

if(ncall == 0) then
   ncall=1

   ! average source locations to get default concentration grid center

   xavg=0.
   yavg=0.
   do nsrc=1,nsources
      xavg=xavg+srcx(nsrc)/nsources
      yavg=yavg+srcy(nsrc)/nsources
   enddo
   call xy_ll (xclat,xclon,pslat,pslon,xavg,yavg)

   ! write HGRID file

   flnm=trim(hyppref)//'-hgrid.txt'
   iun=88
   open(iun,file=flnm(1:len_trim(flnm)),status='unknown')
   write(iun,*) '1 ',nih,njh,nkh,pslat,pslon
   write(iun,'(20f10.0)') (xh(i),i=1,nih)
   write(iun,'(20f10.0)') (yh(i),i=1,njh)
   write(iun,'(20f8.1)')  (zh(i),i=1,nkh)
   write(iun,'(20f7.2)') ((hg%topthg(i,j),i=1,nih),j=1,njh)
   write(iun,'(20f7.2)') ((hg%pctlhg(i,j),i=1,nih),j=1,njh)
   close(iun)

   ! grids to write data for

   if(irgrid.eq.0) then
      nstart=1
      nfinish=ngrh
   elseif(irgrid.lt.0) then
      nstart=abs(irgrid)
      nfinish=ngrh
   elseif(irgrid.gt.0) then
      nstart=irgrid
      nfinish=irgrid
   endif
endif

! Find actual date/time taking into account backward run...
!print *,' cstart_time=',cstart_time, hytime

if (ihyprun == 2) then
   call date_add_to_big(cstart_time,hytime,'s',hcurdate)
elseif (ihyprun == 3) then
   call date_add_to_big(cend_time,-hytime,'s',hcurdate)
endif
!print*, '----------->hcurdate-',cend_time
!print*, '----------->hcurdate-',hcurdate
!print*, '----------->hcurdate-',-hytime
call date_unmake_big(inyear,inmonth,indate,inhour,hcurdate)



!--------------------------------------------------------------
! write Lagrangian particle files (asci) on the coarsest grid
! of interest (defined by irgrid)
!--------------------------------------------------------------

if(ipartout.eq.1) then

   print*

   read(hcurdate( 9:10),'(i2)') ih
   read(hcurdate(11:12),'(i2)') im
   read(hcurdate(13:14),'(i2)') is

   write(cgrid,'(a1,i1)') 'g',0
   
   call makefnam(flnm,hyppref,0.,inyear,inmonth,indate,inhour &
               ,'P',cgrid,'txt')

   print*,'Writing Lag Part file - ',trim(flnm)

   iun=90
   open(iun,file=flnm,status='unknown')
   ng=nstart

   ! need to go through the parts to get header info

   ntotparts=0
   do nsp=1,nspecies
      nspecparts(nsp)=0
   enddo
   do nsrc=1,nsources
      nsrcparts(nsrc)=0
   enddo
   do l=1,npart
      if(atp(l)%ncluster /= 0 .and. ioutspec(atp(l)%nspecies) == 1) then
      
         call part_loc (atp(l)%hi,atp(l)%hj,atp(l)%hk,nih,njh,nkh,hg%mg,hg%topthg  &
                       ,ihp,jhp,khp,htopo,atp11,atp12,atp13,atp14,mgp)
                      
         if(mgp.ne.-1) then

            ! count the number of particles on these grids

            ntotparts=ntotparts+1
            nsrcparts(atp(l)%nsource)=nsrcparts(atp(l)%nsource)+1
            nspecparts(atp(l)%nspecies)=nspecparts(atp(l)%nspecies)+1
         endif
      endif
   enddo

 ! write main header line
   write(iun,'(i3,e16.8,8i8)') 1,abs(hytime-time_offset)  &
                              ,inyear,inmonth,indate,ih,im,ng,nsources  &
                              ,ntotparts

! write source header lines - note the difference between the internal HYPACT
! code reference numnbers and the database numbers
!      nsrc is the internal HYPACT source number
!      psource(nsrc) is the source number from the database
!      species(nsrc) is the internal HYPACT species number
!      pspecies(species(nsrc)) is the species number from the database
!      nsrcparts(psource(nsrc)) is the number of particles from this source

   do nsrc=1,nsources
      write(iun,'(4i8)') psource(nsrc),species(nsrc)  &
                        ,pspecies(species(nsrc)),nsrcparts(nsrc)
   enddo

   do nsrc=1,nsources
   
      if(ilagprob == 1) then
         xw=1.e15
         xe=-1.e15
         ys=1.e15
         yn=-1.e15
      endif
      
      ngmin=maxg
      ngmax=0
      totmass=0.
      
      do l=1,npart

         ! for each part get rams grid, location on RAMS grid, and thickness
         ! of level

         if( atp(l)%nsource == nsrc .and. &
             atp(l)%nspecies == species(nsrc) .and. &
             atp(l)%ncluster /= 0 .and. &
             ioutspec(species(nsrc)).eq.1) then
            
            call part_loc (atp(l)%hi,atp(l)%hj,atp(l)%hk,nih,njh,nkh,hg%mg,hg%topthg  &
                          ,ihp,jhp,khp,htopo,atp11,atp12,atp13,atp14,mgp)

            if(mgp /= -1) then

               ! write and tally the parts if on the grid
               ! Not sure about this one - what is agl and asl?
               !write(iun,'(7e16.8,i5)') atp11,atp12,atp13+htopo,atp13  &
               write(iun,'(7e16.8,3i5)') atp11,atp12,atp14,atp14-htopo  &
                                       ,atp(l)%mass,atp(l)%ppm,hytime-atp(l)%reltime  &
                                       ,atp(l)%ncluster,pspecies(atp(l)%nspecies),psource(atp(l)%nsource)
               totmass=totmass+atp(l)%mass
               ngmin=min(ngmin,mgp)
               ngmax=max(ngmax,mgp)
               
               ! Not sure about this one?
               !if(atp13 < 2000. .and. ilagprob==1) then
               if(atp14-htopo < 2000. .and. ilagprob==1) then
                  ! get max/min x-y and lat-lon below spec hgt agl
                  ! want to make this the pbl based off tke or other
                  xw=min(xw,atp11)
                  xe=max(xe,atp11)
                  ys=min(ys,atp12)
                  yn=max(yn,atp12)
               endif
               
            endif
         endif
      enddo
      ln=len_trim(units(species(nsrc)))

      ! write a summary for each source to std out

      write(*,'(a,i7,a,e12.6,3a,i3,3(a,i3))') ' total mass of '  &
          ,nspecparts(species(nsrc)),' particles = ',totmass,' '  &
          ,units(species(nsrc))(1:ln),' for species'  &
          ,pspecies(species(nsrc)),', source',psource(nsrc)  &
          ,', covering grids',ngmin,' to',ngmax
   
      if(ilagprob==1) then
         call xy_ll(hswlat,hswlon,pslat,pslon,xw,ys)
         call xy_ll(hselat,hselon,pslat,pslon,xe,ys)
         call xy_ll(hnwlat,hnwlon,pslat,pslon,xw,yn)
         call xy_ll(hnelat,hnelon,pslat,pslon,xe,yn)
         hwlon=min(hswlon,hnwlon)
         helon=max(hselon,hnelon)
         hslat=min(hswlat,hselat)
         hnlat=max(hnwlat,hnelat)
         print*,'max-min lat-lon: se begline',hslat,hwlon
         print*,'max-min lat-lon: sw drwline',hslat,helon
         print*,'max-min lat-lon: nw drwline',hnlat,helon
         print*,'max-min lat-lon: ne drwline',hnlat,hwlon
         print*,'max-min lat-lon: se endline',hslat,hwlon
      endif
      
   enddo
   close(iun)

endif

!-------------------------------------------------------------
! write vfile format concentration and met files
!-------------------------------------------------------------
   
! allocate header table structure

nhypvars=4
if(imetout /= 0) nhypvars=nhypvars+7
nhypvars=nhypvars+(4*nspecies)       ! max number of conc fields per grid
if(ilagprob==1) nhypvars=nhypvars+1  ! particle probabilities

nhypvars=nhypvars*ngrh

if(allocated(hyp_table)) deallocate(hyp_table)
allocate (hyp_table(nhypvars))

if(ioutfmt == 1) then
  itab=0
  do ng=nstart,nfinish
    npts=nnzh(ng)*nnxh(ng)*nnyh(ng)
    npointer=0
    write(cgrid,'(a1,i1)')'g',ng
    call makefnam(flnm,hyppref,0.,inyear,inmonth,indate,inhour &
               ,'C',cgrid,'vfm')
    print*
    print*,'Writing v-format conc vfile-',flnm(1:len_trim(flnm))
    call rams_c_open (trim(flnm)//char(0),'w'//char(0))

    ! write the surface stuff

    ! topo
    itab=itab+1
    hyp_table(itab)%string='TOPT'
    hyp_table(itab)%npointer=npointer
    hyp_table(itab)%idim_type=2
    hyp_table(itab)%ngrid=ng
    hyp_table(itab)%nvalues=nnxh(ng)*nnyh(ng)
    call vforecr (iun,grid(ng)%topt,nnxh(ng)*nnyh(ng),18  &
                 ,scr%scr2,scr%scr3,'LIN',npointer)
    ! latitude
    itab=itab+1
    hyp_table(itab)%string='GLAT'
    hyp_table(itab)%npointer=npointer
    hyp_table(itab)%idim_type=2
    hyp_table(itab)%ngrid=ng
    hyp_table(itab)%nvalues=nnxh(ng)*nnyh(ng)
    call vforecr (iun,grid(ng)%glat,nnxh(ng)*nnyh(ng),18  &
                 ,scr%scr2,scr%scr3,'LIN',npointer)
    ! longitude
    itab=itab+1
    hyp_table(itab)%string='GLON'
    hyp_table(itab)%npointer=npointer
    hyp_table(itab)%idim_type=2
    hyp_table(itab)%ngrid=ng
    hyp_table(itab)%nvalues=nnxh(ng)*nnyh(ng)
    call vforecr (iun,grid(ng)%glon,nnxh(ng)*nnyh(ng),18  &
                 ,scr%scr2,scr%scr3,'LIN',npointer)

    ! land percentage - Put out phony patch_area for now!!!!
    itab=itab+1
    hyp_table(itab)%string='PATCH_AREA'
    hyp_table(itab)%npointer=npointer
    hyp_table(itab)%idim_type=4
    hyp_table(itab)%ngrid=ng
    hyp_table(itab)%nvalues=nnxh(ng)*nnyh(ng)*nnpatchh
    if(allocated(dum_patch)) deallocate(dum_patch)
    allocate (dum_patch(nnxh(ng),nnyh(ng),nnpatchh))
    call dum_patch_area (grid(ng)%pctl,dum_patch,nnxh(ng),nnyh(ng)  &
                        ,nnpatchh)
    call vforecr (iun,dum_patch(1,1,1),nnxh(ng)*nnyh(ng)*nnpatchh,18  &
                 ,scr%scr2,scr%scr3,'LIN',npointer)
    if(allocated(dum_patch)) deallocate(dum_patch)

    if(imetout.eq.1) then

      ! write the met

      ! u
      itab=itab+1
      hyp_table(itab)%string='UP'
      hyp_table(itab)%npointer=npointer
      hyp_table(itab)%idim_type=3
      hyp_table(itab)%ngrid=ng
      hyp_table(itab)%nvalues=npts
      call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),metc(ng)%u,scr%scr1)
      call vforecr (iun,scr%scr1,npts,18,scr%scr2,scr%scr3  &
                   ,'LIN',npointer)

      ! v
      itab=itab+1
      hyp_table(itab)%string='VP'
      hyp_table(itab)%npointer=npointer
      hyp_table(itab)%idim_type=3
      hyp_table(itab)%ngrid=ng
      hyp_table(itab)%nvalues=npts
      call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),metc(ng)%v,scr%scr1)
      call vforecr (iun,scr%scr1,npts,18,scr%scr2,scr%scr3  &
                   ,'LIN',npointer)

      ! w
      itab=itab+1
      hyp_table(itab)%string='WP'
      hyp_table(itab)%npointer=npointer
      hyp_table(itab)%idim_type=3
      hyp_table(itab)%ngrid=ng
      hyp_table(itab)%nvalues=npts
      call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),metc(ng)%w,scr%scr1)
      call vforecr (iun,scr%scr1,npts,18,scr%scr2,scr%scr3  &
                   ,'LIN',npointer)

      ! tke
      itab=itab+1
      hyp_table(itab)%string='TKEP'
      hyp_table(itab)%npointer=npointer
      hyp_table(itab)%idim_type=3
      hyp_table(itab)%ngrid=ng
      hyp_table(itab)%nvalues=npts
      call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),metc(ng)%tke,scr%scr1)
      call vforecr (iun,scr%scr1,npts,18,scr%scr2,scr%scr3  &
                   ,'LIN',npointer)

      ! theta
      itab=itab+1
      hyp_table(itab)%string='THETA'
      hyp_table(itab)%npointer=npointer
      hyp_table(itab)%idim_type=3
      hyp_table(itab)%ngrid=ng
      hyp_table(itab)%nvalues=npts
      call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),metc(ng)%theta,scr%scr1)
      call vforecr (iun,scr%scr1,npts,18,scr%scr2,scr%scr3  &
                   ,'LIN',npointer)

      ! khv
      itab=itab+1
      hyp_table(itab)%string='VKH'
      hyp_table(itab)%npointer=npointer
      hyp_table(itab)%idim_type=3
      hyp_table(itab)%ngrid=ng
      hyp_table(itab)%nvalues=npts
      call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),metc(ng)%khv,scr%scr1)
      call vforecr (iun,scr%scr1,npts,18,scr%scr2,scr%scr3  &
                   ,'LIN',npointer)

      ! khh
      itab=itab+1
      hyp_table(itab)%string='HKH'
      hyp_table(itab)%npointer=npointer
      hyp_table(itab)%idim_type=3
      hyp_table(itab)%ngrid=ng
      hyp_table(itab)%nvalues=npts
      call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),metc(ng)%khh,scr%scr1)
      call vforecr (iun,scr%scr1,npts,18,scr%scr2,scr%scr3  &
                   ,'LIN',npointer)
 
    endif

    if(ilagout /= 0 .or. ieulout /= 0) then
      do nsp=1,nspecies

        ! for each species write

        if(ioutspec(nsp) == 1) then

          write(cspec,'(i3.3)') pspecies(nsp)

          ! Lagrangian particle concentration
 
          if(ilagout.eq.1.or.ilagout.eq.3) then
            print*,'Lagrangian concentration for spec:        ',pspecies(nsp)
            itab=itab+1
            hyp_table(itab)%string='CONC_LAG_I'//cspec
            hyp_table(itab)%npointer=npointer
            hyp_table(itab)%idim_type=3
            hyp_table(itab)%ngrid=ng
            hyp_table(itab)%nvalues=npts
            call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),eul(nsp,ng)%specl  &
                           ,scr%scr1)
            call vforecr (iun,scr%scr1,npts,18  &
                         ,scr%scr2,scr%scr3,'LIN',npointer)
 
            if(ilagprob == 1) then
              ! added for particle probability
              print*,'Lagrangian probablilties for spec:        ',pspecies(nsp)
              itab=itab+1
              hyp_table(itab)%string='PROB_LAG_I'//cspec
              hyp_table(itab)%npointer=npointer
              hyp_table(itab)%idim_type=3
              hyp_table(itab)%ngrid=ng
              hyp_table(itab)%nvalues=npts
              call rearrange (nnzh(ng),nnxh(ng),nnyh(ng)  &
                              ,eul(nsp,ng)%speclprob,scr%scr1)
              call sum_prob (ng,nnzh(ng),nnxh(ng),nnyh(ng),scr%scr1)
              call vforecr (iun,scr%scr1,npts,18  &
                           ,scr%scr2,scr%scr3,'LIN',npointer)
            endif

          endif

          ! average Lagrangian particle concentration
 
          if(ilagout == 2 .or. ilagout == 3) then
            print*,'average Lagrangian concentration for spec:',pspecies(nsp)

            itab=itab+1
            hyp_table(itab)%string='CONC_LAG_A'//cspec
            hyp_table(itab)%npointer=npointer
            hyp_table(itab)%idim_type=3
            hyp_table(itab)%ngrid=ng
            hyp_table(itab)%nvalues=npts
            call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),eul(nsp,ng)%speclavg  &
                           ,scr%scr1)
            call vforecr (iun,scr%scr1,npts,18  &
                         ,scr%scr2,scr%scr3,'LIN',npointer)
          endif

          ! Eulerian concentration
 
          if(ieulout.eq.1.or.ieulout.eq.3) then
            print*,'Eulerian concentration for spec:          ',pspecies(nsp)
            itab=itab+1
            hyp_table(itab)%string='CONC_EUL_I'//cspec
            hyp_table(itab)%npointer=npointer
            hyp_table(itab)%idim_type=3
            hyp_table(itab)%ngrid=ng
            hyp_table(itab)%nvalues=npts
            call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),eul(nsp,ng)%spec  &
                           ,scr%scr1)
            call vforecr (iun,scr%scr1,npts,18  &
                         ,scr%scr2,scr%scr3,'LIN',npointer)
          endif

          ! average Eulerian concentration
 
          if(ieulout == 2 .or. ieulout == 3) then
            print*,'average Eulerian concentration for spec:  ',pspecies(nsp)
            itab=itab+1
            hyp_table(itab)%string='CONC_EUL_A'//cspec
            hyp_table(itab)%npointer=npointer
            hyp_table(itab)%idim_type=3
            hyp_table(itab)%ngrid=ng
            hyp_table(itab)%nvalues=npts
            call rearrange (nnzh(ng),nnxh(ng),nnyh(ng),eul(nsp,ng)%specavg  &
                           ,scr%scr1)
            call vforecr (iun,scr%scr1,npts,18  &
                         ,scr%scr2,scr%scr3,'LIN',npointer)
          endif

        endif
      enddo
    endif
    call RAMS_c_close()
  enddo
endif

! write header file

call makefnam(flnm,hyppref,0.,inyear,inmonth,indate,inhour &
               ,'C','head','txt')
print*
print*,'Writing v-format header file-',flnm(1:len_trim(flnm))
iun=88
open(iun,file=flnm,status='unknown')

write(iun,110) itab
if(itab > 0) then
   do nv=1,itab
      write(iun,120) hyp_table(nv)%string   &
                    ,hyp_table(nv)%npointer  &
                    ,hyp_table(nv)%idim_type  &
                    ,hyp_table(nv)%ngrid  &
                    ,hyp_table(nv)%nvalues
   enddo
110 format(i6)
120 format(a16,1x,i12,i3,i3,1x,i9)
endif

call rams_head_fill()

call rams_head_write(iun)

close(iun)

if(allocated(hyp_table)) deallocate(hyp_table)
print*

return
end

!***************************************************************************

subroutine dum_patch_area (pctl,dum_patch,nxp,nyp,npatch)

implicit none
integer :: nxp,nyp,npatch
real :: pctl(nxp,nyp),dum_patch(nxp,nyp,npatch)
integer :: i,j,np

! Fill all patches with water area

do np=1,npatch
   do j=1,nyp
      do i=1,nxp
         dum_patch(i,j,np)=1.-pctl(i,j)
      enddo
   enddo
enddo

return
end
