!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine hyp_rdanal (met,numfl,ngrid)

use an_header
use ra_file_info
use hyp_coms
use mem_met
use mem_grid

implicit none

integer :: numfl,ngrid
type(met_vars) :: met

character(len=1) :: cgrid
character(len=128):: flnma,flnm
integer :: idata(6),ng,nval,ierr,RAMS_getvar
logical, save :: firstcall=.t.

!print*,' reading analysis written at time ',time,nvbtab 
 
! allocate memory for this file

!print*,'------------------------------------------------------'
!print*,'allocating memory for RAMS analysis variables'
!print*,'------------------------------------------------------'

! Get grid point info from analysis file
print '(a,f12.1,i6,i4)','Reading analysis file time '  &
         ,ftimes(numfl),numfl,ng

call ra_set_file(1,numfl)
call ra_get_cdata (0,flnma,nval)
call rams_read_header (flnma, action='read')

flnm = flnma(1:len_trim(flnma)-9)

if (firstcall) then
   ! Set up parameters for all grids
   !----------------------------------
   call ra_get_idata(1,1,ngrh,nval)
   do ng = 1, ngrh
      call ra_get_idata(0,ng,idata,nval)
      nnxh(ng)=idata(1) ; nnyh(ng)=idata(2) 
      nnzh(ng)=idata(3) ; nngh    =idata(4)
      nnsh    =idata(5) ; nnpatchh=idata(6)
   enddo
endif

! Allocate data type for this grid
call dealloc_met(met)
call alloc_met  (met,nnzh(ngrid),nnxh(ngrid),nnyh(ngrid))


if (firstcall) then
   
   
   
   call ra_get_cdata(1,startdate,nval)

   ! Allocate grid datatype
   do ng = 1, ngrh
   
      call alloc_grid(grid(ng),nnzh(ng),nnxh(ng),nnyh(ng))
      
      ! Allocate current time data if first call
      call dealloc_met(metc(ng))
      call alloc_met  (metc(ng),nnzh(ng),nnxh(ng),nnyh(ng))
   
      !print*,'reading topt (4) - itopth =',itopth(ngrid)
      ierr = RAMS_getvar('TOPT',ng,grid(ng)%topt,met%scr2,flnm)

      !print*,'reading pctl (16) - ipctlh =',ipctlh(ngrid)
      ierr = RAMS_getvar('PATCH_AREA',ng,met%scr1(1),met%scr2,flnm)
      call HYP_comp_1minus(nnxh(ng),nnyh(ng),1,met%scr1,grid(ng)%pctl)
   enddo

   call ra_get_fdata(6,1,pslat,nval)
   call ra_get_fdata(7,1,pslon,nval)
   call ra_get_fdata(3,1,ztoph,nval)
   call ra_get_idata(5,1,ihtran,nval)
   call ra_get_idata(6,1,if_adap,nval)
   do ng = 1, ngrh
      call ra_get_fdata(4,ng,delxh(ng),nval)
      call ra_get_fdata(5,ng,delyh(ng),nval)
   
      call ra_get_fdata(11,ng,xmh(1,ng),nval)
      call ra_get_fdata(13,ng,ymh(1,ng),nval)
      call ra_get_fdata(14,ng,zmh(1,ng),nval)
   
      call ra_get_fdata(10,ng,xth(1,ng),nval)
      call ra_get_fdata(12,ng,yth(1,ng),nval)
      call ra_get_fdata( 0,ng,zth(1,ng),nval)
      !print*,'z:',ng,zth(1:4,ng)
      !print*,'z:',ng,zmh(1:4,ng)
      
      call ra_read_fdata('dzmn',ng,dzmh(1,ng),nval)   
      call ra_read_fdata('dztn',ng,dzth(1,ng),nval)   
   enddo
   
   call ra_read_idata('nestz1',1,nestz1,nval)   
   call ra_read_idata('nxtnest',1,nxtnest,nval)   
   call ra_read_idata('ninest',1,ninest,nval)   
   call ra_read_idata('njnest',1,njnest,nval)   
   call ra_read_idata('nknest',1,nknest,nval)   
   call ra_read_idata('nstratx',1,nstratx,nval)   
   call ra_read_idata('nstraty',1,nstraty,nval)   
   call ra_read_idata('nstratz1',1,nstratz1,nval)   
   call ra_read_idata('nnsttop',1,nnsttop,nval)   
   call ra_read_idata('nnstbot',1,nnstbot,nval)   

! set up RAMS grid spaces and coordinates
     
   call hyp_gridset (maxk,maxi,maxj,nnxh,nnyh,nnzh,ngrh &
                     ,nxtnest,nestz1  &
                     ,ninest,njnest,nknest,nstratx,nstraty,nstratz1  &
                     ,delxh,delyh,delzh  &
                     ,xmh,xth,ipmh,ymh,yth,jpmh  &
                     ,zmh,dzmh,zth,dzth,kpmh,nrzh)

   do ng = 1, ngrh

      call hyp_polarst (nnxh(ng),nnyh(ng),maxi,maxj,ng  &
                     ,grid(ng)%glat  &
                     ,grid(ng)%glon,  grid(ng)%fmapu, grid(ng)%fmapv  &
                     ,grid(ng)%fmapt, grid(ng)%fmapm, grid(ng)%fmapui  &
                     ,grid(ng)%fmapvi,grid(ng)%fmapti,grid(ng)%fmapmi  &
                     ,pslat,pslon,xth,xmh,yth,ymh,ihtran)
                 
      call hyp_grdspc (nnxh(ng),nnyh(ng),maxi,maxj,ng  &
                     ,grid(ng)%dxu,grid(ng)%dxv,grid(ng)%dxt,grid(ng)%dxm  &
                     ,grid(ng)%dyu,grid(ng)%dyv,grid(ng)%dyt,grid(ng)%dym  &
                     ,grid(ng)%fmapu,grid(ng)%fmapv,grid(ng)%fmapt,grid(ng)%fmapm  &
                     ,xth,xmh,yth,ymh )
                
      call hyp_jacob (nnzh(ng),nnxh(ng),nnyh(ng),ng,maxi,maxj,maxk  &
                     ,grid(ng)%topt,grid(ng)%topu,grid(ng)%topv,grid(ng)%topm  &
                     ,grid(ng)%rtgt,grid(ng)%rtgu,grid(ng)%rtgv,grid(ng)%rtgm  &
                     ,grid(ng)%f13u,grid(ng)%f13v,grid(ng)%f13t,grid(ng)%f13m  &
                     ,grid(ng)%f23u,grid(ng)%f23v,grid(ng)%f23t,grid(ng)%f23m  &
                     ,grid(ng)%dxu, grid(ng)%dxv, grid(ng)%dxt, grid(ng)%dxm   &
                     ,grid(ng)%dyu, grid(ng)%dyv, grid(ng)%dyt, grid(ng)%dym  &
                     ,xth,xmh,yth,ymh,zth,zmh,hth,hwh,ztoph)

   enddo

   call hyp_nestcof (ngrh,nnzh,nnxh,nnyh,maxk,maxi,maxj,kpmh  &
                 ,ei1h,ei2h,ei3h,ej1h,ej2h,ej3h,ek1h,ek2h,ek3h  &
                 ,nxtnest,nstratx,nstraty  &
                 ,zth,dzth,zmh)
                 
   firstcall = .f.

endif                 




! read the necessary analysis file variables

print*

! read the 3-d variables

!print*,'reading up (1) - iuph =',iuph(ngrid)
ierr = RAMS_getvar('UP',ngrid,met%scr1,met%scr2,flnm)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
               ,met%scr1,met%u)

!print*,'reading vp (2) - ivph =',ivph(ngrid)
ierr = RAMS_getvar('VP',ngrid,met%scr1,met%scr2,flnm)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
               ,met%scr1,met%v)

!print*,'reading wp (3) - iwph =',iwph(ngrid)
ierr = RAMS_getvar('WP',ngrid,met%scr1,met%scr2,flnm)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
               ,met%scr1,met%w)

!print*,'reading tkep (14) - itkeph =',itkeph(ngrid)
ierr = RAMS_getvar('TKEP',ngrid,met%scr1,met%scr2,flnm)
call HYP_comp_noneg(nnxh(ngrid),nnyh(ngrid),nnzh(ngrid)  &
                     ,met%scr1)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
              ,met%scr1,met%tke)

!print*,'reading thet (19) - itheth =',itheth(ngrid)
ierr = RAMS_getvar('THETA',ngrid,met%scr1,met%scr2,flnm)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
               ,met%scr1,met%theta)

!print*,'reading khv (68) - ikhvh = ',ikhvh(ngrid)
ierr = RAMS_getvar('VKH',ngrid,met%scr1,met%scr2,flnm)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
               ,met%scr1,met%khv)

!print*,'reading khh (69) - ikhhh = ',ikhhh(ngrid)
ierr = RAMS_getvar('HKH',ngrid,met%scr1,met%scr2,flnm)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
               ,met%scr1,met%khh)

!print*,'reading pph (6) - ipresh = ',ipresh(ngrid)
ierr = RAMS_getvar('PI',ngrid,met%scr1,met%scr2,flnm)
met%scr3=met%scr1
call HYP_comp_press(nnxh(ngrid),nnyh(ngrid),nnzh(ngrid)  &
              ,met%scr1)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
              ,met%scr1,met%press)

!print*,'computing temp (19) - itemph = ',itemph(ngrid)
ierr = RAMS_getvar('THETA',ngrid,met%scr1,met%scr2,flnm)
call HYP_comp_tempK(nnxh(ngrid),nnyh(ngrid),nnzh(ngrid)  &
                    ,met%scr1,met%scr3)
call unarrange(nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
                    ,met%scr1,met%temp)


call diagt4a (nnzh(ngrid),nnxh(ngrid),nnyh(ngrid)  &
             ,met%u,met%v,met%theta  &
             ,met%tke,met%sigu,met%sigv  &
             ,met%sigw,met%ds2  &
             ,met%tlu,met%tlv,met%tlw  &
             ,zth(1,ngrid),zmh(1,ngrid),grid(ngrid)%rtgt)

 
return
end

!***************************************************************************

subroutine hyp_gridavg ()

use hyp_coms
use mem_met

implicit none

integer :: ng,i,j,k,npts
real :: wtp


do ng=1,ngrh
   wtp= (hytime-ftimes(ipstfl(ng)))/(ftimes(ifutfl(ng))-ftimes(ipstfl(ng)))         
   
!print '(a,i2,5f12.3)','hyp_gridavg: averaging-',ng,wtp,hytime  &
!               ,ftimes(ipstfl(ng)),ftimes(ifutfl(ng))
!   do j=1,nnyh(ng)
!      do i=1,nnxh(ng)
!         do k=1,nnzh(ng)
!            metc(ng)%u(k,i,j)    = metp(ng)%u(k,i,j)    + &
!                     wtp * (metf(ng)%u(k,i,j) - metp(ng)%u(k,i,j))
!            metc(ng)%v(k,i,j)    = metp(ng)%v(k,i,j)    + &
!                     wtp * (metf(ng)%v(k,i,j) - metp(ng)%v(k,i,j))
!            metc(ng)%w(k,i,j)    = metp(ng)%w(k,i,j)    + &
!                     wtp * (metf(ng)%w(k,i,j) - metp(ng)%w(k,i,j))
!            metc(ng)%tke(k,i,j)  = metp(ng)%tke(k,i,j)  + &
!                     wtp * (metf(ng)%tke(k,i,j) - metp(ng)%tke(k,i,j))
!            metc(ng)%theta(k,i,j)= metp(ng)%theta(k,i,j)+ &
!                     wtp * (metf(ng)%theta(k,i,j) - metp(ng)%theta(k,i,j))
!            metc(ng)%sigu(k,i,j) = metp(ng)%sigu(k,i,j) + &
!                     wtp * (metf(ng)%sigu(k,i,j) - metp(ng)%sigu(k,i,j))
!            metc(ng)%sigv(k,i,j) = metp(ng)%sigv(k,i,j) + &
!                     wtp * (metf(ng)%sigv(k,i,j) - metp(ng)%sigv(k,i,j))
!            metc(ng)%sigw(k,i,j) = metp(ng)%sigw(k,i,j) + &
!                     wtp * (metf(ng)%sigw(k,i,j) - metp(ng)%sigw(k,i,j))
!            metc(ng)%tlu(k,i,j)  = metp(ng)%tlu(k,i,j)  + &
!                     wtp * (metf(ng)%tlu(k,i,j) - metp(ng)%tlu(k,i,j))
!            metc(ng)%tlv(k,i,j)  = metp(ng)%tlv(k,i,j)  + &
!                     wtp * (metf(ng)%tlv(k,i,j) - metp(ng)%tlv(k,i,j))
!            metc(ng)%tlw(k,i,j)  = metp(ng)%tlw(k,i,j)  + &
!                     wtp * (metf(ng)%tlw(k,i,j) - metp(ng)%tlw(k,i,j))
!            metc(ng)%ds2(k,i,j)  = metp(ng)%ds2(k,i,j)  + &
!                     wtp * (metf(ng)%ds2(k,i,j) - metp(ng)%ds2(k,i,j))
!            metc(ng)%khv(k,i,j)  = metp(ng)%khv(k,i,j)  + &
!                     wtp * (metf(ng)%khv(k,i,j) - metp(ng)%khv(k,i,j))
!            metc(ng)%khh(k,i,j)  = metp(ng)%khh(k,i,j)  + &
!                     wtp * (metf(ng)%khh(k,i,j) - metp(ng)%khh(k,i,j))
!            metc(ng)%temp(k,i,j) = metp(ng)%temp(k,i,j) + &
!                     wtp * (metf(ng)%temp(k,i,j) - metp(ng)%temp(k,i,j))
!            metc(ng)%press(k,i,j)= metp(ng)%press(k,i,j)+ &
!                     wtp * (metf(ng)%press(k,i,j) - metp(ng)%press(k,i,j))
!         enddo
!      enddo
!   enddo

   ! This executes much faster than above explicit loops.

   npts=nnzh(ng)*nnxh(ng)*nnyh(ng)
   call hmet_update(npts,metc(ng)%u(1,1,1),metp(ng)%u(1,1,1),metf(ng)%u(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%v(1,1,1),metp(ng)%v(1,1,1),metf(ng)%v(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%w(1,1,1),metp(ng)%w(1,1,1),metf(ng)%w(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%sigu(1,1,1),metp(ng)%sigu(1,1,1),metf(ng)%sigu(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%sigv(1,1,1),metp(ng)%sigv(1,1,1),metf(ng)%sigv(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%sigw(1,1,1),metp(ng)%sigw(1,1,1),metf(ng)%sigw(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%tlu(1,1,1),metp(ng)%tlu(1,1,1),metf(ng)%tlu(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%tlv(1,1,1),metp(ng)%tlv(1,1,1),metf(ng)%tlv(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%tlw(1,1,1),metp(ng)%tlw(1,1,1),metf(ng)%tlw(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%ds2(1,1,1),metp(ng)%ds2(1,1,1),metf(ng)%ds2(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%tke(1,1,1),metp(ng)%tke(1,1,1),metf(ng)%tke(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%theta(1,1,1),metp(ng)%theta(1,1,1),metf(ng)%theta(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%khv(1,1,1),metp(ng)%khv(1,1,1),metf(ng)%khv(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%khh(1,1,1),metp(ng)%khh(1,1,1),metf(ng)%khh(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%temp(1,1,1),metp(ng)%temp(1,1,1),metf(ng)%temp(1,1,1),wtp)
   call hmet_update(npts,metc(ng)%press(1,1,1),metp(ng)%press(1,1,1),metf(ng)%press(1,1,1),wtp)

enddo

! manual settings for the met arrays (for test purposes)

if(izeromet==1) then
   if(iwarn==1) print*,'warning: hyp_gridavg: CHANGED METEOROLOGY'
   do ng=1,ngrh
      do j=1,nnyh(ng)
         do i=1,nnxh(ng)
            do k=1,nnzh(ng)
               metc(ng)%u(k,i,j)    =0.
               metc(ng)%v(k,i,j)    =0.
               metc(ng)%w(k,i,j)    =0.
               metc(ng)%tke(k,i,j)  =0.
               metc(ng)%theta(k,i,j)=0.
               metc(ng)%sigu(k,i,j) =0.
               metc(ng)%sigv(k,i,j) =0.
               metc(ng)%sigw(k,i,j) =0.
               metc(ng)%tlu(k,i,j)  =0.
               metc(ng)%tlv(k,i,j)  =0.
               metc(ng)%tlw(k,i,j)  =0.
               metc(ng)%ds2(k,i,j)  =0.
               metc(ng)%khv(k,i,j)  =0.
               metc(ng)%khh(k,i,j)  =0.
               metc(ng)%temp(k,i,j) =0.
               metc(ng)%press(k,i,j)=0.
            enddo
         enddo
      enddo
   enddo
endif

return
end

subroutine hmet_update(npts,c,p,f,wt)
implicit none
integer :: npts
real, dimension(npts) :: c,p,f
real :: wt

integer :: n

do n=1,npts
   c(n)=p(n)+wt*(f(n)-p(n))
enddo

return
end
