!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

subroutine HYP_comp_tempK(n1,n2,n3,a,b)

use rconstants

integer :: n1,n2,n3
real, dimension(n1,n2,n3) :: a,b

do k=1,n3
   do j=1,n2
      do i=1,n1
         a(i,j,k)=a(i,j,k)*b(i,j,k)/cp
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine HYP_comp_press(n1,n2,n3,a)

use rconstants

integer :: n1,n2,n3
real, dimension(n1,n2,n3) :: a

do k=1,n3
   do j=1,n2
      do i=1,n1
         a(i,j,k)=(a(i,j,k)/cp)**cpor*p00*.01
      enddo
   enddo
enddo
   
return
end

!***************************************************************************

subroutine HYP_comp_noneg(n1,n2,n3,a)

integer :: n1,n2,n3
real, dimension(n1,n2,n3) :: a

do k=1,n3
   do j=1,n2
      do i=1,n1
         a(i,j,k)=max(a(i,j,k),0.)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine HYP_comp_1minus(n1,n2,n3,a,b)

integer :: n1,n2,n3
real, dimension(n1,n2,n3) :: a,b

do k=1,n3
   do j=1,n2
      do i=1,n1
         b(i,j,k)=1.-a(i,j,k)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine eul_avg ()

use hyp_coms
use mem_euler

implicit none

integer :: nstart,nfinish,ng,n,npts
real :: emax

ieavgcall=ieavgcall+1

if (irgrid == 0) then
   nstart=1 ;  nfinish=ngrh
elseif (irgrid < 0) then
   nstart=abs(irgrid)  ;  nfinish=ngrh
elseif( irgrid > 0) then
   nstart=irgrid  ;  nfinish=irgrid
endif


do ng=nstart,nfinish
   npts=nnzh(ng)*nnxh(ng)*nnyh(ng)
   do n=1,nspecies
      call es_avg(npts,eul(n,ng)%specavg(1,1,1),eul(n,ng)%spec(1,1,1)  &
                 ,ieavgcall,emax)
      write(*,'(2(a,i3),2(a,e10.4))')  &
            'Eul Avg - grid: ',ng,' species: ',pspecies(n)  &
           ,' current max: ',emax
   enddo
enddo

return
end

!***************************************************************************

subroutine es_avg(npts,eavg,e,navg,emax)
implicit none
integer :: npts,navg
real, dimension(npts) :: eavg,e
real :: emax

integer :: n
real :: xavgi

xavgi=1./float(navg)
do n=1,npts
   eavg(n)=(eavg(n)*(navg-1)+e(n))*xavgi
enddo

emax=maxval(e(1:npts))

return
end

!***************************************************************************

subroutine lag_avg ()

use hyp_coms
use mem_euler

implicit none

integer :: nstart,nfinish,ng,n,npts
real :: lmax

ilavgcall=ilavgcall+1

if (irgrid == 0) then
   nstart=1 ;  nfinish=ngrh
elseif (irgrid < 0) then
   nstart=abs(irgrid)  ;  nfinish=ngrh
elseif( irgrid > 0) then
   nstart=irgrid  ;  nfinish=irgrid
endif

do ng=nstart,nfinish
   npts=nnzh(ng)*nnxh(ng)*nnyh(ng)
   do n=1,nspecies
      call es_avg(npts,eul(n,ng)%speclavg(1,1,1),eul(n,ng)%specl(1,1,1)  &
                 ,ilavgcall,lmax)
      write(*,'(2(a,i3),2(a,e10.4))')  &
            'Lag Avg - grid: ',ng,' species: ',pspecies(n)  &
           ,' current max: ',lmax
   enddo
enddo

return
end

!***************************************************************************

subroutine reset_avg ()

use hyp_coms
use mem_euler
use mem_scratch

implicit none

integer :: nstart,nfinish,ng,n,npts

if (irgrid == 0) then
   nstart=1 ;  nfinish=ngrh
elseif (irgrid < 0) then
   nstart=abs(irgrid)  ;  nfinish=ngrh
elseif( irgrid > 0) then
   nstart=irgrid  ;  nfinish=irgrid
endif

scr%scr1=0.

do ng=nstart,nfinish
   npts=nnzh(ng)*nnxh(ng)*nnyh(ng)
   do n=1,nspecies
      if (ieulout >= 2) call azero(npts,eul(n,ng)%specavg(1,1,1))
      if (ilagout >= 2) call azero(npts,eul(n,ng)%speclavg(1,1,1))
   enddo
enddo

ilavgcall=0
ieavgcall=0

return
end

!***************************************************************************

subroutine lag_conc ()

use hyp_coms
use mem_euler

implicit none


integer :: nstart,nfinish,ng,l,n,i,npts

if (irgrid == 0) then
   nstart=1 ;  nfinish=ngrh
elseif (irgrid < 0) then
   nstart=abs(irgrid)  ;  nfinish=ngrh
elseif( irgrid > 0) then
   nstart=irgrid  ;  nfinish=irgrid
endif

do ng=nstart,nfinish
   npts=nnzh(ng)*nnxh(ng)*nnyh(ng)
   do n=1,nspecies

      ! clear out concentration array
      call azero(npts,eul(n,ng)%specl(1,1,1))
 
      ! for each part, calc conc contribution on each grid
      
      do l=1,npart
         if(atp(l)%nspecies == n .and. atp(l)%ncluster /= 0 .and. &
            ioutspec(n) == 1) then
            
            call lag_to_eul (atp(l)%hi,atp(l)%hj,atp(l)%hk,atp(l)%mass  &
                            ,ng,n,2)
            
         endif
      enddo

      ! mirror boundary values
      call lag_conc_bnd (nnzh(ng),nnxh(ng),nnyh(ng),eul(n,ng)%specl(1,1,1))

   enddo
enddo

return
end

!***************************************************************************

subroutine lag_prob ()

use hyp_coms
use mem_euler

implicit none

integer :: nstart,nfinish,ng,l,n,i,npts

if (irgrid == 0) then
   nstart=1 ;  nfinish=ngrh
elseif (irgrid < 0) then
   nstart=abs(irgrid)  ;  nfinish=ngrh
elseif( irgrid > 0) then
   nstart=irgrid  ;  nfinish=irgrid
endif

do ng=nstart,nfinish
   npts=nnzh(ng)*nnxh(ng)*nnyh(ng)
   do n=1,nspecies

      ! clear out concentration array
      call azero(npts,eul(n,ng)%specl(1,1,1))
 
      ! for each part, calc prob contribution on each grid
      
      do l=1,npart
         if(atp(l)%nspecies == n .and. atp(l)%ncluster /= 0 .and. &
            ioutspec(n) == 1) then
            
            call lag_to_eul (atp(l)%hi,atp(l)%hj,atp(l)%hk,atp(l)%mass  &
                            ,ng,n,3)
            
         endif
      enddo

      ! mirror boundary values
      call lag_conc_bnd (nnzh(ng),nnxh(ng),nnyh(ng)  &
               ,eul(n,ng)%speclprob(1,1,1))

   enddo
enddo

return
end

!***************************************************************************

subroutine lag_conc_bnd (n1,n2,n3,e)

implicit none

integer :: n1,n2,n3,k,i,j
real :: e(n1,n2,n3)

do i=2,n2-1
   do k=2,n1-1
      e(k,i,1)=e(k,i,2)
      e(k,i,n3)=e(k,i,n3-1)
   enddo
enddo

do j=1,n3
   do k=2,n1-1
      e(k,1,j)=e(k,2,j)
      e(k,n2,j)=e(k,n2-1,j)
   enddo
enddo

do i=1,n2
   do j=1,n3
      e(1,i,j)=e(2,i,j)
      e(n1,i,j)=e(n1-1,i,j)
   enddo
enddo

return
end

!***************************************************************************

subroutine part_loc (atp1,atp2,atp3,nh1,nh2,nh3,mg,topth  &
                    ,ihp,jhp,khp,htopo,atp11,atp12,atp13,atp14,mgp)

! gets particle coords on RAMS grid and calcs thickness of layer
! accounting for the particle position and terrain height

use hyp_coms

implicit none

integer :: nh1,nh2,nh3,ihp,jhp,khp,mgp
integer, dimension(nh1,nh2,nh3) :: mg
real :: atp1,atp2,atp3,atp11,atp12,atp13,atp14,htopo
real, dimension(nh1,nh2) :: topth

real :: hg_xy,hg_z,rmg

! particle coords on hypact grid
         
ihp=int(atp1)
jhp=int(atp2)
khp=int(atp3)

! get the rams grid

if(ihp.lt.1.or.jhp.lt.1.or.khp.lt.1.or.  &
   ihp.ge.nih.or.jhp.ge.njh.or.khp.ge.nkh) then
   mgp=-1
else
   mgp=mg(ihp,jhp,khp)
endif

! get the coords, height and topo in m from pole lat long

atp11=hg_xy (nih,xh,atp1)
atp12=hg_xy (njh,yh,atp2)
atp13=hg_xy (nkh,zh,atp3)
atp14=hg_z (nih,njh,nkh,xh,yh,zh,atp1,atp2,atp3,topth,nh1,nh2,ztoph,htopo)

return
end

!***************************************************************************

subroutine sum_prob (ng,nzp,nxp,nyp,a)

! added for particle probability

dimension a(nxp,nyp,nzp)

do i=1,nxp
   do j=1,nyp
      a(i,j,1)=0
      do k=2,nzp
         a(i,j,1)=a(i,j,1)+a(i,j,k)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine iprtpt(n1,n2,n3,ia,i,j,k,str)

dimension ia(n1,n2,n3)
character*(*) str

print '(a,3i4,a,i15)', str,i,j,k,': ',ia(i,j,k)

return
end

!***************************************************************************

subroutine printsl(str,c,n,n1,n2,n3,k1,k2,i1,i2,j1,j2,es)

character*(*) str
dimension es(n1,n2,n3)

do j=j1,j2
   do i=i1,i2
      do k=k1,k2
         if(es(k,i,j) >= c) print '(a,4i4,g12.5)',str,n,k,i,j,es(k,i,j)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine normdist(anorm,nent)

dimension anorm(*)

! This routine generates a table ANORM of normally-distributed
! numbers.  The number of table entries is set by the user in
! NENT which must be set to a prime number and must be less than
! or equal to MAXENT.

mident=nent/2+1
rnenti=1.0/float(nent)
cnorm=1.0/sqrt(2.0*3.14159265)
anorm(mident)=0.0
anorm(mident-1)=-rnenti/0.39894
do ip=mident+1,nent
   xnorm=1.5*anorm(ip-1)-0.5*anorm(ip-2)
   znorm=cnorm*exp(-0.5*xnorm*xnorm)
   anorm(ip)=anorm(ip-1)+rnenti/znorm
   anorm(nent+1-ip)=-anorm(ip)
enddo

return
end

!***************************************************************************

function hkfromz (nkh,zh,z)

dimension zh(*)

if(z.lt.zh(1).or.z.gt.zh(nkh)) stop 'hkfromz: height exceeds grid limits'

klow=1
khigh=nkh
do m=2,11
   kmid=max(2,(klow+khigh)/2)
   msw=int(max(0.,min(1.5,1.e20*(z-zh(kmid)))))
   klow=klow*(1-msw)+kmid*msw
   khigh=max(2,khigh*msw+kmid*(1-msw))
enddo
hkfromz=float(klow)+(z-zh(klow))/(zh(khigh)-zh(klow))

return
end
