!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================


Module hyp_coms

!---------------------------------------------------------------------------
!     HYPACT commons
!---------------------------------------------------------------------------

! Main Eulerian and Lagrangian memory arrays

!real, allocatable :: espec(:,:,:),especsrc(:,:,:),especeavg(:,:,:)
!real, allocatable :: especl(:,:,:),especlavg(:,:,:),especlp(:,:,:)
!real, allocatable :: escr(:)

real, allocatable :: eb(:)

! Particle attribute datatype

type part_att
   real :: hi,hj,hk,upp,vpp,wpp,fallvel,mass,ppm ! atp
   integer :: nrelpart,ncluster               ! latp
   real :: reltime                            ! catp
   integer :: nsource,nspecies                ! lcatp
end type

type(part_att), allocatable :: atp(:)

! hgrid arrays

Type hgrid_vars
integer, pointer, dimension(:,:,:) :: mg
real, pointer, dimension(:,:,:)    :: rim,rjm,rkm,rkt
real, pointer, dimension(:,:)      :: topthg,pctlhg
end type

Type hgridg_vars
real, pointer, dimension(:,:,:)    :: rim,rjm,rkm,rkt
real, pointer, dimension(:,:)      :: topthg,pctlhg
end type

type (hgrid_vars)              :: hg
type (hgridg_vars), allocatable :: hgg(:)


! grid and met variables
!---------------------------------------------------------------------------
integer, parameter :: maxg=10,maxi=400,maxj=400,maxk=100
integer, parameter :: maxhi=400,maxhj=400,maxhk=100
integer, parameter :: maxfiles=400
!---------------------------------------------------------------------------
integer                       :: ngrh,nngh,nnsh,nnpatchh
integer, dimension(maxg)      :: nnxh,nnyh,nnzh
integer, dimension(maxk,maxg) :: nrzh,kpmh
integer, dimension(maxi,maxg) :: ipmh
integer, dimension(maxj,maxg) :: jpmh
real                          :: pslat,pslon
real, dimension(maxg)         :: delxh,delyh,delzh
real, dimension(maxi,maxg)    :: xmh,xth
real, dimension(maxj,maxg)    :: ymh,yth
real, dimension(maxk,maxg)    :: zmh,zth,dzmh,dzth,hth,hwh
!---------------------------------------------------------------------------
real, dimension(maxi,maxg) :: ei1h,ei2h,ei3h
real, dimension(maxj,maxg) :: ej1h,ej2h,ej3h
real, dimension(maxk,maxg) :: ek1h,ek2h,ek3h
!---------------------------------------------------------------------------

integer, dimension(maxg)    :: ie1,in1,iu1,ie1n1,ie1u1,in1u1,ie1n1u1  &
                              ,ie12d,in12d,ie1n12d,in,jn,kn
integer                     :: nih,njh,nkh                              
real                        :: ztoph
real, dimension(maxhi)      :: xh,bizon,dxh,dxhi
real, dimension(maxhj)      :: yh,bjzon,dyh,dyhi
real, dimension(maxhk)      :: zh,dzh,dzhi
real, dimension(maxhi,maxg) :: drim
real, dimension(maxhj,maxg) :: drjm
real, dimension(maxhk,maxg) :: drkm
!---------------------------------------------------------------------------
character(len=24) :: startdate

! general namelist variables
!---------------------------------------------------------------------------
integer, parameter :: maxspec=50,maxsrc=100,polypts=10,maxem=300
!---------------------------------------------------------------------------
character(len=80) :: metpref,hyppref,histpref,histfile
!---------------------------------------------------------------------------
integer :: ihyprun,maxpart,npartbl,ihturb,iadvord,ihybpart,ihybmin,ihgrid  &
          ,izeromet,iwarn
real    :: dtpart,freqavg,hybfreq,hybhoriz,hybvert,hgridtop
!---------------------------------------------------------------------------
integer :: ipartout,ieulout,ilagout,imetout,ioutfmt,irgrid,ilagprob
real    :: hypfreq,histfreq,avgtime
!---------------------------------------------------------------------------

integer :: numanf,ipstfl(maxg),ifutfl(maxg),nend
real :: time_offset
real, dimension(maxfiles) :: ftimes
logical, dimension(maxfiles,maxg) :: fexist
integer, dimension(maxfiles) :: fnums

character(len=14) :: cstart_time,cend_time

integer, dimension(maxg) :: nnfmh,ncnth
integer ::  nestz1,ihtran,if_adap
integer, dimension(maxg) :: nxtnest,ninest,njnest,nknest  &
                           ,nstratx,nstraty,nndtrat,nnsttop,nnstbot
integer, dimension(maxk) :: nstratz1


! database namelist variables
!---------------------------------------------------------------------------
character(len=80) :: gridfile_db(maxsrc),srcfile,specfile,emfile
character(len=30) :: srcname_db(maxsrc),specname_db(maxspec),units_db(maxspec)
character(len=10) :: shape_db(maxsrc),type_db(maxem),scaling_db(maxem)
!---------------------------------------------------------------------------
integer                    :: nspecies_db
integer, dimension(maxsrc) :: ihfall_db,ioutspec_db
real, dimension(maxsrc)    :: wgtmol_db,szpwr_db,szmin_db,szmax_db
!---------------------------------------------------------------------------
integer                         :: nsources_db
real, dimension(maxsrc)         :: srcx_db,srcy_db,srcz_db,xsize_db  &
                                  ,ysize_db,zsize_db,rotation_db
real, dimension(polypts,maxsrc) :: polylat_db,polylon_db
!---------------------------------------------------------------------------
integer                   :: nemissions_db
character(len=19), dimension(maxem) :: irelstrt_db,irelend_db

integer, dimension(maxem) :: iemit_db,isource_db,ispecies_db,numparts_db
real, dimension(maxem)    :: rate_db,ratio_db
!---------------------------------------------------------------------------

! model variables
!---------------------------------------------------------------------------
integer, parameter :: maxent=1009,nscratch=11
!---------------------------------------------------------------------------
character(len=80) :: gridfile(maxsrc)
character(len=30) :: srcname(maxem),specname(maxspec),units(maxspec)
character(len=10) :: sshape(maxem),type(maxem),scaling(maxem)
!---------------------------------------------------------------------------
integer                   :: nspecies
integer, dimension(maxem) :: species,ihfall,ioutspec
real, dimension(maxem)    :: wgtmol,szpwr,szmin,szmax
!---------------------------------------------------------------------------
integer                        :: nsources
integer, dimension(maxem)      :: psource
real, dimension(maxem)         :: srcy,ysize,srcz,srcx,xsize,zsize,rotation
real, dimension(polypts,maxem) :: polylat,polylon
!---------------------------------------------------------------------------
character(len=19)                   :: csimend
character(len=19), dimension(maxem) :: crelstrt,crelend
integer, dimension(maxem) :: numparts
real, dimension(maxem)    :: emission,rparts,ratio
!---------------------------------------------------------------------------
integer                   :: ieuler,ilagran,npart,nent  &
                            ,nrelpart,ieavgcall,ilavgcall
integer, dimension(maxem) :: pspecies
real                      :: hytimemax,hytime,start_time
real, dimension(maxent)   :: anorm
real, dimension(maxem)    :: relstrt,relend,scale,relrate
!---------------------------------------------------------------------------
integer, dimension(maxem)          :: npolypts
real, dimension(polypts,maxem)     :: rpartsp,ratiop,emissionp,scalep
real, dimension(6*polypts+2,maxem) :: xy
!---------------------------------------------------------------------------
integer, dimension(nscratch)     :: iescr
integer, dimension(maxspec,maxg) :: ispec,ispeceavg,ispeclavg,ispecl,ispeclp
integer, dimension(maxg,maxspec) :: mspec,mspeceavg,mspeclavg,mspecl,mspeclp
!---------------------------------------------------------------------------
integer, dimension(maxspec,maxg) :: ibsxh,ibsyh,ibszh
!---------------------------------------------------------------------------
integer :: maxxp,maxyp,maxzp
!---------------------------------------------------------------------------

! clusters
integer, dimension(maxem) :: ncpart,ncclust
integer, allocatable :: ncfill(:)
integer, dimension(polypts,maxem) :: ncpartp,ncclustp
integer :: ncluster,maxclusters

End Module
