!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

Module rams_head

type rheader

   integer :: ngrids,nzg,nzs,npatch,iyear1,imonth1,idate1,itime1  &
             ,ihtran,if_adap
   integer, pointer, dimension(:) :: nnxp,nnyp,nnzp
   character(len=128) :: expnme
   real :: time
   real, pointer, dimension(:) :: deltaxn,deltayn,platn,plonn,slz
   real, pointer, dimension(:,:) :: xtn,xmn,ytn,ymn,ztn,zmn
end type

type(rheader) :: r


Contains

!--------------------------------------------------------------------------

subroutine rams_head_fill()
                        
use hyp_coms                        

implicit none

integer,save :: ncall=0
integer :: ns,jday,rhrs,rmins,rsecs,iiyear1,iimonth1,iidate1,iitime1
integer :: maxx,maxy,maxz,ng,i,j
real :: swx,swy
character(len=14) :: ctime1,ctime2

if (ncall == 0) then
   ncall = 1
   ! Find maximum dimensions
   r%ngrids=ngrh
   maxx=maxval(nnxh(1:ngrh))
   maxy=maxval(nnyh(1:ngrh))
   maxz=maxval(nnzh(1:ngrh))
   
   ! Allocate arrays in header
   allocate(r%nnxp(ngrh),r%nnyp(ngrh),r%nnzp(ngrh))
   allocate(r%deltaxn(ngrh),r%deltayn(ngrh),r%platn(ngrh),r%plonn(ngrh))
   allocate(r%xtn(maxx,ngrh),r%xmn(maxx,ngrh))
   allocate(r%ytn(maxy,ngrh),r%ymn(maxy,ngrh))
   allocate(r%ztn(maxz,ngrh),r%zmn(maxz,ngrh))
   allocate(r%slz(maxz))


   ! Start fillin'. Only need to do the following once since 
   !     they won't change in time.
   r%nnxp(1:ngrh)=nnxh(1:ngrh)
   r%nnyp(1:ngrh)=nnyh(1:ngrh)
   r%nnzp(1:ngrh)=nnzh(1:ngrh)
   r%ngrids=ngrh

   ! Soil/snow stuff are dummies
   r%npatch=nnpatchh
   r%nzg=nngh
   r%nzs=nnsh
   r%slz(r%nzg)=0.
   do ns=r%nzg-1,1,-1
      r%slz(ns)=r%slz(ns+1)-.1
   enddo

   r%ihtran=ihtran
   r%if_adap=if_adap

   r%expnme='HYPACT run'

   ! Get beginning date and time in normal terms!
   read(startdate(1:4),*) r%iyear1 ; read(startdate(6:7),*) r%imonth1
   read(startdate(9:10),*) r%idate1 ; read(startdate(12:15),*) r%itime1

   r%platn(1:r%ngrids)=pslat
   r%plonn(1:r%ngrids)=pslon
   r%deltaxn(1:r%ngrids)=delxh(1:ngrh)
   r%deltayn(1:r%ngrids)=delyh(1:ngrh)
   
   do ng=1,r%ngrids
      r%xtn(1:r%nnxp(ng),ng) = xth(1:r%nnxp(ng),ng)
      r%xmn(1:r%nnxp(ng),ng) = xmh(1:r%nnxp(ng),ng)
      r%ytn(1:r%nnyp(ng),ng) = yth(1:r%nnyp(ng),ng)
      r%ymn(1:r%nnyp(ng),ng) = ymh(1:r%nnyp(ng),ng)
      r%ztn(1:r%nnzp(ng),ng) = zth(1:r%nnzp(ng),ng)
      r%zmn(1:r%nnzp(ng),ng) = zmh(1:r%nnzp(ng),ng)
   enddo
   
endif

! Now do things that change in time...

r%time=hytime


return
end


!--------------------------------------------------------------------------
subroutine rams_head_write(iun)

implicit none

integer :: iun

integer :: irw,ie,ng
integer, external :: cio_i,cio_f,cio_c
character(len=2) :: cng

irw=2
ie=cio_c(iun,irw,'expnme',r%expnme,1)
ie=cio_f(iun,irw,'time',r%time,1)
ie=cio_i(iun,irw,'ngrids',r%ngrids,1)
ie=cio_i(iun,irw,'nnxp',r%nnxp,r%ngrids)
ie=cio_i(iun,irw,'nnyp',r%nnyp,r%ngrids)
ie=cio_i(iun,irw,'nnzp',r%nnzp,r%ngrids)
ie=cio_i(iun,irw,'nzg',r%nzg,1)
ie=cio_i(iun,irw,'nzs',r%nzs,1)
ie=cio_i(iun,irw,'npatch',r%npatch,1)
ie=cio_i(iun,irw,'if_adap',r%if_adap,1)
ie=cio_i(iun,irw,'ihtran',r%ihtran,1)
ie=cio_f(iun,irw,'deltaxn',r%deltaxn,r%ngrids)
ie=cio_f(iun,irw,'deltayn',r%deltayn,r%ngrids)
ie=cio_f(iun,irw,'platn',r%platn,r%ngrids)
ie=cio_f(iun,irw,'plonn',r%plonn,r%ngrids)
ie=cio_i(iun,irw,'iyear1',r%iyear1,1)
ie=cio_i(iun,irw,'imonth1',r%imonth1,1)
ie=cio_i(iun,irw,'idate1',r%idate1,1)
ie=cio_i(iun,irw,'itime1',r%itime1,1)
ie=cio_f(iun,irw,'slz',r%slz,r%nzg)

do ng=1,r%ngrids
   write(cng,1) ng
1       format(i2.2)
   ie=cio_f(iun,irw,'xmn'//cng,r%xmn(1,ng),r%nnxp(ng))
   ie=cio_f(iun,irw,'xtn'//cng,r%xtn(1,ng),r%nnxp(ng))
   ie=cio_f(iun,irw,'ymn'//cng,r%ymn(1,ng),r%nnyp(ng))
   ie=cio_f(iun,irw,'ytn'//cng,r%ytn(1,ng),r%nnyp(ng))
   ie=cio_f(iun,irw,'zmn'//cng,r%zmn(1,ng),r%nnzp(ng))
   ie=cio_f(iun,irw,'ztn'//cng,r%ztn(1,ng),r%nnzp(ng))
   ie=cio_f(iun,irw,'slz'//cng,r%slz(1),r%nzg)
enddo



return
end

end Module




