subroutine draw_colorbar(iwk,collab,numboxes, conlo,coninc,midpoint  &
         ,px1,px2,py1,py2,ipanl,ihalfbar,cdunits,labpos,ifillbeg)

! iwk - workstation id
! collab - label color
! conlo - lowest value 
! coninc - value increment
! midpoint - "middle" value, can be zero if not defined
! px1,px2,py1,py2 - plotter coords of last plot
! ipanl - panel number, 1 if upper left or only panel
! ihalfbar - =0 - full bar,  =1 - half bar top,  =2 - half bar bottom
! cdunits - variable units
! labpos - 0=labels at lines,  1=labels at box midpoint
! ifillbeg - color index where fill colors begin

use define_colors

implicit none

integer :: iwk,ipanl,numboxes,ihalfbar,midpoint,labpos,ifillbeg
character(len=*) :: collab,cdunits
real :: conlo,coninc,px1,px2,py1,py2

integer :: ic,labstep,ln,ib,ici
real :: sszz,step,cbargap,yinc,valuelab,offset
character(len=16) :: numbr,number,cform
real :: rif(6),rjf(6),dst(6),ind(8)

call set(0.,1.,0.,1.,0.,1.,0.,1.,1)


! Set label color index
ic=5
call coloralias_set(iwk,collab,ic)
call gsplci(ic)
call gstxci(ic)


! Find individual box size

! x plotting coords
rif(1)=px2+.01
if(ipanl == 1) then
   rif(2)=rif(1)+.03
   sszz=-.63
   step=min(1.,(py2-py1)/(px2-px1))
else
   rif(2)=rif(1)+.01
   sszz=-.55
   step=min(1.,.4*(py2-py1)/(px2-px1))
endif
rif(3)=rif(2)
rif(4)=rif(1)
rif(5)=rif(1)

! y plotting coords and label step
if(ihalfbar >= 1) then
   cbargap=0.05
   if(ihalfbar == 1) then
      rjf(1)=py1+(py2-py1)/2+cbargap/2
   else
      rjf(1)=py1
   endif
   yinc=(py2-cbargap-py1)/(float(numboxes)*2)
   labstep=max(1,nint(float(numboxes)/(10.*step)))
else
   rjf(1)=py1
   yinc=(py2-py1)/float(numboxes)
   labstep=max(1,nint(float(numboxes)/(20.*step)))
endif
rjf(2)=rjf(1)
rjf(5)=rjf(1)


! Starting value- lowest label

valuelab=conlo

if(mod(midpoint,labstep) == 0 .and. labpos == 0 .and.ipanl == 1) then
   
   call nice_number_format(coninc,cform)
   write(number,trim(cform)) valuelab

   call deblank(number,numbr,ln)
   if(numbr(ln:ln)=='.') ln=len_trim(numbr)-1

   call plchlq (rif(2)+.01,rjf(1),numbr(1:ln),sszz,0.,-1.)
   
   valuelab=valuelab+coninc
   
endif


! Plot units
if(ipanl == 1) then
   call plchlq (rif(2)+.01,rjf(2)-0.025,trim(cdunits),sszz,0.,-1.)
else
   call plchlq (rif(2)+.01,rjf(2)+0.01,trim(cdunits),sszz,0.,-1.)
endif

! Label offset from line between boxes
offset=0.
if(labpos == 1) offset=-.5*yinc

! Main loop - draw labels and fill boxes
do ib=1,numboxes

   rjf(3)=rjf(2)+yinc
   rjf(4)=rjf(3)
   
   !colors
   ici=ib+ifillbeg

   call sfsgfa (rif,rjf,4,dst,6,ind,8,ici)

   if(mod(midpoint-ib,labstep) == 0) then
   
      call nice_number_format(coninc,cform)
      write(number,trim(cform)) valuelab

      call deblank(number,numbr,ln)
      if(numbr(ln:ln)=='.') ln=len_trim(numbr)-1
  
      call plchlq (rif(2)+.01,rjf(3)+offset,numbr(1:ln),sszz,0.,-1.)

   endif

   rjf(1)=rjf(3)
   rjf(2)=rjf(3)
   rjf(5)=rjf(3)
   
   valuelab=valuelab+coninc

enddo

return
end


subroutine nice_number_format(xval,cform)
implicit none
real :: xval
character(len=*) :: cform

! Return a format that will nicely print the value

if    (xval >= .01 .and. xval < .1) then
   cform='(f6.3)'
elseif(xval >= .1  .and. xval < 1.) then
  cform='(f6.2)'
elseif(xval >= 1.  .and. xval < 100.) then
   cform='(f6.1)'
elseif(xval >= 100.  .and. xval < 10000.) then
   cform='(f6.0)'
else
   cform='(e8.2)'
endif

return
end

