Module define_colors


! Manipulates and retrieves information from a color table.
!   A color table defined by name and rgb is maintained with these routines.
!   Other routines will make these known to GKS.

type colortable

   character(len=16) :: name
   real :: rgb(1:3)
end type

type aliastable
   character(len=16) :: name
   integer :: ialias
end type

integer, parameter :: maxcolors=256

type (colortable) :: colortab(maxcolors)
type (aliastable) :: colalias(maxcolors)


integer :: ncolors=0,nalias=0

Contains

!-------------------------------------------

subroutine colortab_init ()

implicit none


! Define first 40 default colors

call colortab_add ('black      ','rgb',0.,0.,0.)   
call colortab_add ('white      ','rgb',1.,1.,1.)   
call colortab_add ('gray10  ','rgb',.1,.1,.1)   
call colortab_add ('gray20  ','rgb',.2,.2,.2)   
call colortab_add ('gray30  ','rgb',.3,.3,.3)   
call colortab_add ('gray40  ','rgb',.4,.4,.4)   
call colortab_add ('gray50  ','rgb',.5,.5,.5)   
call colortab_add ('gray60  ','rgb',.6,.6,.6)   
call colortab_add ('gray70  ','rgb',.7,.7,.7)   
call colortab_add ('gray80  ','rgb',.8,.8,.8)   
call colortab_add ('gray90  ','rgb',.9,.9,.9)   
call colortab_add ('darkgray   ','rgb',.65,.65,.65)
call colortab_add ('gray       ','rgb',.5,.5,.5)   
call colortab_add ('lightgray  ','rgb',.8,.8,.8)   

call colortab_add ('darkred    ','rgb',.5,0.,0.)   
call colortab_add ('red        ','rgb',1.,0.,0.)   
call colortab_add ('midred     ','rgb',1.,.65,.65)  
call colortab_add ('lightred   ','rgb',1.,.8,.8)    

call colortab_add ('darkgreen  ','rgb',0.  ,.5  ,0. )    
call colortab_add ('green      ','rgb',0.  ,1.  ,0. )    
call colortab_add ('midgreen   ','rgb', .65,1.  ,.65)  
call colortab_add ('lightgreen ','rgb', .8 , 1., .8 )    
call colortab_add ('forestgreen','rgb', .14,.56, .14) 
call colortab_add ('shadygreen ','rgb',0.  ,.7 ,0.  )    

call colortab_add ('darkblue   ','rgb',0.,0.,.5)    
call colortab_add ('blue       ','rgb',0.,0.,1.)    
call colortab_add ('midblue    ','rgb',.65,.65,1.)  
call colortab_add ('lightblue  ','rgb',.8,.8,1.)    
call colortab_add ('deepskyblue','rgb',.00,.75,1.0) 
call colortab_add ('skyblue    ','rgb',.20,.56,.80) 
call colortab_add ('royalblue  ','rgb',.25,.45,.95) 
call colortab_add ('slateblue  ','rgb',.40,.35,.80) 
call colortab_add ('cyan       ','rgb',0.,1.,1.)    
call colortab_add ('aqua       ','rgb',0.1,1.0,.85) 

call colortab_add ('yellow     ','rgb',1.,1.,0.)    
call colortab_add ('yellowgreen','rgb',.5,1.,0.)    
call colortab_add ('greenyellow','rgb',.70,1.0,0.2) 

call colortab_add ('purple     ','rgb',1.,0.,1.)    
call colortab_add ('lavender   ','rgb',0.8,0.8,1.0) 
call colortab_add ('darkviolet ','rgb',0.6,0.0,0.8) 
call colortab_add ('bluemagenta','rgb',0.5,0.0,1.0) 
call colortab_add ('magenta    ','rgb',1.0,0.0,1.0) 

call colortab_add ('tan        ','rgb',.86,.58,.44) 
call colortab_add ('wheat      ','rgb',0.5,0.45,0.3) 
call colortab_add ('brown      ','rgb',.65,.16,.16) 
call colortab_add ('orangered  ','rgb',1.0,0.0,0.2) 
call colortab_add ('orange     ','rgb',1.0,.50,0.0) 
call colortab_add ('gold       ','rgb',1.0,.85,0.0) 

return
end subroutine

!-------------------------------------------

subroutine colortab_print ()

implicit none

integer :: ic,i
real :: h,s,v
! print the color table

print*,ncolors
do ic=1,ncolors
   print'(i3,3f8.2,a1,a8)',ic,(colortab(ic)%rgb(i),i=1,3),' ',trim(colortab(ic)%name)
   call colortab_vals('hsv',colortab(ic)%name,h,s,v)
   print'(i3,3f8.2,a1,a8)',ic,h,s,v,' ',trim(colortab(ic)%name)
enddo

return
end subroutine
   
!-------------------------------------------

subroutine colortab_add (coloradd,cmod,c1,c2,c3)

implicit none
character(len=*) :: coloradd,cmod
real :: c1,c2,c3,h,l,s,r,g,b

integer :: ic
   
! add a new named color to the color table
   !if(coloradd(1:4) == 'tile') print*,'tileadd:',coloradd,c1,c2,c3

! See if this color already exists. If so, give it new values
do ic=1,ncolors
   if(trim(coloradd) == trim(colortab(ic)%name)) then
      if(cmod(1:3) == 'hls') then
         call hlsrgb (c1,c2,c3,r,g,b)
      elseif(cmod(1:3) == 'hsv') then
         call hsvrgb (c1,c2,c3,r,g,b)
      elseif(cmod(1:3) == 'rgb') then
         r=c1
         g=c2
         b=c3
      endif
      colortab(ic)%rgb(1)=r
      colortab(ic)%rgb(2)=g
      colortab(ic)%rgb(3)=b
      return
   endif
enddo

! The color is actually new. Set it up...
ncolors=ncolors+1
if(ncolors > maxcolors) then
   print*,'Cannot allocate any more colors - limit: ',maxcolors
   stop 'colortab_add'
endif
if(cmod(1:3) == 'hls') then
   call hlsrgb (c1,c2,c3,r,g,b)
elseif(cmod(1:3) == 'hsv') then
   call hsvrgb (c1,c2,c3,r,g,b)
   !if(coloradd(1:4) == 'tile') print*,'tileadd:',coloradd,h,l,s,c1,c2,c3
elseif(cmod(1:3) == 'rgb') then
   r=c1
   g=c2
   b=c3
endif
ic=ncolors
colortab(ic)%name=coloradd
colortab(ic)%rgb(1)=r
colortab(ic)%rgb(2)=g
colortab(ic)%rgb(3)=b
print*,'add:',ic,r,g,b,coloradd,cmod

!call colortab_print()
!pause


return
end subroutine

!-------------------------------------------

subroutine colortab_vals (cmod,color,c1,c2,c3)

implicit none
character(len=*) :: color,cmod
real :: c1,c2,c3

integer :: ic
   
! input a color name and return rgb or hls (cmod)

do ic=1,ncolors
   if(trim(color) == trim(colortab(ic)%name)) then
      if(cmod(1:3) == 'hls') then
         call rgbhls (colortab(ic)%rgb(1),colortab(ic)%rgb(2)  &
                     ,colortab(ic)%rgb(3),c1,c2,c3)
      elseif(cmod(1:3) == 'hsv') then
         call rgbhsv (colortab(ic)%rgb(1),colortab(ic)%rgb(2)  &
                     ,colortab(ic)%rgb(3),c1,c2,c3)
      elseif(cmod(1:3) == 'rgb') then
         c1=colortab(ic)%rgb(1)
         c2=colortab(ic)%rgb(2)
         c3=colortab(ic)%rgb(3)
      endif
!print*,'get:',ic,c1,c2,c3,color,cmod
      return
   endif
enddo

print*,'colortab_vals: Could not find color: ',color
stop 'colortab_vals'

return
end subroutine
   
!-------------------------------------------

subroutine colortab_ind (colorind,icolor)

implicit none
character(len=*) :: colorind
integer :: icolor

integer :: ic
   
! get the color table integer index of a color

if(len_trim(colortab(1)%name) == 0) then  ! color table not defined yet
   icolor=1
   return
endif
do ic=1,ncolors
   if(trim(colorind) == trim(colortab(ic)%name)) then
      icolor=ic
      return
   endif
enddo
print*,'Could not find color: ',colorind
stop 'colortab_ind'

return
end subroutine

   
!-------------------------------------------

subroutine colortab_col (colorcol,icolor)

implicit none
character(len=*) :: colorcol
integer :: icolor

integer :: ic
   
! get the color name from the integer index

if(icolor > maxcolors) then
   print*,'Requesting color index above maxcols: ',icolor,maxcolors
   stop 'colortab_col'
endif
colorcol=colortab(icolor)%name
if(colorcol == '') colorcol='unset'

return
end subroutine

!-------------------------------------------

subroutine colortab_set (iwk,color,ind)

implicit none
character(len=*) :: color
integer :: ind,iwk

integer :: ic
   
! Find the color integer index
do ic=1,ncolors
   if(trim(color) == trim(colortab(ic)%name)) then
      ! Got it.... find actual color
      ! Make it known to GKS if desired
      if(ind >= 0) call gscr (iwk,ind,colortab(ic)%rgb(1),colortab(ic)%rgb(2)  &
                     ,colortab(ic)%rgb(3))
      return
   endif
enddo
print*,'Could not find color: ',color
stop 'colortab_set'

return
end subroutine

   
!-------------------------------------------

subroutine coloralias_add (acolor,color)

implicit none
character(len=*) :: color,acolor

integer :: ic,ia
   
! set up a color alias

! The aliased color must be an existing color in colortab
do ic=1,ncolors
   if(trim(color) == trim(colortab(ic)%name)) then
      ! Found it..... now see if this alias is already used
      do ia=1,nalias
      !print*,'check old alias add:',ia,ic,trim(color),trim(colalias(ia)%name)
         if(trim(acolor) == trim(colalias(ia)%name)) then
            ! Yes it is...
            colalias(ia)%ialias=ic
      !print*,'old alias add:',ia,colortab(ia)%rgb(1),colortab(ia)%rgb(2)  &
      !               ,colortab(ia)%rgb(3),acolor
            return
         endif
      enddo
      
      ! No it isn't....
      nalias=nalias+1
      if(nalias > maxcolors) then
         print*,'Cannot allocate any more alias colors - limit: ',maxcolors
         stop 'coloralias_add'
      endif
      colalias(nalias)%name=acolor
      colalias(nalias)%ialias=ic
      !print*,'new alias add:',ia,colortab(ia)%rgb(1),colortab(ia)%rgb(2)  &
      !               ,colortab(ia)%rgb(3),acolor
      return
   endif
enddo
print*,'Requested alias color doesn''t exist: ',acolor,'  ',color
stop 'coloralias_add'


return
end subroutine
   
!-------------------------------------------

subroutine coloralias_set (iwk,acolor,ind)

implicit none
character(len=*) :: acolor
integer :: iwk,ind

integer :: ia,ic

! Find the alias integer index
do ic=1,nalias
   if(trim(acolor) == trim(colalias(ic)%name)) then
      ! Got it.... find actual color
      ia=colalias(ic)%ialias
      ! Make it known to GKS if desired
      if(ind >= 0) call gscr (iwk,ind,colortab(ia)%rgb(1),colortab(ia)%rgb(2)  &
                     ,colortab(ia)%rgb(3))
                     
                     
     ! print*,'alias set:',ia,ind,colortab(ia)%rgb(1),colortab(ia)%rgb(2)  &
     !                ,colortab(ia)%rgb(3),acolor
      return
   endif
enddo

! If we got here, we didn't find an alias. Look for an actual color...
do ic=1,ncolors
   if(trim(acolor) == trim(colortab(ic)%name)) then
      ! Make it known to GKS if desired
      if(ind >= 0) call gscr (iwk,ind,colortab(ic)%rgb(1),colortab(ic)%rgb(2)  &
                     ,colortab(ic)%rgb(3))
                     
                     
      !print*,'alias/actual set:',ia,ind,colortab(ic)%rgb(1),colortab(ic)%rgb(2)  &
      !               ,colortab(ic)%rgb(3),acolor
      return
   endif
enddo


print*,'Could not find color: ',acolor
stop 'coloralias_set'


return
end subroutine

End Module
