!############################# Change Log ##################################
! 3.0.0
!
! 000829 CJT RAMS_fill_fld ##
!            Changed from passing arguments to using an_header module. ##
!
!###########################################################################
!  Copyright (C)  1990, 1995, 1999, 2000 - All Rights Reserved
!  Regional Atmospheric Modeling System - RAMS
!  Mission Research Corporation / *ASTeR Division
!###########################################################################


!---------------------------------------------------------------------------

subroutine read_pl_fields(nf,cvar,icfile,icgrid,ictrans,iccoor,icvar  &
                         ,cwinds,icwinds,itrans)

use pl_arrays

implicit none

integer :: nf,icfile,icgrid,ictrans,iccoor,icvar,itrans,icwinds
character(len=*) :: cvar(*), cwinds

integer :: ngr,nv

ngr = f(nf)%nvgr

! Read the following if the grid number of file number has changed. Only checking
!   file number if numbers of grids or configuration has changed. It could be 
!   automatically checked to determine if we need to check this.

!print*,'flnm_base:',f(nf)%flnm_base
!print*,'flnm_base:',trim(f(nf)%flnm_base), '-----'
!print*,'flnm_base:',f(nf)%iv_topo, allocated(f(nf)%vars(f(nf)%iv_topo)%arr)

if(icfile == 1.or.icgrid == 1)  then
   ! Read the topo - used for everything
   call RAMS_varlib (nf,'topo',f(nf)%vars(f(nf)%iv_topo)%arr(1) &
                     ,f(nf)%flnm_base)
   call get_var_info( f(nf)%vars(f(nf)%iv_topo)%ivartype &
                     ,f(nf)%vars(f(nf)%iv_topo)%cdname   &
                     ,f(nf)%vars(f(nf)%iv_topo)%cdunits)  
                      
   ! Read topta - topography if ADAP is used
   call RAMS_varlib (nf,'topta',f(nf)%vars(f(nf)%iv_topta)%arr(1) &
                     ,f(nf)%flnm_base)
   call get_var_info( f(nf)%vars(f(nf)%iv_topta)%ivartype &
                     ,f(nf)%vars(f(nf)%iv_topta)%cdname   &
                     ,f(nf)%vars(f(nf)%iv_topta)%cdunits)  

   ! Read volt - grid cell volume at T point if ADAP is used
   call RAMS_varlib (nf,'volt',f(nf)%vars(f(nf)%iv_volt)%arr(1) &
                     ,f(nf)%flnm_base)
   call get_var_info( f(nf)%vars(f(nf)%iv_volt)%ivartype &
                     ,f(nf)%vars(f(nf)%iv_volt)%cdname   &
                     ,f(nf)%vars(f(nf)%iv_volt)%cdunits)  
   print*,'read VOLT:',maxval(f(nf)%vars(f(nf)%iv_volt)%arr(1:  &
   f(nf)%nnxp(ngr)*f(nf)%nnyp(ngr)*f(nf)%nnzp(ngr)))  
   print*,'read VOLT:',minval(f(nf)%vars(f(nf)%iv_volt)%arr(1:  &
   f(nf)%nnxp(ngr)*f(nf)%nnyp(ngr)*f(nf)%nnzp(ngr)))  
   
   ! Read the land percentage - used on vertical cross-sections
   call RAMS_varlib (nf,'land',f(nf)%vars(f(nf)%iv_pctlnd)%arr(1) &
                     ,f(nf)%flnm_base)
   call get_var_info( f(nf)%vars(f(nf)%iv_pctlnd)%ivartype &
                     ,f(nf)%vars(f(nf)%iv_pctlnd)%cdname   &
                     ,f(nf)%vars(f(nf)%iv_pctlnd)%cdunits)  

   ! Read the LEAF patch area
   call RAMS_varlib (nf,'patch_area',f(nf)%vars(f(nf)%iv_parea)%arr(1) &
                     ,f(nf)%flnm_base)
   call get_var_info( f(nf)%vars(f(nf)%iv_parea)%ivartype &
                     ,f(nf)%vars(f(nf)%iv_parea)%cdname   &
                     ,f(nf)%vars(f(nf)%iv_parea)%cdunits)  

endif

! Read the user variables
if(icfile == 1.or.icgrid == 1.or.icvar == 1.or.  &
   ictrans == 1.or.(iccoor == 1) ) then    ! why iccoor here, only for HYP parts???
   
   do nv = 1, maxplvar
print*,'rpl2',nv,cvar(nv)(1:4)
      if (cvar(nv)(1:4) /= 'none'.and.cvar(nv)(1:1) /= ' ' .and. &
          cvar(nv)(1:1) /= char(0) ) then
      
         if(cvar(nv)(1:8) == 'part_lag') then 
           ! call HYP_fill_fld (1,f(nf)%vars(nv)%ivartype,f(nf)%flnm_base)
         else
         
            call RAMS_fill_fld (nf &
                      ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
                      ,f(nf)%vars(nv)%arr(1),scr%scr1(1),scr%scr2(1)  &
                      ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
                      ,f(nf)%flnm_base,ngr,cvar(nv),itrans  &
                      ,f(nf)%vars(nv)%cdname  &
                      ,f(nf)%vars(nv)%cdunits,  f(nf)%vars(nv)%ivartype)
         endif
         
         !  If variable not found...
         if(f(nf)%vars(nv)%ivartype == 0) then
            print*,'Variable not found: ',trim(cvar(nv))
            f(nf)%vars(nv)%cdname = trim(cvar(nv))//' not found'
            cycle
         endif

      endif
   
   enddo

endif
   
! If winds (or turb or vorticity) are being plotted, read them now.
!   We are reading all three components, but only two are used.
if((cwinds /= 'n'.and.cwinds /= 'r').and.(icfile == 1.or.  &
   icgrid == 1.or.icwinds == 1.or.ictrans == 1.or.iccoor == 1)) then

   call RAMS_fill_fld (nf &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_u)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'u_avg',itrans  &
             ,f(nf)%vars(f(nf)%iv_u)%cdname  &
             ,f(nf)%vars(f(nf)%iv_u)%cdunits,  f(nf)%vars(f(nf)%iv_u)%ivartype)
   call RAMS_fill_fld (nf  &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_v)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'v_avg',itrans  &
             ,f(nf)%vars(f(nf)%iv_v)%cdname  &
             ,f(nf)%vars(f(nf)%iv_v)%cdunits,  f(nf)%vars(f(nf)%iv_v)%ivartype)
   call RAMS_fill_fld (nf  &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_w)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'w_avg',itrans  &
             ,f(nf)%vars(f(nf)%iv_w)%cdname  &
             ,f(nf)%vars(f(nf)%iv_w)%cdunits,  f(nf)%vars(f(nf)%iv_w)%ivartype)

   ! Add these to carry uu,vv,ww around
   if(cwinds == 't') then
      call RAMS_fill_fld (nf  &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_tu)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'uu',itrans  &
             ,f(nf)%vars(f(nf)%iv_tu)%cdname  &
             ,f(nf)%vars(f(nf)%iv_tu)%cdunits,  f(nf)%vars(f(nf)%iv_tu)%ivartype)
      call RAMS_fill_fld (nf  &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_tv)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'vv',itrans  &
             ,f(nf)%vars(f(nf)%iv_tv)%cdname  &
             ,f(nf)%vars(f(nf)%iv_tv)%cdunits,  f(nf)%vars(f(nf)%iv_tv)%ivartype)
      call RAMS_fill_fld (nf  &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_tw)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'ww',itrans  &
             ,f(nf)%vars(f(nf)%iv_tw)%cdname  &
             ,f(nf)%vars(f(nf)%iv_tw)%cdunits,  f(nf)%vars(f(nf)%iv_tw)%ivartype)
   endif

elseif(cwinds == 'r'.and.(icfile == 1.or.icgrid == 1.or.  &
   icwinds == 1.or.ictrans == 1.or.iccoor == 1)) then

   call RAMS_fill_fld (nf  &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_u)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'relvortx',itrans  &
             ,f(nf)%vars(f(nf)%iv_u)%cdname  &
             ,f(nf)%vars(f(nf)%iv_u)%cdunits,  f(nf)%vars(f(nf)%iv_u)%ivartype)
   call RAMS_fill_fld (nf  &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_v)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'relvorty',itrans  &
             ,f(nf)%vars(f(nf)%iv_v)%cdname  &
             ,f(nf)%vars(f(nf)%iv_v)%cdunits,  f(nf)%vars(f(nf)%iv_v)%ivartype)
   call RAMS_fill_fld (nf  &
             ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr) &
             ,f(nf)%vars(f(nf)%iv_w)%arr(1),scr%scr1(1),scr%scr2(1)  &
             ,f(nf)%vars(f(nf)%iv_topo)%arr(1)  &
             ,f(nf)%flnm_base,ngr,'relvortz',itrans  &
             ,f(nf)%vars(f(nf)%iv_w)%cdname  &
             ,f(nf)%vars(f(nf)%iv_w)%cdunits,  f(nf)%vars(f(nf)%iv_w)%ivartype)
endif


return
end

!---------------------------------------------------------------------------

subroutine slice_pl_fields(nf,nib,nie,njb,nje,icoor,nii,njj  &
                    ,cvar,cwinds,islab,icoord)

use pl_arrays

implicit none

integer :: nf,nib,nie,njb,nje,icoor,nii,njj,icoord,islab
character(len=*) :: cvar(*),cwinds

integer :: ngr,nv, leafdim

ngr = f(nf)%nvgr

! Slice primary and secondary field

! Fix this messy crap for tile plots!!!!!


do nv = 1, maxplvar
   if (cvar(nv)(1:4) /= 'none'.and.cvar(nv)(1:1) /= ' ' .and. &
       cvar(nv)(1:1) /= char(0) ) then
print*,'^^^^^^slice:',f(nf)%vars(nv)%cdname,f(nf)%vars(nv)%ivartype

      if(f(nf)%vars(nv)%ivartype == 2 .or. f(nf)%vars(nv)%ivartype == 3 ) then
         icoord=icoor    
         if(f(nf)%vars(nv)%ivartype == 2) icoord=1
         call RAMS_3to2d (islab,f(nf)%vars(nv)%arr(1)  &
                         ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr)  &
                         ,f(nf)%vars(nv)%slab(1),nib,nie,njb,nje,icoord,nii,njj)
      elseif(f(nf)%vars(nv)%ivartype == 6) then
         icoord=1
         call RAMS_3to2d_hpat (  &
               f(nf)%vars(nv)%arr(1),f(nf)%vars(f(nf)%iv_parea)%arr(1) &
              ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%npatch  &
              ,f(nf)%vars(nv)%slab(1),f(nf)%vars(f(nf)%iv_parea)%slab(1) &
              ,nib,nie,njb,nje,nii,njj)

      elseif (f(nf)%vars(nv)%ivartype == 8 .or. f(nf)%vars(nv)%ivartype==5) then
         icoord=icoor
         call RAMS_3to2d_vpat (islab,f(nf)%vars(nv)%arr(1) &
               ,f(nf)%vars(f(nf)%iv_parea)%arr(1)  &
               ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nzg,f(nf)%npatch  &
               ,f(nf)%vars(nv)%slab(1),f(nf)%vars(f(nf)%iv_parea)%slab(1) &
               ,nib,nie,njb,nje,icoord,nii,njj)
      elseif (f(nf)%vars(nv)%ivartype == 7 .or. f(nf)%vars(nv)%ivartype==4) then
         icoord=icoor
         call RAMS_3to2d_vpat (islab,f(nf)%vars(nv)%arr(1)  &
               ,f(nf)%vars(f(nf)%iv_parea)%arr(1)  &
               ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nzs,f(nf)%npatch  &
               ,f(nf)%vars(nv)%slab(1),f(nf)%vars(f(nf)%iv_parea)%slab(1) &
               ,nib,nie,njb,nje,icoord,nii,njj)
      else
         print*,'slice_pl_fields:f(nf)%vars(nv)%ivartype:',f(nf)%vars(nv)%ivartype
         stop 'slice_pl_fields'
      endif
                          
   endif
enddo


! Slice wind fields
if(cwinds /= 'n') then
   icoord=icoor
   call RAMS_3to2d_v (islab,f(nf)%vars(f(nf)%iv_u)%arr(1)  &
                   ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr)  &
                   ,f(nf)%vars(f(nf)%iv_u)%slab(1),nib,nie,njb,nje,icoord,nii,njj)
   call RAMS_3to2d_v (islab,f(nf)%vars(f(nf)%iv_v)%arr(1)  &
                   ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr)  &
                   ,f(nf)%vars(f(nf)%iv_v)%slab(1),nib,nie,njb,nje,icoord,nii,njj)
   call RAMS_3to2d_v (islab,f(nf)%vars(f(nf)%iv_w)%arr(1)  &
                   ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr)  &
                   ,f(nf)%vars(f(nf)%iv_w)%slab(1),nib,nie,njb,nje,icoord,nii,njj)

   ! Also added to carry around turbulence uu,vv,ww
   if(cwinds == 't') then
   call RAMS_3to2d_v (islab,f(nf)%vars(f(nf)%iv_tu)%arr(1)  &
                   ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr)  &
                   ,f(nf)%vars(f(nf)%iv_tu)%slab(1),nib,nie,njb,nje,icoord,nii,njj)
   call RAMS_3to2d_v (islab,f(nf)%vars(f(nf)%iv_tv)%arr(1)  &
                   ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr)  &
                   ,f(nf)%vars(f(nf)%iv_tv)%slab(1),nib,nie,njb,nje,icoord,nii,njj)
   call RAMS_3to2d_v (islab,f(nf)%vars(f(nf)%iv_tw)%arr(1)  &
                   ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr)  &
                   ,f(nf)%vars(f(nf)%iv_tw)%slab(1),nib,nie,njb,nje,icoord,nii,njj)

   endif
endif

if(f(nf)%if_adap == 1) then
   ! Slice volt array
   call RAMS_3to2d (islab,f(nf)%vars(f(nf)%iv_volt)%arr(1)  &
                   ,f(nf)%nnxp(ngr),f(nf)%nnyp(ngr),f(nf)%nnzp(ngr)  &
                   ,f(nf)%vars(f(nf)%iv_volt)%slab(1),nib,nie,njb,nje &
                   ,icoord,nii,njj)
endif

return
end

!----------------------------------------------------------------------------------

subroutine RAMS_fill_fld (nf,n1,n2,n3,a,b,bb,topo,prefile  &
                         ,ngrd,cvar,itrans,cdname,cdunits,ivtype)

use var_info
                         
implicit none

integer :: nf,n1,n2,n3,ngrd,itrans,ivtype,nval
          
real :: a(*),b(*),bb(*),topo(*)
character(len=*) :: prefile,cvar,cdname,cdunits

real, allocatable :: fzt(:)
real :: fztop

call RAMS_varlib(nf,cvar,a,prefile)
call get_var_info(ivtype,cdname,cdunits)

if(itrans == 2.and.ivtype == 3) then
   if(allocated(fzt)) deallocate(fzt) ; allocate(fzt(n3))
   call ra_get_fdata(0,ngrd,fzt(1),nval)
   call ra_get_fdata(3,ngrd,fztop,nval)
   call RAMS_Ctrans(n1,n2,n3,a,b,topo,fzt,fztop)
elseif(itrans == 3.and.ivtype == 3) then
   call RAMS_varlib(nf,'pi',bb,prefile)
   call RAMS_Ptrans(n1,n2,n3,a,bb,b)
endif

return
end

!***************************************************************************

subroutine RAMS_3to2d (islab,indata,ni1,ni2,ni3  &
                      ,outdata,id1,id2,jd1,jd2,numsl,nii,njj)
                         
implicit none
     
integer :: islab ,ni1,ni2,ni3,id1,id2,jd1,jd2,numsl,nii,njj
real indata(ni1,ni2,ni3)
real outdata(nii,njj)

integer :: i,j,k

if(islab == 3) then
   do j=jd1,jd2
      do i=id1,id2
         outdata(i-id1+1,j-jd1+1)=indata(i,j,numsl)
      enddo
   enddo
elseif(islab == 1) then
   do k=jd1,jd2
      do i=id1,id2
         outdata(i-id1+1,k-jd1+1)=indata(i,numsl,k)
      enddo
   enddo
   ! adjust lowest level outdata values if lowest plotting level is
   ! on the ground (jd1=1)
   if(jd1 == 1) then
      do i=id1,id2
         outdata(i-id1+1,1)=(outdata(i-id1+1,1)+outdata(i-id1+1,2))/2
      enddo
   endif
elseif(islab == 2) then
   do k=jd1,jd2
      do j=id1,id2
         outdata(j-id1+1,k-jd1+1)=indata(numsl,j,k)
      enddo
   enddo
   ! adjust lowest level outdata values if lowest plotting level is
   ! on the ground (jd1=1)
   if(jd1 == 1) then
      do j=id1,id2
         outdata(j-id1+1,1)=(outdata(j-id1+1,1)+outdata(j-id1+1,2))/2
      enddo
   endif
endif

return
end

!***************************************************************************

subroutine RAMS_3to2d_v (islab,indata,ni1,ni2,ni3  &
                        ,outdata,id1,id2,jd1,jd2,numsl,nii,njj)
                         
implicit none

integer :: islab ,ni1,ni2,ni3,id1,id2,jd1,jd2,numsl,nii,njj
real :: indata(ni1,ni2,ni3)
real ::outdata(0:nii,0:njj)

integer :: iso,jso,i,j,k

iso = 0
jso = 0
if (id1 >= 2) iso = 1
if (jd1 >= 2) jso = 1

if(islab == 3) then
   do j=jd1-jso,jd2
      do i=id1-iso,id2
         outdata(i-id1+1,j-jd1+1)=indata(i,j,numsl)
      enddo
   enddo
elseif(islab == 1) then
   do k=jd1-jso,jd2
      do i=id1-iso,id2
         outdata(i-id1+1,k-jd1+1)=indata(i,numsl,k)
      enddo
   enddo
elseif(islab == 2) then
   do k=jd1-jso,jd2
      do j=id1-iso,id2
         outdata(j-id1+1,k-jd1+1)=indata(numsl,j,k)
      enddo
   enddo
endif

return
end

!***************************************************************************

subroutine RAMS_3to2d_vpat (islab,indata,inarea,ni1,ni2,ni3,np  &
                           ,outdata,outarea,id1,id2,jd1,jd2,numsl  &
                           ,nii,njj,ivtype)
                         
implicit none
   
integer :: islab ,ni1,ni2,ni3,id1,id2,jd1,jd2,numsl,nii,njj,np,ivtype
real :: indata(ni1,ni2,ni3,np),inarea(ni1,ni2,np)
real :: outdata(nii,njj,np),outarea(nii,njj,np)

integer :: ip,k,i,j

print*,'^^^^^^^^^vpat:',islab,id1,id2,jd1,jd2,numsl,np

if(islab == 1) then

   do k=jd1,jd2
      do i=id1,id2
         do ip=1,np
            outarea(i-id1+1,k-jd1+1,ip)=inarea(i,numsl,ip)
            outdata(i-id1+1,k-jd1+1,ip)=indata(i,numsl,k,ip)
         enddo
      enddo
   enddo

elseif(islab == 2) then

   do k=jd1,jd2
      do j=id1,id2
         do ip=1,np
            outarea(j-id1+1,k-jd1+1,ip)=inarea(numsl,j,ip)
            outdata(j-id1+1,k-jd1+1,ip)=indata(numsl,j,k,ip)
         enddo
      enddo
   enddo

elseif(islab == 3) then

   do j=jd1,jd2
      do i=id1,id2
         do ip=1,np
            outarea(i-id1+1,j-jd1+1,ip)=inarea(i,j,ip)
            outdata(i-id1+1,j-jd1+1,ip)=indata(i,j,numsl,ip)
         enddo
      enddo
   enddo

endif

return
end

!***************************************************************************

subroutine RAMS_3to2d_hpat (indata,inarea,ni1,ni2,np  &
                           ,outdata,outarea,id1,id2,jd1,jd2,nii,njj)
                         
implicit none
                           
integer :: islab ,ni1,ni2,id1,id2,jd1,jd2,nii,njj,np
real :: indata(ni1,ni2,np),inarea(ni1,ni2,np)
real outdata(nii,njj,np),outarea(nii,njj,np)

integer :: i,j,ip

do j=jd1,jd2
   do i=id1,id2
      do ip = 1,np
         outarea(i-id1+1,j-jd1+1,ip)=inarea(i,j,ip)
         outdata(i-id1+1,j-jd1+1,ip)=indata(i,j,ip)
      enddo
   enddo
enddo

return
end


!***************************************************************************

subroutine RAMS_Ptrans (n1,n2,n3,a,pi,b)

use pl_params

dimension a(n1,n2,n3),pi(n1,n2,n3),b(n3,*)

do np=1,nplevs
   b(nplevs-np+1,4)=1004.*(float(iplevs(np))/1000.)**.286
enddo

do j=1,n2
   do i=1,n1
      do k=1,n3
         kk=n3-k+1
         b(kk,1)=a(i,j,k)
         b(kk,2)=pi(i,j,k)
      enddo
      call htint(n3,b(1,1),b(1,2),nplevs,b(1,3),b(1,4))
      do k=1,nplevs
         a(i,j,nplevs-k+1)=b(k,3)
      enddo
   enddo
enddo

return
end

!***************************************************************************

subroutine RAMS_Ctrans (n1,n2,n3,a,b,topo,zt,ztop)

dimension a(n1,n2,n3),b(n3,*),topo(n1,n2),zt(n3)

do j=1,n2
   do i=1,n1
      do k=1,n3
         b(k,1)=a(i,j,k)
         b(k,2)=topo(i,j)+zt(k)*(1.-topo(i,j)/ztop)
      enddo
      call htint(n3,b(1,1),b(1,2),n3,b(1,3),zt)
      do k=1,n3
         a(i,j,k)=b(k,3)
      enddo
   enddo
enddo

return
end

!***************************************************************************

integer function ichvar_changed (cvar,ch)

character*(*) cvar,ch
character*12 ctime,chvar,cgrid,cwinds,ocvar
character*80 cfile
save
data cfile/' '/,ctime/' '/,chvar/' '/,cgrid/' '/  &
    ,cwinds/' '/,ocvar/' '/

ichvar_changed=0
if(cvar == 'cfile') then
   if(ch /= cfile) then
      ichvar_changed=1
      cfile=ch
   endif
elseif(cvar == 'ctime') then
   if(ch /= ctime) then
      ichvar_changed=1
      ctime=ch
   endif
elseif(cvar == 'cvar') then
   if(ch /= chvar) then
      ichvar_changed=1
      chvar=ch
   endif
elseif(cvar == 'ocvar') then
   if(ch /= ocvar) then
      ichvar_changed=1
      ocvar=ch
   endif
elseif(cvar == 'cgrid') then
   if(ch /= cgrid) then
      ichvar_changed=1
      cgrid=ch
   endif
elseif(cvar == 'cwinds') then
   if(ch /= cwinds) then
      ichvar_changed=1
      cwinds=ch
   endif
endif

return
end

!***************************************************************************

integer function intvar_changed (cvar,ival)

character*(*) cvar
save
data iwoffl,iwoffr,iwoffb,iwofft,itrans,icoor,iplotinfo/0,0,0,0,0,0,0/

intvar_changed=0
if(cvar == 'iwoffl') then
   if(ival /= iwoffl) then
      intvar_changed=1
      iwoffl=ival
   endif
elseif(cvar == 'iwoffr') then
   if(ival /= iwoffr) then
      intvar_changed=1
      iwoffr=ival
   endif
elseif(cvar == 'iwoffb') then
   if(ival /= iwoffb) then
      intvar_changed=1
      iwoffb=ival
   endif
elseif(cvar == 'iwofft') then
   if(ival /= iwofft) then
      intvar_changed=1
      iwofft=ival
   endif
elseif(cvar == 'itrans') then
   if(ival /= itrans) then
      intvar_changed=1
      itrans=ival
   endif
elseif(cvar == 'iplotinfo') then
   if(ival /= iplotinfo) then
      intvar_changed=1
      iplotinfo=ival
   endif
elseif(cvar == 'icoor') then
   if(ival /= icoor) then
      intvar_changed=1
      icoor=ival
   endif
endif

return
end
!***************************************************************************

subroutine max_min( a,n,amx,amn)

dimension a(*)

amx=a(1)
amn=a(1)
do i=2,n
   amx=max(amx,a(i))
   amn=min(amn,a(i))
enddo

return
end

!***************************************************************************

subroutine thelatlon (n1,n2,n3,alatlon,all1,i1,j1)

dimension alatlon(n1,n2,n3)

all1=alatlon(i1,j1,1)

return
end

