!############################# Change Log ##################################
! 1.0.0
!
!###########################################################################

Module ra_file_info


   type fileinfo
      
   real, pointer ::     fxt(:,:),fyt(:,:)  &
                       ,fxm(:,:),fym(:,:)  &
                       ,fzt(:,:),fzm(:,:)  &
                       ,fdelx(:),fdely(:)  &
                       ,polelatn(:),polelonn(:)  &
                       ,swx(:),swy(:),fslz(:)
   real :: startutc,httop,ftime
   integer, pointer :: nnxp(:),nnyp(:),nnzp(:)
   logical, pointer :: exist(:)
   integer :: nfiles,ihtranr,if_adapr,nfgrids,ifdate,iftime,nzg,nzs,npatch
   character(len=256) :: fname
   character(len=15)  :: startdate
   character(len=64)  :: exptname
   end type
   
   type file
      type(fileinfo),pointer :: finfo(:)   
      integer :: nfl    ! current file number
   end type
   
   integer :: nset=1    ! These are defaulted to set number 1 awaiting multiple 
   type(file) :: finf(1) !   model development

Contains

!----------------------------------------------------------

subroutine ra_set_file(ns,nf)

implicit none
integer :: ns,nf

nset=ns
finf(nset)%nfl=nf

return
end subroutine

!----------------------------------------------------------

subroutine ra_init (nfile,file_prefix)

use an_header

implicit none

integer :: nfile,nopt,nfl,ngr,nval
character(len=*) :: file_prefix

integer, parameter :: maxg_r=20
integer :: ngrids_r,nzg_r,nzs_r,npatch_r  &
          ,nnxp_r(maxg_r),nnyp_r(maxg_r),nnzp_r(maxg_r)

integer :: nv,ng,k,nfn,ln,ihr1,imin1,iitime,iiday,iimonth,iiyear  &
          ,i,j,itime1,idate1,imonth1,iyear1,iun,ie, maxnxp,maxnyp,maxnzp
real :: val
integer, external :: cio_i, cio_f, cio_c
character(len=2)   :: cng
character(len=4)   :: ctime1
character(len=256) :: fpref,fnames(2000)

iun=10

! get the files
print*,'-----------:',nset,finf(nset)%nfl
fpref=file_prefix
fpref(len_trim(fpref)+1:)='*-head.txt'
call RAMS_filelist(fnames,fpref,nfile)
print*,' files found: ',nfile
print*

allocate(finf(nset)%finfo(nfile))
finf(nset)%finfo(1:nfile)%nfiles=nfile

! construct arrays
do nfn=1,nfile
   finf(nset)%finfo(nfn)%fname=fnames(nfn)
   
   ! Read variable tables.
   call rams_read_header(flnm=finf(nset)%finfo(nfn)%fname  &
                        ,action='read')
   
   ! Open header file and get dimension information
   open(iun, file=trim(finf(nset)%finfo(nfn)%fname))
   ie=cio_i(iun,1,'if_adap',finf(nset)%finfo(nfn)%if_adapr,nval)
   ie=cio_i(iun,1,'ngrids',ngrids_r,nval)
   ie=cio_i(iun,1,'nzg',nzg_r,nval)
   ie=cio_i(iun,1,'nzs',nzs_r,nval)
   ie=cio_i(iun,1,'nnxp',nnxp_r,nval)
   ie=cio_i(iun,1,'nnyp',nnyp_r,nval)
   ie=cio_i(iun,1,'nnzp',nnzp_r,nval)
   ie=cio_i(iun,1,'npatch',npatch_r,nval)
   
   ! Find max dimensions
   maxnxp=maxval(nnxp_r(1:ngrids_r))
   maxnyp=maxval(nnyp_r(1:ngrids_r))
   maxnzp=max(nzg_r,nzs_r,maxval(nnzp_r(1:ngrids_r)))
   
   ! Allocate info arrays
   
   allocate(finf(nset)%finfo(nfn)%fxt(maxnxp,ngrids_r)  &
           ,finf(nset)%finfo(nfn)%fxm(maxnxp,ngrids_r)  &
           ,finf(nset)%finfo(nfn)%fyt(maxnyp,ngrids_r)  &
           ,finf(nset)%finfo(nfn)%fym(maxnyp,ngrids_r)  &
           ,finf(nset)%finfo(nfn)%fzt(maxnzp,ngrids_r)  &
           ,finf(nset)%finfo(nfn)%fzm(maxnzp,ngrids_r)  &
           ,finf(nset)%finfo(nfn)%fdelx(ngrids_r)  &
           ,finf(nset)%finfo(nfn)%fdely(ngrids_r) &
           ,finf(nset)%finfo(nfn)%polelatn(ngrids_r)  &
           ,finf(nset)%finfo(nfn)%polelonn(ngrids_r)  &
           ,finf(nset)%finfo(nfn)%swx(ngrids_r)  &
           ,finf(nset)%finfo(nfn)%swy(ngrids_r)  &
           ,finf(nset)%finfo(nfn)%fslz(maxnzp) &
           ,finf(nset)%finfo(nfn)%nnxp(ngrids_r)  &
           ,finf(nset)%finfo(nfn)%nnyp(ngrids_r)  &
           ,finf(nset)%finfo(nfn)%nnzp(ngrids_r)  &
           ,finf(nset)%finfo(nfn)%exist(ngrids_r))
   
   
   ! Populate module storage
   ie=cio_c(iun,1,'expnme',finf(nset)%finfo(nfn)%exptname,nval)
   finf(nset)%finfo(nfn)%nfgrids  =  ngrids_r
   finf(nset)%finfo(nfn)%nzg      =  nzg_r
   finf(nset)%finfo(nfn)%nzs      =  nzs_r
   finf(nset)%finfo(nfn)%npatch   =  npatch_r
   finf(nset)%finfo(nfn)%nnxp(1:ngrids_r) =  nnxp_r(1:ngrids_r)
   finf(nset)%finfo(nfn)%nnyp(1:ngrids_r) =  nnyp_r(1:ngrids_r)
   finf(nset)%finfo(nfn)%nnzp(1:ngrids_r) =  nnzp_r(1:ngrids_r)
   ie=cio_f(iun,1,'deltaxn',finf(nset)%finfo(nfn)%fdelx(1),nval)
   ie=cio_f(iun,1,'deltayn',finf(nset)%finfo(nfn)%fdely(1),nval)
   ie=cio_f(iun,1,'platn',finf(nset)%finfo(nfn)%polelatn(1),nval)
   ie=cio_f(iun,1,'plonn',finf(nset)%finfo(nfn)%polelonn(1),nval)
   do ng=1,ngrids_r
      write(cng,'(i2.2)') ng
      ie=cio_f(iun,1,'xmn'//cng,finf(nset)%finfo(nfn)%fxm(1,ng),nval)
      ie=cio_f(iun,1,'xtn'//cng,finf(nset)%finfo(nfn)%fxt(1,ng),nval)
      ie=cio_f(iun,1,'ymn'//cng,finf(nset)%finfo(nfn)%fym(1,ng),nval)
      ie=cio_f(iun,1,'ytn'//cng,finf(nset)%finfo(nfn)%fyt(1,ng),nval)
      ie=cio_f(iun,1,'zmn'//cng,finf(nset)%finfo(nfn)%fzm(1,ng),nval)
      ie=cio_f(iun,1,'ztn'//cng,finf(nset)%finfo(nfn)%fzt(1,ng),nval)
      finf(nset)%finfo(nfn)%swx(ng)=finf(nset)%finfo(nfn)%fxt(1,ng)
      finf(nset)%finfo(nfn)%swy(ng)=finf(nset)%finfo(nfn)%fyt(1,ng)
   enddo
   finf(nset)%finfo(nfn)%httop=  &
         finf(nset)%finfo(nfn)%fzm(finf(nset)%finfo(nfn)%nnzp(1)-1,1)
   ie=cio_i(iun,1,'ihtran',finf(nset)%finfo(nfn)%ihtranr,nval)
   
   ! Find starting date/time
   ie=cio_i(iun,1,'iyear1',iyear1,1)
   ie=cio_i(iun,1,'imonth1',imonth1,1)
   ie=cio_i(iun,1,'idate1',idate1,1)
   ie=cio_i(iun,1,'itime1',itime1,1)
   
   if(nfn==1) then
      write(ctime1,'(i4.4)') itime1
      read(ctime1(1:2),*) ihr1
      read(ctime1(3:4),*) imin1
      finf(nset)%finfo(nfn)%startutc=float(ihr1)+float(imin1)/60.
      write(finf(nset)%finfo(nfn)%startdate,'(i4.4,a1,i2.2,a1,i2.2,a1,i4.4)')  &
          iyear1,'-',imonth1,'-',idate1,'-',itime1
      !print*,'RAMS_anal_init: start utc, date ',startutc,' ',startdate
   endif
   
   ln=len_trim(finf(nset)%finfo(nfn)%fname)
   read(fnames(nfn)(ln-14:ln-9),*) iitime
   read(fnames(nfn)(ln-17:ln-16),*) iiday
   read(fnames(nfn)(ln-20:ln-19),*) iimonth
   read(fnames(nfn)(ln-25:ln-22),*) iiyear
   ie=cio_f(iun,1,'time',val,1)
   finf(nset)%finfo(nfn)%ftime=val
   finf(nset)%finfo(nfn)%ifdate=iiyear*10000+iimonth*100+iiday
   finf(nset)%finfo(nfn)%iftime=iitime 
      
   ! Also, see how many grids exist at this time and set "exist" flag
   finf(nset)%finfo(nfn)%exist(1:ngrids_r)=.false.
   call rams_read_header(flnm=finf(nset)%finfo(nfn)%fname  &
                        ,action='grid_inv'  &
                        ,ldata=finf(nset)%finfo(nfn)%exist(1:ngrids_r))
   print*,'exist:',finf(nset)%finfo(nfn)%exist(1:ngrids_r)


   write(*,'(a,i4,f12.2,2i10)') ' files-',nfn,finf(nset)%finfo(nfn)%ftime  &
                                         ,finf(nset)%finfo(nfn)%ifdate  &
                                         ,finf(nset)%finfo(nfn)%iftime


!   ie=cio_f(iun,1,'slz',finf(nset)%finfo(nfn)%fslz(1),nval)


   close(iun)
enddo

return
end subroutine


end Module
