module pscript

type local_var
   character(len=64) :: name
   character(len=64) :: value
end type

type (local_var) :: loc_var(200)
integer :: nloc_var

character(len=256) :: slines(1000)
integer :: nslines, cur_line

integer, parameter :: maxloops=10

character(len=16), dimension(maxloops) :: lp_var,lp_init,lp_max,lp_inc
integer, dimension(maxloops) :: lp_beg_line, lp_app
integer :: cur_loop


contains

!-----------------------------------------------------------------------

subroutine proc_script(sfile)

implicit none

character(len=*) :: sfile

character(len=256) :: line,tokens(10)

integer :: ntok,nt,ns

open (10,file=sfile, status='old')


! First go through entire script, separating lines, and store in memory

nslines=0

do while(.true.)

read(10, '(a256)',end=100, err=100) line

call stokenize(line,tokens,ntok,';')

do nt=1,ntok
   nslines = nslines + 1
   call lead_deblank(tokens(nt),slines(nslines))
   print*,'lllll:',trim(slines(nslines))
enddo

enddo

100 continue

! Start main loop

cur_loop=0
cur_line=1

do while (cur_line <= nslines) 

   !print*,'exec:',cur_line
   ns = cur_line

   if(slines(ns)(1:7) == 'set loc') then
      call set_locvar(slines(ns))

   elseif(slines(ns)(1:7) == 'set app') then
      call set_appvar(slines(ns))

   elseif(slines(ns)(1:4) == 'loop') then
      call proc_loop(slines(ns))

   elseif(slines(ns)(1:7) == 'endloop') then
      call proc_endloop()

   elseif(slines(ns)(1:4) == 'exec') then
      call proc_exec(slines(ns))

   elseif(slines(ns)(1:7) == 'include') then
      call proc_include(slines(ns))
      
   elseif(slines(ns)(1:1) /= '#') then
      print*,'Illegal script command:',trim(slines(ns))
      print*,' Allowable commands: set loc, set app, loop, endloop, exec'
      stop 'proc_script'

   endif

   cur_line = cur_line + 1

enddo



do nt=1,nslines
print*,nt,trim(slines(nt))
enddo
do nt=1,nloc_var
print*,nt,trim(loc_var(nt)%name),' ',trim(loc_var(nt)%value)
enddo

return
end subroutine
!-----------------------------------------------------------------------

subroutine proc_loop(line)

implicit none

character(len=*) :: line

character(len=64) :: tokens(10)
integer :: ntok,nvv

call stokenize(line,tokens,ntok,' ')

cur_loop = cur_loop + 1

lp_beg_line(cur_loop) = cur_line

lp_var(cur_loop)=tokens(3)

! Set up loop index var as a local variable
call get_locvar_id(tokens(3),nvv)
! If = 0, var doesn't exist, so add it.
if (nvv == 0) then
   nloc_var = nloc_var+1
   nvv = nloc_var
   loc_var(nvv)%name = tokens(3)
endif

! Define initial value
if(tokens(4)(1:1) == '$') then
   call get_locvar_val(tokens(4)(2:),loc_var(nvv)%value)
else
   loc_var(nvv)%value = tokens(4)
endif
lp_init(cur_loop)=loc_var(nvv)%value

! If looping on an "app" variable, we will set it up 
!    as a local variable first (above), then set the app value here.
lp_app(cur_loop)=1
if(tokens(2)(1:3) == 'app') then
   lp_app(cur_loop)=1
   call set_appvar_value(tokens(3),tokens(4))
endif

! Store max and increment
if(tokens(5)(1:1) == '$') then
   call get_locvar_val(tokens(5)(2:),lp_max(cur_loop))
else
   lp_max(cur_loop) = tokens(5)
endif

if(ntok >= 6) then
   if(tokens(6)(1:1) == '$') then
      call get_locvar_val(tokens(6)(2:),lp_inc(cur_loop))
   else
      lp_inc(cur_loop) = tokens(6)
   endif
else
   lp_inc(cur_loop) = '1'
endif


return
end subroutine


!-----------------------------------------------------------------------

subroutine proc_endloop()

! We have reached the end of a loop. We need to increment the index variable,
!    check if we have another iteration, and end if we don't.

implicit none

logical :: ismax
integer :: nvv



call get_locvar_id(lp_var(cur_loop),nvv)
call oper_add(loc_var(nvv)%value,loc_var(nvv)%value,lp_inc(cur_loop))

call oper_max(ismax,loc_var(nvv)%value,lp_max(cur_loop))

if( ismax ) then
   ! Loop is finished
   cur_loop = cur_loop - 1 
else
   ! Not done...set current line to do beginning line
   cur_line = lp_beg_line(cur_loop)

   ! If looping on an app var, set app value the same as local value
   if(lp_app(cur_loop) == 1) then
      call set_appvar_value(lp_var(cur_loop),loc_var(nvv)%value)
   endif

endif

!print*,'end:',cur_loop,cur_line

return
end subroutine

!-----------------------------------------------------------------------

subroutine proc_exec(line)

! A type of execute action has been requested. Parse the line and pass the action
!   to a user routine.

implicit none

character(len=*) :: line

character(len=64) :: tokens(10)

integer :: ntok,nv,nvv

! Initialize tokens to blank
tokens(1:10)=' '

! Separate the line into tokens
call stokenize(line,tokens,ntok,' ')

! Just do it
call exec_action(tokens)

return
end subroutine

!-----------------------------------------------------------------------

subroutine oper_add(val1,val2,val3)

! Add val2+val3, define into val1

implicit none

character(len=*) :: val1,val2,val3

integer :: int1,int2,int3
character(len=32) :: vals

! Check and see is all the same type
!!!!! ASSUME ALL INT FOR NOW !!!!!!

read(val2,*) int2
read(val3,*) int3

write(vals,*) int2+int3

call lead_deblank(vals,val1)

return
end subroutine


!-----------------------------------------------------------------------

subroutine oper_max(ismax,val1,val2)

! Return ismax=.T. if val1 > val2

implicit none

logical :: ismax
character(len=*) :: val1,val2

integer :: int1,int2

! Check and see is all the same type
!!!!! ASSUME ALL INT FOR NOW !!!!!!

read(val1,*) int1
read(val2,*) int2

ismax=.F.
if(int1 > int2) ismax = .T.

return
end subroutine

!-----------------------------------------------------------------------

subroutine proc_include(sline)

implicit none

character(len=*) :: sline

character(len=256) :: line,tokens(10),ifile
character(len=256) :: ilines(1000)
integer :: nilines,cur_iline

integer :: ntok,nt,ns,nc
logical :: there

! Process the script include file. Yhis will function much like the main
!   script processor, but we will only allow set commands for now.


! Extract include file name and check for existence
call stokenize(sline,tokens,ntok,' ')
ifile=tokens(2)
inquire(file=ifile, exist=there)

if( .not. there) then
   print*,'Include file not found.'
   print*,'Filename: ',trim(ifile)
   stop 'proc_include: no file'
endif


open (10,file=ifile, status='old')

! First go through entire file, separating lines, and store in memory

nilines=0

10 continue

read(10, '(a128)',end=100, err=100) line

call stokenize(line,tokens,ntok,';')

do nt=1,ntok
   nilines = nilines + 1
   call lead_deblank(tokens(nt),ilines(nilines))
enddo

goto 10

100 continue

! Start main loop

cur_iline=1

do while (cur_iline <= nilines) 

   !print*,'include:',cur_iline,trim(ilines(ns))
   ns = cur_iline

   if(ilines(ns)(1:7) == 'set loc') then
      call set_locvar(ilines(ns))

   elseif(ilines(ns)(1:7) == 'set app') then
      call set_appvar(ilines(ns))
      
   elseif(ilines(ns)(1:1) /= '#') then
      print*,'Illegal include file command:',trim(ilines(ns))
      print*,' Allowable commands: set loc, set app'
      stop 'proc_include'

   endif

   cur_iline = cur_iline + 1

enddo


return
end subroutine

!-----------------------------------------------------------------------

subroutine set_locvar(line)

implicit none

character(len=*) :: line

character(len=256) :: tokens(10)

integer :: ntok,nv,nvv

call stokenize(line,tokens,ntok,' ')

!print*,ntok
!do nv=1,ntok
!print*,nv,tokens(nv)
!enddo

call get_locvar_id(tokens(3),nvv)

! If = 0, var doesn't exist, so add it.
if (nvv == 0) then
   nloc_var=nloc_var+1
   nvv = nloc_var
endif

loc_var(nvv)%name = tokens(3)

! If value starts with a $, then it is a local var setting

if(tokens(4)(1:1) == '$') then
   call get_locvar_val(tokens(4)(2:),loc_var(nvv)%value)
else
   loc_var(nvv)%value = tokens(4)
endif

return
end subroutine


!-----------------------------------------------------------------------

subroutine set_appvar(line)

implicit none

character(len=*) :: line

character(len=256) :: tokens(10),value

integer :: ntok,nv,nvv

call stokenize(line,tokens,ntok,' ')

! If value starts with a $, then it is a local var setting
if(tokens(4)(1:1) == '$') then
   call get_locvar_val(tokens(4)(2:),value)
else
   value = tokens(4)
endif

call set_appvar_value(tokens(3),value)

return
end subroutine


!-----------------------------------------------------------------------

subroutine get_locvar_id(name,id)

! Get the integer id of a local variable

implicit none

character(len=*) :: name
integer :: id

integer :: nv

id=0
do nv=1,nloc_var
   if(trim(name) == trim(loc_var(nv)%name) ) then
      id = nv
      return
   endif
enddo

return
end subroutine

!-----------------------------------------------------------------------

subroutine get_locvar_val(name,value)

! Get the value of a local variable

implicit none

character(len=*) :: name,value

integer :: nv
do nv=1,nloc_var
   if(trim(name) == trim(loc_var(nv)%name) ) then
      value = loc_var(nv)%value
      return
   endif
enddo

call serror_mess('Variable '//trim(name)//' undefined')
stop 'local var undefined'

return
end subroutine

end module
